@@ -22,7 +22,9 @@ import java.io.Serializable;
 import java.util.Date;
 
 import org.apache.openjpa.jdbc.meta.ClassMapping;
-import org.apache.openjpa.jdbc.meta.JavaSQLTypes;
+import org.apache.openjpa.jdbc.meta.Discriminator;
+import org.apache.openjpa.jdbc.meta.strats.NoneDiscriminatorStrategy;
+import org.apache.openjpa.jdbc.meta.strats.VerticalClassStrategy;
 import org.apache.openjpa.jdbc.sql.DBDictionary;
 import org.apache.openjpa.jdbc.sql.Raw;
 import org.apache.openjpa.kernel.exps.AggregateListener;
@@ -56,6 +58,8 @@ public class JDBCExpressionFactory
     private final ClassMapping _type;
     private final SelectConstructor _cons = new SelectConstructor();
     private int _getMapValueAlias = 0;
+    
+    private boolean _isBooleanLiteralAsNumeric = true;
 
     /**
      * Constructor. Supply the type we're querying against.
@@ -64,6 +68,10 @@ public class JDBCExpressionFactory
         _type = type;
     }
 
+    public void setBooleanLiteralAsNumeric(boolean isBooleanLiteralAsNumeric) {
+        _isBooleanLiteralAsNumeric = isBooleanLiteralAsNumeric;
+    }
+    
     /**
      * Use to create SQL select.
      */
@@ -87,10 +95,52 @@ public class JDBCExpressionFactory
             return contains(v1, v2);
         if (v2 instanceof PCPath && ((PCPath) v2).isUnaccessedVariable())
             return contains(v2, v1);
+        if (v1 instanceof Type || v2 instanceof Type) {
+            Value val = v1 instanceof Type ? v1 : v2;
+            verifyTypeOperation(val, null, false);
+            return new EqualTypeExpression((Val) v1, (Val) v2);
+        }
         return new EqualExpression((Val) v1, (Val) v2);
     }
 
+    private void verifyTypeOperation(Value val, Value param, boolean isNotEqual) {
+        if (val.getPath() == null)
+            return;
+        PCPath path = (PCPath) val.getPath();
+        Discriminator disc = ((Type) val).getDiscriminator();
+        if (disc == null || !(val.getMetaData().getPCSuperclass() != null ||
+            val.getMetaData().getPCSubclasses().length > 0))
+            throw new UserException(_loc.
+                get("invalid-type-argument", path.last() != null ? path.getPCPathString() : path.getSchemaAlias()));
+        
+        if (disc.getColumns().length == 0) {
+            if (disc.getStrategy() instanceof NoneDiscriminatorStrategy) {
+                // limited support for table per class inheritance hierarchy
+                if (path.last() != null)
+                    throw new UserException(_loc.
+                        get("type-argument-unsupported", path.last().getName())); 
+                if (isNotEqual) {
+                    if (param != null && param instanceof Null)
+                        throw new UserException(_loc.
+                            get("type-in-expression-unsupported", path.getSchemaAlias()));
+                    else
+                        throw new UserException(_loc.
+                            get("type-not-equal-unsupported", path.getSchemaAlias()));
+                }
+            }
+            if (param != null && param instanceof CollectionParam)
+                throw new UserException(_loc.
+                    get("collection-param-unsupported")); 
+        }
+    }
+
     public Expression notEqual(Value v1, Value v2) {
+        if (v1 instanceof Type || v2 instanceof Type) {
+            Value val = v1 instanceof Type ? v1 : v2;
+            Value param = val == v1 ? (v2 instanceof Null ? v2 : null) : (v1 instanceof Null ? v1 : null);
+            verifyTypeOperation(val, param, true);
+            return new NotEqualTypeExpression((Val) v1, (Val) v2);
+        }
         return new NotEqualExpression((Val) v1, (Val) v2);
     }
 
@@ -123,8 +173,16 @@ public class JDBCExpressionFactory
     }
 
     public Expression contains(Value map, Value arg) {
-        if (map instanceof Const)
+        if (map instanceof Const) {
+            if (arg instanceof Type) {
+                // limited support for table per class inheritance
+                verifyTypeOperation(arg, map, false);
+                if (((ClassMapping) arg.getMetaData()).getDiscriminator().getColumns().length == 0)
+                    return new EqualTypeExpression((Val) arg, (Val) map);
+            }
+
             return new InExpression((Val) arg, (Const) map);
+        }
         if (map instanceof SubQ)
             return new InSubQExpression((Val) arg, (SubQ) map);
         return new ContainsExpression((Val) map, (Val) arg);
@@ -453,10 +511,11 @@ public class JDBCExpressionFactory
                 pType == Literal.TYPE_STRING)
                 value.append("'").append(lit.getValue().toString()).append("'");
             else if (pType == Literal.TYPE_BOOLEAN) {
-                if ((Boolean) lit.getValue())
-                    value.append("1");
+                Boolean boolVal = (Boolean)lit.getValue();
+                if (_isBooleanLiteralAsNumeric)
+                    value.append(boolVal ? "1" : "0");
                 else
-                    value.append("0");
+                    value.append(boolVal ? "true" : "false");
             } else if (pType == Literal.TYPE_ENUM) {
                 lit.setRaw(true);
                 return val;
@@ -515,4 +574,14 @@ public class JDBCExpressionFactory
     public Value newFunction(String functionName, Class<?> resultType, Value... args) {
         return new DatastoreFunction(functionName, resultType, newArgumentList(args));
     }
+    
+    public boolean isVerticalType(Value val) {
+        if (!(val instanceof Type))
+            return false;
+        ClassMapping cm = (ClassMapping)((Type)val).getMetaData();
+        String strat = cm.getMappingInfo().getHierarchyStrategy(); 
+        if (strat != null && strat.equals(VerticalClassStrategy.ALIAS)) 
+            return true;
+        return false;
+    }
 }
