@@ -1049,7 +1049,7 @@ public class PCPath
         else
             cols = getColumns(state);
 
-        if (cols == null) {
+        if (cols == null || cols.length == 0) {
             sql.append("1");
             return;
         }
@@ -1147,6 +1147,8 @@ public class PCPath
         }
 
         public boolean equals(Object other) {
+            if (other == null)
+                return false;
             if (other == this)
                 return true;
             Action a = (Action) other;
