@@ -1349,8 +1349,7 @@ public abstract class MappingInfo
         boolean adapt, boolean fill) {
         // default to the primary key column name if this is a pk join
         DBIdentifier name = given.getIdentifier();
-        if (DBIdentifier.isNull(name) && given != null
-            && given.getFlag(Column.FLAG_PK_JOIN) && cls != null) {
+        if (DBIdentifier.isNull(name) && given.getFlag(Column.FLAG_PK_JOIN) && cls != null) {
             Column[] pks = cls.getPrimaryKeyColumns();
             if (pks.length == 1)
                 name = pks[0].getIdentifier();
@@ -1529,7 +1528,7 @@ public abstract class MappingInfo
         Column tmplate = new Column();
         tmplate.setIdentifier(name);
         if (!constant) {
-            Column tcol = foreign.getColumn(targetName);
+            Column tcol = foreign.getColumn(targetName, false); 
             if (tcol == null)
                 throw new MetaDataException(_loc.get(prefix + "-bad-fktarget",
                     new Object[]{ context, targetName, name, foreign }));
