@@ -337,11 +337,7 @@ public class RelationFieldStrategy
     private Object getMapKeyObj(Map mapObj, Object value) {
         if (value instanceof ReflectingPersistenceCapable)
             value = ((ReflectingPersistenceCapable)value).getManagedInstance(); 
-        Set keySet = mapObj.keySet();
-        for (Object key : keySet) {
-            if (mapObj.get(key) == value)
-                return key;
-        }
+
         Set<Map.Entry> entries = mapObj.entrySet();
         for (Map.Entry entry : entries) {
             if (entry.getValue() == value)
@@ -408,8 +404,12 @@ public class RelationFieldStrategy
 
             // if our foreign key has a delete action, we need to set the
             // related object so constraints can be evaluated
+            Object lastRelPc = sm.fetchObjectField(field.getIndex());
+            if( lastRelPc == null) {
+            	lastRelPc = sm.fetchInitialField(field.getIndex());
+            }
             OpenJPAStateManager rel = RelationStrategies.getStateManager
-                (sm.fetchObjectField(field.getIndex()), store.getContext());
+                (lastRelPc, store.getContext());
             if (rel != null) {
                 ForeignKey fk = field.getForeignKey((ClassMapping)
                     rel.getMetaData());
