@@ -25,12 +25,14 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import org.apache.openjpa.enhance.FieldManager;
 import org.apache.openjpa.enhance.PersistenceCapable;
 import org.apache.openjpa.jdbc.kernel.JDBCFetchConfiguration;
 import org.apache.openjpa.jdbc.kernel.JDBCStore;
 import org.apache.openjpa.jdbc.meta.ClassMapping;
 import org.apache.openjpa.jdbc.meta.FieldMapping;
 import org.apache.openjpa.jdbc.meta.FieldStrategy;
+import org.apache.openjpa.jdbc.meta.RelationId;
 import org.apache.openjpa.jdbc.meta.ValueMapping;
 import org.apache.openjpa.jdbc.schema.Column;
 import org.apache.openjpa.jdbc.schema.ForeignKey;
@@ -41,12 +43,13 @@ import org.apache.openjpa.jdbc.sql.SelectExecutor;
 import org.apache.openjpa.jdbc.sql.Union;
 import org.apache.openjpa.kernel.OpenJPAStateManager;
 import org.apache.openjpa.kernel.StateManagerImpl;
-import org.apache.openjpa.lib.util.Localizer;
 import org.apache.openjpa.meta.ClassMetaData;
 import org.apache.openjpa.meta.JavaTypes;
 import org.apache.openjpa.util.ChangeTracker;
 import org.apache.openjpa.util.Id;
+import org.apache.openjpa.util.OpenJPAId;
 import org.apache.openjpa.util.Proxy;
+import org.apache.openjpa.util.DelayedProxy;
 
 /**
  * Base class for strategies that are stored as a collection, even if
@@ -121,7 +124,7 @@ public abstract class StoreCollectionFieldStrategy
      * By default, assumes the structure is a collection.
      */
     protected void add(JDBCStore store, Object coll, Object obj) {
-        ((Collection) coll).add(obj);
+        ((Collection<Object>) coll).add(obj);
     }
 
     /**
@@ -489,8 +492,21 @@ public abstract class StoreCollectionFieldStrategy
     public void load(final OpenJPAStateManager sm, final JDBCStore store,
         final JDBCFetchConfiguration fetch)
         throws SQLException {
+        
+        Object coll = null;
+        final int fieldIndex = field.getIndex();
+        final boolean delayed = sm.isDelayed(fieldIndex);
+        if (!delayed && field.isDelayCapable()) {
+            coll = sm.newProxy(fieldIndex);
+            if (coll instanceof DelayedProxy) {
+                sm.storeObject(fieldIndex, coll);
+                sm.setDelayed(fieldIndex, true);
+                return;
+            }
+        }
+        
         if (field.isLRS()) {
-            Proxy coll = newLRSProxy();
+            Proxy pcoll = newLRSProxy();
 
             // if this is ordered we need to know the next seq to use in case
             // objects are added to the collection
@@ -511,13 +527,13 @@ public abstract class StoreCollectionFieldStrategy
                 Result res = sel.execute(store, fetch);
                 try {
                     res.next();
-                    coll.getChangeTracker().setNextSequence
+                    pcoll.getChangeTracker().setNextSequence
                         (res.getInt(field) + 1);
                 } finally {
                     res.close();
                 }
             }
-            sm.storeObjectField(field.getIndex(), coll);
+            sm.storeObjectField(fieldIndex, pcoll);
             return;
         }
 
@@ -535,14 +551,27 @@ public abstract class StoreCollectionFieldStrategy
         });
 
         // create proxy
-        Object coll;
         ChangeTracker ct = null;
-        if (field.getTypeCode() == JavaTypes.ARRAY)
-            coll = new ArrayList();
-        else {
-            coll = sm.newProxy(field.getIndex());
+        if (delayed) {
+            if (sm.isDetached() || sm.getOwner() == null) {
+                sm.getPersistenceCapable().pcProvideField(fieldIndex);
+                coll = 
+                    ((FieldManager)sm.getPersistenceCapable().pcGetStateManager()).fetchObjectField(fieldIndex);
+            } else {
+                coll = sm.fetchObjectField(fieldIndex);
+            }
             if (coll instanceof Proxy)
                 ct = ((Proxy) coll).getChangeTracker();
+        } else {
+            if (field.getTypeCode() == JavaTypes.ARRAY)
+                coll = new ArrayList();
+            else {
+                if (coll == null) {
+                    coll = sm.newProxy(fieldIndex);
+                }
+                if (coll instanceof Proxy)
+                    ct = ((Proxy) coll).getChangeTracker();
+            }
         }
 
         // load values
@@ -562,12 +591,14 @@ public abstract class StoreCollectionFieldStrategy
             res.close();
         }
 
-        // set into sm
-        if (field.getTypeCode() == JavaTypes.ARRAY)
-            sm.storeObject(field.getIndex(), JavaTypes.toArray
-                ((Collection) coll, field.getElement().getType()));
-        else
-            sm.storeObject(field.getIndex(), coll);
+        // if not a delayed collection, set into sm
+        if (!delayed) {
+            if (field.getTypeCode() == JavaTypes.ARRAY)
+                sm.storeObject(fieldIndex, JavaTypes.toArray
+                    ((Collection) coll, field.getElement().getType()));
+            else
+                sm.storeObject(fieldIndex, coll);
+        }
     }
 
     /**
@@ -576,8 +607,9 @@ public abstract class StoreCollectionFieldStrategy
     protected Joins selectAll(Select sel, ClassMapping elem,
         OpenJPAStateManager sm, JDBCStore store, JDBCFetchConfiguration fetch,
         int eagerMode) {
-        sel.whereForeignKey(getJoinForeignKey(elem), sm.getObjectId(),
-            field.getDefiningMapping(), store);
+        ForeignKey fk = getJoinForeignKey(elem);
+        Object oid = getObjectIdForJoin(fk, sm);
+        sel.whereForeignKey(fk, oid, field.getDefiningMapping(), store);
 
         // order first, then select so that if the projection introduces
         // additional ordering, it will be after our required ordering
@@ -594,11 +626,31 @@ public abstract class StoreCollectionFieldStrategy
         return loadElement(null, store, fetch, res, joins);
     }
 
-    protected ForeignKey getJoinForeignKey() {
+    public ForeignKey getJoinForeignKey() {
         return getJoinForeignKey(getDefaultElementMapping(false));
     }
     
     boolean requiresOrderBy() {
     	return List.class.isAssignableFrom(field.getProxyType());
     }
+    
+    /**
+     * Gets the identity value of the given instance that is suitable to join to the given foreign key.
+     * The special case of the foreign key being a relation identifier will encode the value. 
+     */
+    Object getObjectIdForJoin(ForeignKey fk, OpenJPAStateManager sm) {
+        Object oid = sm.getObjectId();
+        if (!RelationStrategies.isRelationId(fk)) {
+            return oid;
+        }
+        
+        FieldMapping owningField = field.getMappedByMapping();
+        if (owningField != null && owningField.getHandler() instanceof RelationId) {
+            return ((RelationId)owningField.getHandler()).toRelationDataStoreValue(sm, null);
+        } 
+        if (oid instanceof OpenJPAId) {
+            return ((OpenJPAId)oid).getIdObject();
+        }
+        return oid;
+    }
 }
