@@ -573,6 +573,7 @@ public class ForeignKey
         if (_joins == null)
             _joins = new LinkedHashMap();
         _joins.put(local, toPK);
+        local.addConstraint(this);
         if (_joinsPK == null)
             _joinsPK = new LinkedHashMap();
         _joinsPK.put(toPK, local);
@@ -580,7 +581,7 @@ public class ForeignKey
         // force re-cache
         _locals = null;
         _pks = null;
-        if (_autoAssign == Boolean.FALSE)
+        if (Boolean.FALSE.equals(_autoAssign))
             _autoAssign = null;
     }
 
@@ -629,6 +630,7 @@ public class ForeignKey
 
         if (_joins != null) {
             rem = _joins.remove(col);
+            col.removeConstraint(this);
             if (rem != null) {
                 _locals = null;
                 _pks = null;
@@ -667,7 +669,7 @@ public class ForeignKey
         if ((_joins == null || _joins.isEmpty())
             && (_constsPK == null || _constsPK.isEmpty()))
             _pkTable = null;
-        if (remd && _autoAssign == Boolean.TRUE)
+        if (remd && Boolean.TRUE.equals(_autoAssign))
             _autoAssign = null;
         return remd;
     }
@@ -824,7 +826,7 @@ public class ForeignKey
                     Column[] locCols = fks[i].getColumns();
                     Column[] pkCols = fks[i].getPrimaryKeyColumns();
                     // Column counts must match
-                    if (locCols != null && pkCols != null & 
+                    if (locCols != null && pkCols != null &&
                         locCols.length != pkCols.length) {
                         Log log = dbdict.getLog();
                         if (log.isTraceEnabled()) {
