@@ -70,6 +70,8 @@ public class SchemaGenerator {
 
     private List<Listener> _listeners = null;
     private int _schemaObjects = 0;
+    
+    private Connection _conn = null;
 
     /**
      * Constructor.
@@ -265,63 +267,64 @@ public class SchemaGenerator {
      * {@link #generatePrimaryKeys}, and {@link #generateForeignKeys}
      * automatically.
      */
-    public void generateSchemas(DBIdentifier[] schemasAndTables)
-        throws SQLException {
+    public void generateSchemas(DBIdentifier[] schemasAndTables) throws SQLException {
         fireGenerationEvent(_loc.get("generating-schemas"));
-
-        Object[][] schemaMap;
-        if (schemasAndTables == null || schemasAndTables.length == 0)
-            schemaMap = _allowed;
-        else
-            schemaMap = parseSchemasList(schemasAndTables);
-
-        if (schemaMap == null) {
-            generateSchema(DBIdentifier.NULL, (DBIdentifier[])null);
-
-            // estimate the number of schema objects we will need to visit
-            // in order to estimate progress total for any listeners
-            int numTables = getTables(null).size();
-            _schemaObjects += numTables
-                + (_pks ? numTables : 0)
-                + (_indexes ? numTables : 0)
-                + (_fks ? numTables : 0);
-
-            if (_pks)
-                generatePrimaryKeys(DBIdentifier.NULL, null);
-            if (_indexes)
-                generateIndexes(DBIdentifier.NULL, null);
-            if (_fks)
-                generateForeignKeys(DBIdentifier.NULL, null);
-            return;
-        }
-
         // generate all schemas and tables
-        for (int i = 0; i < schemaMap.length; i++)
-            generateSchema((DBIdentifier) schemaMap[i][0],
-                (DBIdentifier[]) schemaMap[i][1]);
+        try {
+            getConn();
+            Object[][] schemaMap;
+            if (schemasAndTables == null || schemasAndTables.length == 0)
+                schemaMap = _allowed;
+            else
+                schemaMap = parseSchemasList(schemasAndTables);
 
-        // generate pks, indexes, fks
-        DBIdentifier schemaName = DBIdentifier.NULL;
-        DBIdentifier[] tableNames;
-        for (int i = 0; i < schemaMap.length; i++) {
-            schemaName = (DBIdentifier) schemaMap[i][0];
-            tableNames = (DBIdentifier[]) schemaMap[i][1];
+            if (schemaMap == null) {
+                generateSchema(DBIdentifier.NULL, (DBIdentifier[]) null);
 
-            // estimate the number of schema objects we will need to visit
-            // in order to estimate progress total for any listeners
-            int numTables = (tableNames != null) ? tableNames.length
-                : getTables(schemaName).size();
-            _schemaObjects += numTables
-                + (_pks ? numTables : 0)
-                + (_indexes ? numTables : 0)
-                + (_fks ? numTables : 0);
+                // estimate the number of schema objects we will need to visit
+                // in order to estimate progress total for any listeners
+                int numTables = getTables(null).size();
+                _schemaObjects +=
+                    numTables + (_pks ? numTables : 0) + (_indexes ? numTables : 0) + (_fks ? numTables : 0);
 
-            if (_pks)
-                generatePrimaryKeys(schemaName, tableNames);
-            if (_indexes)
-                generateIndexes(schemaName, tableNames);
-            if (_fks)
-                generateForeignKeys(schemaName, tableNames);
+                if (_pks)
+                    generatePrimaryKeys(DBIdentifier.NULL, null);
+                if (_indexes)
+                    generateIndexes(DBIdentifier.NULL, null);
+                if (_fks)
+                    generateForeignKeys(DBIdentifier.NULL, null);
+                return;
+            }
+
+            for (int i = 0; i < schemaMap.length; i++) {
+                generateSchema((DBIdentifier) schemaMap[i][0], (DBIdentifier[]) schemaMap[i][1]);
+            }
+
+            // generate pks, indexes, fks
+            DBIdentifier schemaName = DBIdentifier.NULL;
+            DBIdentifier[] tableNames;
+            for (int i = 0; i < schemaMap.length; i++) {
+                schemaName = (DBIdentifier) schemaMap[i][0];
+                tableNames = (DBIdentifier[]) schemaMap[i][1];
+
+                // estimate the number of schema objects we will need to visit
+                // in order to estimate progress total for any listeners
+                int numTables = (tableNames != null) ? tableNames.length : getTables(schemaName).size();
+                _schemaObjects +=
+                    numTables + (_pks ? numTables : 0) + (_indexes ? numTables : 0) + (_fks ? numTables : 0);
+
+                if (_pks) {
+                    generatePrimaryKeys(schemaName, tableNames);
+                }
+                if (_indexes) {
+                    generateIndexes(schemaName, tableNames);
+                }
+                if (_fks) {
+                    generateForeignKeys(schemaName, tableNames);
+                }
+            }
+        } finally {
+            closeConn();
         }
     }
 
@@ -352,25 +355,21 @@ public class SchemaGenerator {
         fireGenerationEvent(_loc.get("generating-schema", name));
 
         // generate tables, including columns and primary keys
-        Connection conn = _ds.getConnection();
-        DatabaseMetaData meta = conn.getMetaData();
+        DatabaseMetaData meta = _conn.getMetaData();
         try {
             if (tableNames == null)
-                generateTables(name, DBIdentifier.NULL, conn, meta);
+                generateTables(name, DBIdentifier.NULL, _conn, meta);
             else
                 for (int i = 0; i < tableNames.length; i++)
-                    generateTables(name, tableNames[i], conn, meta);
+                    generateTables(name, tableNames[i], _conn, meta);
 
-            if (_seqs)
-                generateSequences(name, DBIdentifier.NULL, conn, meta);
+            if (_seqs) {
+                generateSequences(name, DBIdentifier.NULL, _conn, meta);
+            }
         } finally {
             // some databases require a commit after metadata to release locks
             try {
-                conn.commit();
-            } catch (SQLException se) {
-            }
-            try {
-                conn.close();
+                _conn.commit();
             } catch (SQLException se) {
             }
         }
@@ -402,22 +401,17 @@ public class SchemaGenerator {
         throws SQLException {
         fireGenerationEvent(_loc.get("generating-all-primaries", schemaName));
 
-        Connection conn = _ds.getConnection();
-        DatabaseMetaData meta = conn.getMetaData();
+        DatabaseMetaData meta = _conn.getMetaData();
         try {
             if (tableNames == null)
-                generatePrimaryKeys(schemaName, null, conn, meta);
+                generatePrimaryKeys(schemaName, null, _conn, meta);
             else
                 for (int i = 0; i < tableNames.length; i++)
-                    generatePrimaryKeys(schemaName, tableNames[i], conn, meta);
+                    generatePrimaryKeys(schemaName, tableNames[i], _conn, meta);
         } finally {
             // some databases require a commit after metadata to release locks
             try {
-                conn.commit();
-            } catch (SQLException se) {
-            }
-            try {
-                conn.close();
+                _conn.commit();
             } catch (SQLException se) {
             }
         }
@@ -448,22 +442,17 @@ public class SchemaGenerator {
         throws SQLException {
         fireGenerationEvent(_loc.get("generating-all-indexes", schemaName));
 
-        Connection conn = _ds.getConnection();
-        DatabaseMetaData meta = conn.getMetaData();
+        DatabaseMetaData meta = _conn.getMetaData();
         try {
             if (tableNames == null)
-                generateIndexes(schemaName, null, conn, meta);
+                generateIndexes(schemaName, null, _conn, meta);
             else
                 for (int i = 0; i < tableNames.length; i++)
-                    generateIndexes(schemaName, tableNames[i], conn, meta);
+                    generateIndexes(schemaName, tableNames[i], _conn, meta);
         } finally {
             // some databases require a commit after metadata to release locks
             try {
-                conn.commit();
-            } catch (SQLException se) {
-            }
-            try {
-                conn.close();
+                _conn.commit();
             } catch (SQLException se) {
             }
         }
@@ -495,22 +484,17 @@ public class SchemaGenerator {
         throws SQLException {
         fireGenerationEvent(_loc.get("generating-all-foreigns", schemaName));
 
-        Connection conn = _ds.getConnection();
-        DatabaseMetaData meta = conn.getMetaData();
+        DatabaseMetaData meta = _conn.getMetaData();
         try {
             if (tableNames == null)
-                generateForeignKeys(schemaName, null, conn, meta);
+                generateForeignKeys(schemaName, null, _conn, meta);
             else
                 for (int i = 0; i < tableNames.length; i++)
-                    generateForeignKeys(schemaName, tableNames[i], conn, meta);
+                    generateForeignKeys(schemaName, tableNames[i], _conn, meta);
         } finally {
             // some databases require a commit after metadata to release locks
             try {
-                conn.commit();
-            } catch (SQLException se) {
-            }
-            try {
-                conn.close();
+                _conn.commit();
             } catch (SQLException se) {
             }
         }
@@ -577,7 +561,7 @@ public class SchemaGenerator {
             if (DBIdentifier.isNull(baseTableName) || baseTableName.equals("%")) {
                 tableName = cols[i].getTableIdentifier();
             } else {
-            	tableName = baseTableName;
+                tableName = baseTableName;
             }
             if (DBIdentifier.isNull(schemaName)) {
                 tableSchema = DBIdentifier.trimToNull(cols[i].getSchemaIdentifier());
@@ -974,7 +958,7 @@ public class SchemaGenerator {
                     || seqUpper.startsWith("JDO_"))) // legacy
                 continue;
             if (_dict.isSystemSequence(sequenceName, sequenceSchema,
-                schemaName != null))
+                schemaName != null, conn))
                 continue;
             if (!isAllowedTable(sequenceSchema, null))
                 continue;
@@ -1062,7 +1046,7 @@ public class SchemaGenerator {
      * An event corresponding to the generation of a schema object.
      */
     @SuppressWarnings("serial")
-    public class Event
+    public static class Event
         extends EventObject {
 
         private final int _total;
@@ -1076,4 +1060,18 @@ public class SchemaGenerator {
             return _total;
         }
     }
+    
+    private void getConn() throws SQLException {
+        if (_conn == null) {
+            _conn = _ds.getConnection();
+        }
+    }
+
+    private void closeConn() throws SQLException {
+        if (_conn != null ) {
+            if(! _conn.isClosed()) {
+                _conn.close();
+            }
+        }
+    }
 }
