@@ -19,6 +19,8 @@
 package org.apache.openjpa.jdbc.sql;
 
 import java.io.ByteArrayInputStream;
+import java.io.InputStream;
+import java.io.Reader;
 import java.lang.reflect.Method;
 import java.sql.Blob;
 import java.sql.Connection;
@@ -29,12 +31,14 @@ import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Types;
 import java.util.Arrays;
-import java.util.Set;
+import java.util.Date;
 import java.util.StringTokenizer;
 
+import javax.sql.DataSource;
+
 import org.apache.openjpa.jdbc.identifier.DBIdentifier;
-import org.apache.openjpa.jdbc.identifier.QualifiedDBIdentifier;
 import org.apache.openjpa.jdbc.kernel.JDBCFetchConfiguration;
+import org.apache.openjpa.jdbc.kernel.JDBCStore;
 import org.apache.openjpa.jdbc.kernel.exps.FilterValue;
 import org.apache.openjpa.jdbc.kernel.exps.Lit;
 import org.apache.openjpa.jdbc.kernel.exps.Param;
@@ -42,7 +46,6 @@ import org.apache.openjpa.jdbc.kernel.exps.Val;
 import org.apache.openjpa.jdbc.schema.Column;
 import org.apache.openjpa.jdbc.schema.Index;
 import org.apache.openjpa.jdbc.schema.Schema;
-import org.apache.openjpa.jdbc.schema.Sequence;
 import org.apache.openjpa.jdbc.schema.Table;
 import org.apache.openjpa.kernel.Filters;
 import org.apache.openjpa.kernel.MixedLockLevels;
@@ -52,6 +55,7 @@ import org.apache.openjpa.meta.JavaTypes;
 import org.apache.openjpa.util.OpenJPAException;
 import org.apache.openjpa.util.StoreException;
 import org.apache.openjpa.util.UnsupportedException;
+import org.apache.openjpa.util.UserException;
 
 import serp.util.Strings;
 
@@ -77,15 +81,13 @@ public class DB2Dictionary
     protected static final String withCSClause = "WITH CS";
     protected static final String withRSClause = "WITH RS";
     protected static final String withRRClause = "WITH RR";
-    protected static final String useKeepShareLockClause
-        = "USE AND KEEP SHARE LOCKS";
-    protected static final String useKeepUpdateLockClause
-        = "USE AND KEEP UPDATE LOCKS";
-    protected static final String useKeepExclusiveLockClause
-        = "USE AND KEEP EXCLUSIVE LOCKS";
+    protected static final String useKeepShareLockClause     = "USE AND KEEP SHARE LOCKS";
+    protected static final String useKeepUpdateLockClause    = "USE AND KEEP UPDATE LOCKS";
+    protected static final String useKeepExclusiveLockClause = "USE AND KEEP EXCLUSIVE LOCKS";
     protected static final String forReadOnlyClause = "FOR READ ONLY";
     protected static final String defaultSequenceSQL 
         = "SELECT SEQSCHEMA AS SEQUENCE_SCHEMA, SEQNAME AS SEQUENCE_NAME FROM SYSCAT.SEQUENCES";
+    static final String SYSDUMMY = "SYSIBM.SYSDUMMY1";
 
     protected String databaseProductName = "";
     protected String databaseProductVersion = "";
@@ -93,11 +95,11 @@ public class DB2Dictionary
     protected int min = 0;
     
     private int defaultBatchLimit = 100;
+    public boolean appendExtendedExceptionText = true;
     
     public DB2Dictionary() {
         platform = "DB2";
-        validationSQL = "SELECT DISTINCT(CURRENT TIMESTAMP) FROM "
-            + "SYSIBM.SYSTABLES";
+        validationSQL = "SELECT DISTINCT(CURRENT TIMESTAMP) FROM SYSIBM.SYSTABLES";
         supportsSelectEndIndex = true;
 
         nextSequenceQuery = "VALUES NEXTVAL FOR {0}";
@@ -118,15 +120,15 @@ public class DB2Dictionary
         fixedSizeTypeNameSet.addAll(Arrays.asList(new String[]{
             "LONG VARCHAR FOR BIT DATA", "LONG VARCHAR", "LONG VARGRAPHIC",
         }));
-        systemSchemas = new String(
-                "SYSCAT,SYSIBM,SYSSTAT,SYSIBMADM,SYSTOOLS");
+        systemSchemas = "SYSCAT,SYSIBM,SYSSTAT,SYSIBMADM,SYSTOOLS";
         maxConstraintNameLength = 18;
-        maxIndexNameLength = 18;
+        maxIndexNameLength = 128;
         maxColumnNameLength = 30;
         supportsDeferredConstraints = false;
         supportsDefaultDeleteAction = false;
         supportsAlterTableWithDropColumn = false;
         supportsLockingWithOrderClause = true;
+        supportsNullUniqueColumn = false;
 
         supportsNullTableForGetColumns = false;
         requiresCastForMathFunctions = true;
@@ -211,7 +213,7 @@ public class DB2Dictionary
 
         // ... and finish the cast
         if (toCast) {
-            Class c = ((Lit) val).getType();
+            Class<?> c = ((Lit) val).getType();
             int javaTypeCode = JavaTypes.getTypeCode(c);
             int jdbcTypeCode = getJDBCType(javaTypeCode, false);
             String typeName = getTypeName(jdbcTypeCode);
@@ -220,19 +222,12 @@ public class DB2Dictionary
             // if the literal is a string, use the default char col size
             // in the cast statement.
             if (String.class.equals(c))
-                selectSQL.append("(" + characterColumnSize + ")");
+                selectSQL.append("(" + getCastStringColumnSize(val) + ")");
 
             selectSQL.append(")");
         }
     }
 
-    public String[] getCreateSequenceSQL(Sequence seq) {
-        String[] sql = super.getCreateSequenceSQL(seq);
-        if (seq.getAllocate() > 1)
-            sql[0] += " CACHE " + seq.getAllocate();
-        return sql;
-    }
-
     @Override
     protected String getSequencesSQL(String schemaName, String sequenceName) {
         return getSequencesSQL(DBIdentifier.newSchema(schemaName), 
@@ -277,21 +272,6 @@ public class DB2Dictionary
         super.connectedConfiguration(conn);
 
         DatabaseMetaData metaData = conn.getMetaData();
-        try {
-            String str = "SELECT CURRENT SCHEMA FROM SYSIBM.SYSDUMMY1";
-            Statement stmnt = conn.createStatement();
-            ResultSet rs = stmnt.executeQuery(str);
-            if (rs.next()) {
-                String currSchema = rs.getString(1);
-                if (currSchema != null)
-                    setDefaultSchemaName(currSchema.trim());
-            }
-            rs.close();
-            stmnt.close();
-        } catch (SQLException e) {
-            if (log.isTraceEnabled())
-                log.trace(_loc.get("can_not_get_current_schema", e.getMessage()));
-        }
 
         String driverName = metaData.getDriverName();
         if (driverName != null && driverName.startsWith("IBM DB2"))
@@ -340,23 +320,22 @@ public class DB2Dictionary
             supportsLockingWithInnerJoin = true;
             supportsLockingWithOuterJoin = true;
             forUpdateClause = "WITH RR USE AND KEEP UPDATE LOCKS";
-            if (maj >=9)
+            
+            if (maj >=9) { 
                 supportsXMLColumn = true;
+            }
         }
 
         // platform specific settings
         switch (db2ServerType) {
         case db2UDBV82OrLater:
-            lastGeneratedKeyQuery = "SELECT IDENTITY_VAL_LOCAL() FROM "
-                + "SYSIBM.SYSDUMMY1";
+            lastGeneratedKeyQuery = "SELECT IDENTITY_VAL_LOCAL() FROM " + SYSDUMMY;
             break;
         case  db2ZOSV8xOrLater:
             // DB2 Z/OS 
             characterColumnSize = 255;
-            lastGeneratedKeyQuery = "SELECT IDENTITY_VAL_LOCAL() FROM "
-                + "SYSIBM.SYSDUMMY1";
-            nextSequenceQuery = "SELECT NEXTVAL FOR {0} FROM "
-                + "SYSIBM.SYSDUMMY1";
+            lastGeneratedKeyQuery = "SELECT IDENTITY_VAL_LOCAL() FROM " + SYSDUMMY;
+            nextSequenceQuery = "SELECT NEXTVAL FOR {0} FROM " + SYSDUMMY;
             // allow users to set a non default sequenceSQL. 
             if (defaultSequenceSQL.equals(sequenceSQL)){            	
 	            sequenceSQL = "SELECT SCHEMA AS SEQUENCE_SCHEMA, "
@@ -375,10 +354,8 @@ public class DB2Dictionary
             break;
         case db2ISeriesV5R3OrEarlier:
         case db2ISeriesV5R4OrLater:
-            lastGeneratedKeyQuery = "SELECT IDENTITY_VAL_LOCAL() FROM "
-                + "SYSIBM.SYSDUMMY1";
-            nextSequenceQuery = "SELECT NEXTVAL FOR {0} FROM "
-                + "SYSIBM.SYSDUMMY1";
+            lastGeneratedKeyQuery = "SELECT IDENTITY_VAL_LOCAL() FROM " + SYSDUMMY;
+            nextSequenceQuery = "SELECT NEXTVAL FOR {0} FROM " + SYSDUMMY;
             validationSQL = "SELECT DISTINCT(CURRENT TIMESTAMP) FROM "
                 + "QSYS2.SYSTABLES";
             // allow users to set a non default sequenceSQL.
@@ -391,6 +368,10 @@ public class DB2Dictionary
             }
             sequenceSchemaSQL = "SEQUENCE_SCHEMA = ?";
             sequenceNameSQL = "SEQUENCE_NAME = ?";
+            // V5R4 and earlier systems do not support retrieval of generated keys
+            if (isDB2ISeriesV5R4OrEarlier()) {
+            	supportsGetGeneratedKeys = false;
+            }
             break;
         }
     }
@@ -401,7 +382,8 @@ public class DB2Dictionary
 
     /**
      * Get the update clause for the query based on the
-     * updateClause and isolationLevel hints
+     * isolationLevel hints if it is for update.
+     * It also handles the UR hint when it is not for update.
      */
     protected String getForUpdateClause(JDBCFetchConfiguration fetch,
         boolean isForUpdate, Select sel) {
@@ -454,6 +436,11 @@ public class DB2Dictionary
                 }
                 break;
             }
+        } else {
+        	if ( fetch != null && fetch.getIsolation() == Connection.TRANSACTION_READ_UNCOMMITTED ) {
+	            forUpdateString.append(" ").append(forReadOnlyClause)
+	            .append(" ").append(withURClause);
+	    	}
         }
    
         return forUpdateString.toString();
@@ -481,6 +468,11 @@ public class DB2Dictionary
            && (maj >=6 || (maj == 5 && min >=4));
     }
 
+    public boolean isDB2ISeriesV5R4OrEarlier() {
+        return (databaseProductName.indexOf("AS") != -1
+            && ((maj == 5 && min <=4) || maj < 5));
+     }
+
     public boolean isDB2UDBV81OrEarlier() {
         return (databaseProductVersion.indexOf("SQL") != -1 
             || databaseProductName.indexOf("DB2/") != -1) 
@@ -571,10 +563,10 @@ public class DB2Dictionary
         return "";
     }
 
-    public OpenJPAException newStoreException(String msg, SQLException[] causes,
-        Object failed) {
-        if (causes != null && causes.length > 0)
+    public OpenJPAException newStoreException(String msg, SQLException[] causes, Object failed) {
+        if (appendExtendedExceptionText == true && causes != null && causes.length > 0) {
             msg = appendExtendedExceptionMsg(msg, causes[0]);
+        }
         return super.newStoreException(msg, causes, failed);
     }
 
@@ -584,7 +576,6 @@ public class DB2Dictionary
      */
     private String appendExtendedExceptionMsg(String msg, SQLException sqle){
        final String GETSQLCA ="getSqlca";
-       String exceptionMsg = new String();
        try {
             Method sqlcaM2 = sqle.getNextException().getClass()
                              .getMethod(GETSQLCA,null);
@@ -602,17 +593,23 @@ public class DB2Dictionary
             for (int i = 0; i < errds.length; i++)
                 errdStr.append(errdStr.length() > 0 ? ", " : "").
                     append(errds[i]);
-            exceptionMsg = exceptionMsg.concat( "SQLCA OUTPUT" +
-                    "[Errp=" + getSqlErrpMethd.invoke(sqlca, new Object[]{})
-                    + ", Errd=" + errdStr);
+            StringBuilder exceptionMsg = new StringBuilder();
+            exceptionMsg.append("SQLCA OUTPUT");
+            exceptionMsg.append("[Errp=");
+            exceptionMsg.append(getSqlErrpMethd.invoke(sqlca, new Object[]{}));
+            exceptionMsg.append(", Errd=");
+            exceptionMsg.append(errdStr);
 
             String Warn = new String((char[]) getSqlWarnMethd.
                     invoke(sqlca, new Object[]{}));
-            if (Warn.trim().length() != 0)
-                exceptionMsg = exceptionMsg.concat(", Warn=" +Warn + "]" );
-            else
-                exceptionMsg = exceptionMsg.concat( "]" );
-            msg = msg.concat(exceptionMsg);
+            if (Warn.trim().length() != 0) {
+                exceptionMsg.append(", Warn=");
+                exceptionMsg.append(Warn);
+                exceptionMsg.append("]");
+            } else {
+                exceptionMsg.append("]");
+            }
+            msg = msg.concat(exceptionMsg.toString());
             
             // for batched execution failures, SQLExceptions are nested
             SQLException sqle2 = sqle.getNextException();
@@ -670,7 +667,7 @@ public class DB2Dictionary
     private void appendXmlComparison1(SQLBuffer buf, String op, 
             FilterValue lhs, FilterValue rhs) {
         boolean castrhs = false;
-        Class rc = Filters.wrap(rhs.getType());
+        Class<?> rc = Filters.wrap(rhs.getType());
         int type = 0;
         if (rhs.isConstant()) {
             type = getJDBCType(JavaTypes.getTypeCode(rc), false);
@@ -776,7 +773,7 @@ public class DB2Dictionary
         String type = getTypeName(getJDBCType(JavaTypes.getTypeCode(val
             .getType()), false));
         if (String.class.equals(val.getType()))
-            type = type + "(" + characterColumnSize + ")";
+            type = type + "(" + getCastStringColumnSize(val) + ")";
         fstring = "CAST(? AS " + type + ")";
         return fstring;
     }
@@ -839,12 +836,12 @@ public class DB2Dictionary
     public void indexOf(SQLBuffer buf, FilterValue str, FilterValue find,
             FilterValue start) {
         if (find.getValue() != null) { // non constants
-            buf.append("(LOCATE(CAST((");
+            buf.append("LOCATE(CAST((");
             find.appendTo(buf);
             buf.append(") AS VARCHAR(1000)), ");
         } else {
             // this is a constant
-            buf.append("(LOCATE(");
+            buf.append("LOCATE(");
             find.appendTo(buf);
             buf.append(", ");
         }
@@ -856,16 +853,16 @@ public class DB2Dictionary
             str.appendTo(buf);
         }
         if (start != null) {
-            if (start.getValue() == null) {
+            if (start.getValue() != null) {
                 buf.append(", CAST((");
                 start.appendTo(buf);
-                buf.append(") AS INTEGER) + 1");
+                buf.append(") AS INTEGER)");
             } else {
                 buf.append(", ");
                 start.appendTo(buf);
             }
         }
-        buf.append(") - 1)");
+        buf.append(")");
     }
     
     /** 
@@ -904,7 +901,7 @@ public class DB2Dictionary
                 // case "(?" - convert to "CAST(? AS type"
                 String typeName = getTypeName(type);
                 if (String.class.equals(val.getType()))
-                    typeName = typeName + "(" + characterColumnSize + ")";
+                    typeName = typeName + "(" + getCastStringColumnSize(val) + ")";
                 String str = "CAST(? AS " + typeName + ")";
                 buf.replaceSqlString(sqlString.length() - 1,
                         sqlString.length(), str);
@@ -923,7 +920,7 @@ public class DB2Dictionary
 
     public void createIndexIfNecessary(Schema schema, DBIdentifier table,
             Column pkColumn) {
-        if (isDB2ZOSV8xOrLater()) {
+        if (db2ServerType == db2ZOSV8xOrLater) {
             // build the index for the sequence tables
             // the index name will be the fully qualified table name + _IDX
             Table tab = schema.getTable(table);
@@ -980,7 +977,7 @@ public class DB2Dictionary
     @Override
     public void setQueryTimeout(PreparedStatement stmnt, int timeout)
         throws SQLException {
-        if(isDB2ZOSV8xOrLater()) { 
+        if (db2ServerType == db2ZOSV8xOrLater) { 
             try { 
                 super.setQueryTimeout(stmnt, timeout);
             }
@@ -1052,4 +1049,157 @@ public class DB2Dictionary
             }
         }
     }
+
+    private int getCastStringColumnSize(Object val) {
+        int colSize = characterColumnSize;
+        if (val instanceof Lit) {
+            String literal = (String) ((Lit) val).getValue();
+            if (literal != null) {
+                int literalLen = literal.length();
+                if (literalLen > characterColumnSize) {
+                    colSize = literalLen;
+                }
+            }
+        }
+        return colSize;
+    }
+
+    @Override
+    public void insertBlobForStreamingLoad(Row row, Column col, 
+            JDBCStore store, Object ob, Select sel) throws SQLException {
+        if (ob != null) {
+            row.setBinaryStream(col, (InputStream)ob, -1);
+        } else {
+            row.setNull(col);
+        }
+    }
+
+    @Override
+    public void insertClobForStreamingLoad(Row row, Column col, Object ob)
+    throws SQLException {
+        if (ob != null) {
+            row.setCharacterStream(col, (Reader)ob, -1);
+        } else {
+            row.setNull(col);
+        }
+    }
+
+    @Override
+    public void updateBlob(Select sel, JDBCStore store, InputStream is)
+        throws SQLException {
+        //NO-OP
+    }
+
+    @Override
+    public void updateClob(Select sel, JDBCStore store, Reader reader)
+        throws SQLException {
+        //NO-OP
+    }
+
+    /**
+     * Set the given date value as a parameter to the statement.
+     */
+    public void setDate(PreparedStatement stmnt, int idx, Date val, Column col)
+        throws SQLException {
+        // When column metadata is not available, DB2 on z/OS does not like the value produced
+        // by the default dictionary - java.util.Date is converted to java.sql.Timestamp.
+        if (db2ServerType == db2ZOSV8xOrLater) {
+            if (col == null && val != null && "java.util.Date".equals(val.getClass().getName())) {
+                setDate(stmnt, idx, new java.sql.Date(val.getTime()), null, col);
+                return;
+            }
+        }
+        super.setDate(stmnt, idx, val, col);
+    }
+
+    public int getDB2MajorVersion() {
+        return maj;
+    }
+
+    public int getDB2MinorVersion() {
+        return min;
+    }
+    
+    public String getDefaultSchemaName()  {
+        if (defaultSchemaName == null) {
+            Connection conn = null;
+            Statement stmnt = null;
+            ResultSet rs = null;
+            try {
+                String str = "SELECT CURRENT SCHEMA FROM " + SYSDUMMY;
+                conn = getConnection(); 
+                if (conn != null) {
+                    stmnt = conn.createStatement();
+                    rs = stmnt.executeQuery(str);
+                    if (rs.next()) {
+                        String currSchema = rs.getString(1);
+                        if (currSchema != null) {
+                            setDefaultSchemaName(currSchema.trim());
+                        }
+                    }
+                } else {
+                    if (log.isTraceEnabled()) {
+                        log.trace(_loc.get("can_not_get_current_schema", "Unable to obtain a datasource"));
+                    }
+                }
+            } catch (SQLException e) {
+                if (log.isTraceEnabled()) {
+                    log.trace(_loc.get("can_not_get_current_schema", e.getMessage()));
+                }
+            } finally {
+                if (rs != null) {
+                    try {
+                        rs.close();
+                    } catch (SQLException se) {
+                        // ignore
+                    }
+                }
+                if (stmnt != null) {
+                    try {
+                        stmnt.close();
+                    } catch (SQLException se) {
+                        // ignore
+                    }
+                }
+                if (conn != null) { 
+                    try { 
+                        conn.close(); 
+                    }
+                    catch(SQLException se) { 
+                        // ignore
+                    }
+                }
+            }
+        }
+        return defaultSchemaName;
+    }
+
+    /**
+     * Obtain a connection from the configuration. Tries to use the jta-data-source first but falls back on the
+     * non-jta-data-source if no jta-data-source has been defined.
+     * 
+     * In practice this method is only called by getDefaultSchemaName which in turn is only used by the schema tool.
+     * 
+     * @throws SQLException If neither datasource is available.
+     * @return A connection which may be used to obtain the default schema name. 
+     */
+    private Connection getConnection() throws SQLException {
+    	DataSource  ds = null;
+    	try {
+            // try to obtain a connection from the primary datasource 
+            ds = conf.getDataSource(null);
+    	} catch (UserException uex) {
+    	}
+    	if (ds==null) {
+        	try {
+                // use datasource 2 if available
+                ds = conf.getDataSource2(null);
+        	} catch (UserException uex2) {
+        	}
+        }
+        if (ds != null) {
+            return ds.getConnection();
+        }
+        return null;
+    }
 }
