@@ -148,7 +148,7 @@ public class DBDictionaryFactory {
         String dclass, String props, Connection conn) {
         DBDictionary dict = null;
         try {
-            Class c = Class.forName(dclass, true,
+            Class<?> c = Class.forName(dclass, true,
                 AccessController.doPrivileged(
                     J2DoPrivHelper.getClassLoaderAction(
                         DBDictionary.class)));
@@ -158,7 +158,7 @@ public class DBDictionaryFactory {
             // if the dictionary was not found, make another attempt
             // at loading the dictionary using the current thread.
             try {
-                Class c = Thread.currentThread().getContextClassLoader().loadClass(dclass);
+                Class<?> c = Thread.currentThread().getContextClassLoader().loadClass(dclass);
                 dict = (DBDictionary) AccessController.doPrivileged(
                         J2DoPrivHelper.newInstanceAction(c));
             } catch (Exception e) {
@@ -259,6 +259,9 @@ public class DBDictionaryFactory {
             return CacheDictionary.class.getName();
         if (prod.indexOf("derby") != -1)
             return dbdictionaryPlugin.unalias("derby");
+        if (prod.indexOf("sapdb") != -1) {
+            return dbdictionaryPlugin.unalias("maxdb");
+        }
         // test h2 in a special way, because there's a decent chance the string 
         // h2 could appear in the URL of another database
         if (prod.indexOf("jdbc:h2:") != -1)
@@ -270,14 +273,14 @@ public class DBDictionaryFactory {
         // "testdb2" or something)
         if (prod.indexOf("db2") != -1 || prod.indexOf("as400") != -1)
             return dbdictionaryPlugin.unalias("db2");
+        if (prod.indexOf("soliddb") != -1) 
+            return dbdictionaryPlugin.unalias("soliddb");
 
         // known dbs that we don't support
         if (prod.indexOf("cloudscape") != -1)
             return DBDictionary.class.getName();
         if (prod.indexOf("daffodil") != -1)
             return DBDictionary.class.getName();
-        if (prod.indexOf("sapdb") != -1)
-            return DBDictionary.class.getName();
         if (prod.indexOf("idb") != -1) // instantdb
             return DBDictionary.class.getName();
 
