@@ -32,6 +32,7 @@ import java.util.Set;
 
 import org.apache.commons.lang.StringUtils;
 import org.apache.openjpa.jdbc.identifier.DBIdentifier;
+import org.apache.openjpa.jdbc.identifier.DBIdentifier.DBIdentifierType;
 import org.apache.openjpa.jdbc.kernel.JDBCFetchConfiguration;
 import org.apache.openjpa.jdbc.kernel.JDBCStore;
 import org.apache.openjpa.jdbc.kernel.exps.FilterValue;
@@ -121,7 +122,7 @@ public class MySQLDictionary
             "AUTO_INCREMENT", "BINARY", "BLOB", "CHANGE", "ENUM", "INFILE",
             "INT1", "INT2", "INT4", "FLOAT1", "FLOAT2", "FLOAT4", "LOAD",
             "MEDIUMINT", "OUTFILE", "REPLACE", "STARTING", "TEXT", "UNSIGNED", 
-            "ZEROFILL",
+            "ZEROFILL", "INDEX", 
         }));
 
         // reservedWordSet subset that CANNOT be used as valid column names
@@ -145,6 +146,7 @@ public class MySQLDictionary
             "STARTING", "TABLE", "THEN", "TO", "TRAILING", "TRUE", "UNION",
             "UNIQUE", "UNSIGNED", "UPDATE", "USAGE", "USING", "VALUES",
             "VARCHAR", "VARYING", "WHEN", "WHERE", "WITH", "WRITE", "ZEROFILL",
+            "INDEX", 
         }));
 
         // MySQL requires double-escape for strings
@@ -459,8 +461,9 @@ public class MySQLDictionary
      */
     @Override
     public String getTypeName(Column col) {
-        if (col.getType() == Types.BLOB) {
-            if (col.getSize() == 0)   // unknown size
+        // handle blobs differently, if the DBItentifierType is NULL (e.g. no column definition is set). 
+        if (col.getType() == Types.BLOB && col.getTypeIdentifier().getType() == DBIdentifierType.NULL) {
+            if (col.getSize() <= 0)   // unknown size
                 return blobTypeName;  // return old default of 64KB
             else if (col.getSize() <= 255)
                 return tinyBlobTypeName;
@@ -474,5 +477,19 @@ public class MySQLDictionary
             return super.getTypeName(col);
         }
     }
+
+    @Override
+    public void indexOf(SQLBuffer buf, FilterValue str, FilterValue find,
+        FilterValue start) {
+        buf.append("LOCATE(");
+        find.appendTo(buf);
+        buf.append(", ");
+        str.appendTo(buf);
+        if (start != null) {
+            buf.append(", ");
+            start.appendTo(buf);
+        }
+        buf.append(")");
+    }
 }
 
