@@ -22,6 +22,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.lang.reflect.Constructor;
+import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.security.AccessController;
@@ -36,18 +37,21 @@ import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.Arrays;
 import java.util.Date;
+import java.util.HashSet;
 import java.util.Map;
+import java.util.Set;
 
 import org.apache.openjpa.jdbc.identifier.DBIdentifier;
+import org.apache.openjpa.jdbc.identifier.Normalizer;
 import org.apache.openjpa.jdbc.kernel.JDBCFetchConfiguration;
 import org.apache.openjpa.jdbc.kernel.JDBCStore;
 import org.apache.openjpa.jdbc.kernel.exps.FilterValue;
 import org.apache.openjpa.jdbc.schema.Column;
-import org.apache.openjpa.jdbc.schema.Sequence;
 import org.apache.openjpa.jdbc.schema.Table;
 import org.apache.openjpa.kernel.Filters;
 import org.apache.openjpa.lib.jdbc.DelegatingConnection;
 import org.apache.openjpa.lib.jdbc.DelegatingPreparedStatement;
+import org.apache.openjpa.lib.jdbc.ReportingSQLException;
 import org.apache.openjpa.lib.util.ConcreteClassGenerator;
 import org.apache.openjpa.lib.util.J2DoPrivHelper;
 import org.apache.openjpa.lib.util.Localizer;
@@ -72,6 +76,8 @@ public class PostgresDictionary
 
     private Method dbcpGetDelegate;
     private Method connectionUnwrap;
+    
+    protected Set<String> _timestampTypes = new HashSet<String>(); 
 
     static {
         try {
@@ -124,6 +130,15 @@ public class PostgresDictionary
      * method.
      */
     public boolean supportsSetFetchSize = true;
+    
+    /**
+     * Statement used to determine whether a sequence is owned.  Owned 
+     * sequences are managed by the database and are considered system 
+     * sequences.
+     * parm 1: '<table_name.schema_name>'
+     * parm 2: '<column_name>'
+     */
+    public String isOwnedSequenceSQL = "SELECT pg_get_serial_sequence(?, ?)";
 
     public PostgresDictionary() {
         platform = "PostgreSQL";
@@ -134,9 +149,6 @@ public class PostgresDictionary
         supportsSelectStartIndex = true;
         supportsSelectEndIndex = true;
 
-        // PostgreSQL requires double-escape for strings
-        searchStringEscape = "\\\\";
-
         maxTableNameLength = 63;
         maxColumnNameLength = 63;
         maxIndexNameLength = 63;
@@ -159,7 +171,7 @@ public class PostgresDictionary
         useSetStringForClobs = true;
         bitTypeName = "BOOL";
         smallintTypeName = "SMALLINT";
-        realTypeName = "FLOAT8";
+        realTypeName = "FLOAT4";
         tinyintTypeName = "SMALLINT";
         binaryTypeName = "BYTEA";
         blobTypeName = "BYTEA";
@@ -168,7 +180,7 @@ public class PostgresDictionary
         clobTypeName = "TEXT";
         longVarcharTypeName = "TEXT";
         doubleTypeName = "DOUBLE PRECISION";
-        timestampTypeName = "ABSTIME";
+        timestampTypeName = "TIMESTAMP";
         fixedSizeTypeNameSet.addAll(Arrays.asList(new String[]{
             "BOOL", "BYTEA", "NAME", "INT8", "INT2", "INT2VECTOR", "INT4",
             "REGPROC", "TEXT", "OID", "TID", "XID", "CID", "OIDVECTOR",
@@ -179,7 +191,6 @@ public class PostgresDictionary
         supportsLockingWithDistinctClause = false;
         supportsQueryTimeout = false;
         supportsLockingWithOuterJoin = false;
-        supportsNullTableForGetImportedKeys = true;
 
         reservedWordSet.addAll(Arrays.asList(new String[]{
             "ABORT", "ACL", "AGGREGATE", "APPEND", "ARCHIVE", "ARCH_STORE",
@@ -208,6 +219,10 @@ public class PostgresDictionary
             "THEN", "TO", "TRAILING", "TRUE", "UNION", "UNIQUE", "USER", 
             "USING", "VERBOSE", "WHEN", "WHERE",
         }));
+        
+        _timestampTypes.add("ABSTIME");
+        _timestampTypes.add("TIMESTAMP");
+        _timestampTypes.add(timestampTypeName.toUpperCase()); // handle user configured timestamp types.
     }
 
     @Override
@@ -334,22 +349,14 @@ public class PostgresDictionary
             substring(buf, str, start, null);
         else
             str.appendTo(buf);
-        buf.append(") - 1");
+        buf.append(")");
         if (start != null) {
-            buf.append(" + ");
+            buf.append(" - 1 + ");
             start.appendTo(buf);
         }
         buf.append(")");
     }
 
-    @Override
-    public String[] getCreateSequenceSQL(Sequence seq) {
-        String[] sql = super.getCreateSequenceSQL(seq);
-        if (seq.getAllocate() > 1)
-            sql[0] += " CACHE " + seq.getAllocate();
-        return sql;
-    }
-
     @Override
     protected boolean supportsDeferredUniqueConstraints() {
         // Postgres only supports deferred foreign key constraints.
@@ -378,13 +385,182 @@ public class PostgresDictionary
 
     public boolean isSystemSequence(DBIdentifier name, DBIdentifier schema,
         boolean targetSchema) {
+        return isSystemSequence(name, schema, targetSchema, null);
+    }
+    
+    public boolean isSystemSequence(DBIdentifier name, DBIdentifier schema,
+        boolean targetSchema, Connection conn) {
         if (super.isSystemSequence(name, schema, targetSchema))
             return true;
+        
+        if (isOwnedSequence(name, schema, conn)) {
+            return true;
+        }
+        return false;
+    }
 
+    /**
+     * Uses the native Postgres function pg_get_serial_sequence to determine whether
+     * a sequence is owned by the database.  Column types such as bigserial use a 
+     * system assigned sequence generator of the format: table_column_seq
+     * 
+     * @see http://www.postgresql.org/docs/current/static/functions-info.html
+     */
+    public boolean isOwnedSequence(DBIdentifier name, DBIdentifier schema, Connection conn) {
+        
+        String strName = DBIdentifier.isNull(name) ? "" : name.getName();
+        // basic check for SEQ suffix.  not SEQ, not an owned sequence
+        if (strName == null || !strName.toUpperCase().endsWith("_SEQ"))
+            return false;
+
+        // If no connection, use secondary method to determine ownership
+        if (conn == null) {
+            return isOwnedSequence(strName);
+        }
+        
+        // Build permutations of table, column pairs from the provided 
+        // sequence name.  If any of them are determined owned, assume the 
+        // sequence is owned.  This is not perfect, but considerably better than
+        // considering all sequences suffixed with _seq are db owned.
+        String[][] namePairs = buildNames(strName);
+        
+        if(namePairs != null) { // unable to parse strName. 
+            try {
+                for (int i = 0; i < namePairs.length; i++) {
+                    if (queryOwnership(conn, namePairs[i], schema)) {
+                        return true;
+                    }
+                }
+            } catch (Throwable t) {
+                if (log.isWarnEnabled())
+                    log.warn(_loc.get("psql-owned-seq-warning"), t);
+                return isOwnedSequence(strName);
+            }
+        } else { 
+            if(log.isTraceEnabled()) { 
+                log.trace(String.format("Unable to query ownership for sequence %s using the connection. " +
+                		"Falling back to simpler detection based on the name", 
+                    name.getName()));
+            }
+            
+            return isOwnedSequence(strName);
+        }
+        return false;
+    }
+    
+    private boolean queryOwnership(Connection conn, String[] namePair,
+        DBIdentifier schema) throws Throwable {
+        PreparedStatement ps = null;
+        ResultSet rs = null;
+        try {
+            ps = prepareStatement(conn, isOwnedSequenceSQL);
+            String tblName = "";
+            if (!DBIdentifier.isEmpty(schema)) {
+                tblName = schema.getName() + getIdentifierDelimiter();  
+            }
+            tblName += namePair[0];
+            ps.setString(1, tblName);
+            String colName = toDBName(DBIdentifier.newColumn(namePair[1]));
+            ps.setString(2, colName);
+            ps.execute();
+            rs = ps.getResultSet();
+            if (rs == null || !rs.next()) {
+                return false;
+            }
+            String val = getString(rs, 1);
+            if (val == null || val.length() == 0) {
+                return false;
+            }
+            return true;
+        } catch (Throwable t) {
+            if (t instanceof ReportingSQLException) {
+                // Handle known/acceptable exceptions
+                // 42P01 - table does not exist
+                // 42703 - column does not exist within table
+                ReportingSQLException rse = (ReportingSQLException)t;
+                if ("42P01".equals(rse.getSQLState()) ||
+                    "42703".equals(rse.getSQLState())) {
+                    return false;
+                }
+            }
+            throw t;
+        }
+        finally {
+            if (rs != null) {
+                try {
+                    rs.close();
+                } catch (Throwable t) {}
+            }
+            if (ps != null) {
+                try {
+                    ps.close();
+                } catch (Throwable t) {}
+            }
+        }
+    }
+
+    /**
+     * Owned sequences are of the form <table>_<col>_seq. Table and column 
+     * names can contain underscores so permutations of these names must be 
+     * produced for ownership verification.
+     * @param strName
+     * @return If strName cannot be split into three or more parts null will be returned.
+     *  Otherwise a String[][] of the potential sequence names will be returned.
+     */
+    private String[][] buildNames(String strName) {
+        // split the sequence name into components
+        // owned sequences are of the form <table>_<col>_seq
+        String[] parts = Normalizer.splitName(strName, "_");
+        
+        if (parts == null || parts.length < 3) {
+            if(log.isTraceEnabled()) { 
+                log.trace(String.format("Unable to parse sequences from %s. Found %s parts. Returning null", 
+                    strName, parts == null ? 0 : parts.length));
+            }
+            return null;
+        }
+        // Simple and most common case
+        if (parts.length == 3) {
+            return new String[][] { {parts[0], parts[1]} };
+        }
+        // If table or column names contain underscores, build a list
+        // of possibilities
+        String[][] names = new String[(parts.length - 2)][2];
+        for (int i = 0; i < parts.length - 2; i++) {
+            String[] namePair = new String[2];
+            StringBuilder name0 = new StringBuilder();
+            StringBuilder name1 = new StringBuilder();
+            for (int j = 0; j < parts.length - 1; j++) {
+                if (j <= i) {
+                    name0.append(parts[j]);
+                    if (j < i) {
+                        name0.append("_");
+                    }
+                } else {
+                    name1.append(parts[j]);
+                    if (j < parts.length - 2) {
+                        name1.append("_");
+                    }
+                }
+            }
+            namePair[0] = name0.toString();
+            namePair[1] = name1.toString();
+            names[i] = namePair;
+        }
+        return names;
+    }
+
+    /**
+     * Secondary logic if owned sequences cannot be determined by calling the
+     * db.  This logic assumes that any sequence suffixed with _SEQ is an
+     * owned sequence (identical to the behavior of prior versions of OpenJPA).
+     * @param strName
+     * @return
+     */
+    private boolean isOwnedSequence(String strName) {
         // filter out generated sequences used for bigserial cols, which are
         // of the form <table>_<col>_seq
-        String strName = DBIdentifier.isNull(name) ? null : name.getName();
-        int idx = strName.indexOf('_');
+        int idx = (strName == null) ? -1 : strName.indexOf('_');
         return idx != -1 && idx != strName.length() - 4
             && strName.toUpperCase().endsWith("_SEQ");
     }
@@ -561,7 +737,7 @@ public class PostgresDictionary
     }
 
     /**
-     * Determine whether XML column is supported.
+     * Determine XML column support and backslash handling.
      */
     public void connectedConfiguration(Connection conn) throws SQLException {
         super.connectedConfiguration(conn);
@@ -589,6 +765,10 @@ public class PostgresDictionary
         if ((maj >= 9 || (maj == 8 && min >= 3))) {
             supportsXMLColumn = true;
         }
+        // Old PostgreSQL requires double-escape for strings.
+        if ((maj <= 8 || (maj == 9 && min == 0))) {
+            searchStringEscape = "\\\\";
+        }
     }
  
     /**
@@ -627,7 +807,7 @@ public class PostgresDictionary
             try {
                 Method m = obj.getClass().getMethod("getType", (Class[]) null);
                 Object type = m.invoke(obj, (Object[]) null);
-                if (((String) type).equalsIgnoreCase(timestampTypeName)) {
+                if(_timestampTypes.contains(((String) type).toUpperCase())) { 
                     return rs.getTimestamp(column);
                 }
             } catch (Throwable t) {
@@ -742,6 +922,19 @@ public class PostgresDictionary
                 Class<?> dbcpConnectionClass =
                     Class.forName("org.apache.commons.dbcp.DelegatingConnection", true, AccessController
                         .doPrivileged(J2DoPrivHelper.getContextClassLoaderAction()));
+                Class<?> poolingDataSource = Class.forName(
+                        "org.apache.commons.dbcp.PoolingDataSource", true,
+                        AccessController.doPrivileged(J2DoPrivHelper
+                                .getContextClassLoaderAction()));
+                Method setAccessToUnderlyingConnectionAllowed = poolingDataSource
+                        .getMethod("setAccessToUnderlyingConnectionAllowed",
+                                boolean.class);
+                
+                Field this$0 = conn.getClass().getDeclaredField("this$0");
+                this$0.setAccessible(true);
+                Object poolingDataSourceObj = this$0.get(conn);
+                setAccessToUnderlyingConnectionAllowed.invoke(poolingDataSourceObj,
+                        true);
                 
                 dbcpGetDelegate = dbcpConnectionClass.getMethod("getInnermostDelegate");
             }
@@ -855,3 +1048,4 @@ public class PostgresDictionary
         }
     }
 }
+
