@@ -37,6 +37,7 @@ import org.apache.openjpa.jdbc.kernel.exps.Val;
 import org.apache.openjpa.jdbc.schema.Column;
 import org.apache.openjpa.jdbc.schema.Sequence;
 import org.apache.openjpa.jdbc.schema.Table;
+import org.apache.openjpa.kernel.Filters;
 import org.apache.openjpa.kernel.exps.Parameter;
 
 
@@ -167,9 +168,8 @@ public final class SQLBuffer
                     _userParams.addAll(paramIndex, buf._userParams);
                 }
                 if (buf._userIndex != null) {
-                     if (_userIndex == null) {
+                     if (_userIndex == null)
                          _userIndex = new ArrayList();
-                     }
                      _userIndex.addAll(buf._userIndex);
                 }
                 if (buf._cols != null)
@@ -270,7 +270,7 @@ public final class SQLBuffer
     public SQLBuffer appendValue(Object o) {
         return appendValue(o, null);
     }
-    
+
     /**
      * Append a system inserted parameter value for a specific column.
      */
@@ -287,155 +287,73 @@ public final class SQLBuffer
      * @param userParam if non-null, designates a 'user' parameter.
      */
     public SQLBuffer appendValue(Object o, Column col, Parameter userParam) {
+        return appendValue(o, col, userParam, true);
+    }
+
+    public SQLBuffer appendValue(Object o, Column col, Parameter userParam, boolean useParamToken) {
         if (o == null)
             _sql.append("NULL");
         else if (o instanceof Raw)
             _sql.append(o.toString());
         else {
-            _sql.append(PARAMETER_TOKEN);
+            Class<?> type = Filters.wrap(o.getClass());
+            if (useParamToken || !validParamLiteralType(type)) {
+                _sql.append(PARAMETER_TOKEN);
 
-            // initialize param and col lists; we hold off on col list until
-            // we get the first non-null col
-            if (_params == null)
-                _params = new ArrayList();
-            if (_userParams == null)
-                _userParams = new ArrayList();
-            if (col != null && _cols == null) {
-                _cols = new ArrayList();
-                while (_cols.size() < _params.size())
-                    _cols.add(null);
-            }
+                // initialize param and col lists; we hold off on col list until
+                // we get the first non-null col
+                if (_params == null)
+                    _params = new ArrayList();
+                if (_userParams == null)
+                    _userParams = new ArrayList();
+                if (col != null && _cols == null) {
+                    _cols = new ArrayList();
+                    while (_cols.size() < _params.size())
+                        _cols.add(null);
+                }
 
-            _params.add(o);
-            if (userParam != null) {
-                Object param = userParam;
-                if (userParam instanceof CollectionParam)
-                    param = ((CollectionParam) userParam).clone();
-                _userParams.add(param);
-                if (_userIndex == null)
-                    _userIndex = new ArrayList();
-                int index = _params.size()-1;
-                _userIndex.add(index);
-                _userIndex.add(param);
+                _params.add(o);
+                if (userParam != null) {
+                    Object param = userParam;
+                    if (userParam instanceof CollectionParam)
+                        param = ((CollectionParam) userParam).clone();
+                    _userParams.add(param);
+                    if (_userIndex == null)
+                        _userIndex = new ArrayList();
+                    int index = _params.size()-1;
+                    _userIndex.add(index);
+                    _userIndex.add(param);
+                }
+                else
+                    _userParams.add(o);
+                if (_cols != null)
+                    _cols.add(col);
+            } else {
+                if (type == String.class) {
+                    _sql.append("'" + o.toString().replace("'", "''") + "'");
+                } else if ( type == Character.class ) {
+                    if (_dict.storeCharsAsNumbers) {
+                        _sql.append(Integer.toString(((Character)o).charValue()));
+                    } else {
+                        _sql.append("'" + o.toString().replace("'", "''") + "'");
+                    }
+                } else {
+                    _sql.append(o.toString());
+                }
             }
-            else
-                _userParams.add(o);
-            if (_cols != null)
-                _cols.add(col);
         }
         return this;
     }
 
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(boolean b) {
-        return appendValue(b, null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(boolean b, Column col) {
-        return appendValue((b) ? Boolean.TRUE : Boolean.FALSE, col);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(byte b) {
-        return appendValue(b, null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(byte b, Column col) {
-        return appendValue(new Byte(b), col);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(char c) {
-        return appendValue(c, null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(char c, Column col) {
-        return appendValue(new Character(c), col);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(double d) {
-        return appendValue(d, null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(double d, Column col) {
-        return appendValue(new Double(d), col);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(float f) {
-        return appendValue(f, null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(float f, Column col) {
-        return appendValue(new Float(f), col);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(int i) {
-        return appendValue(Integer.valueOf(i), null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(int i, Column col) {
-        return appendValue(Integer.valueOf(i), col);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(long l) {
-        return appendValue(Long.valueOf(l), null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(long l, Column col) {
-        return appendValue(Long.valueOf(l), col);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(short s) {
-        return appendValue(s, null);
-    }
-
-    /**
-     * Append a parameter value.
-     */
-    public SQLBuffer appendValue(short s, Column col) {
-        return appendValue(new Short(s), col);
+    private boolean validParamLiteralType(Class<?> type) {
+        boolean ret = type == String.class
+                || type == Integer.class 
+                || type == Character.class 
+                || type == Boolean.class
+                || type == Short.class
+                || type == Long.class
+                || type == Byte.class;
+        return ret;
     }
 
     /**
