@@ -737,4 +737,36 @@ public interface Select
      * Extended trace that logs eager relations
      */
     public void logEagerRelations();
+
+    /**
+     * Set table-per-class metadata for polymorphic queries
+     */
+    public void setTablePerClassMeta(ClassMapping meta);
+
+    /**
+     * get table-per-class metadata for polymorphic queries
+     */
+    public ClassMapping getTablePerClassMeta();
+
+    /**
+     * Set joined table metadatas for polymorphic queries
+     */
+    public void setJoinedTableClassMeta(List meta);
+
+    /**
+     * get joined table metadatas for polymorphic queries
+     */
+    public List getJoinedTableClassMeta();
+
+    /**
+     * Set joined table metadatas excluded for polymorphic queries
+     */
+    public void setExcludedJoinedTableClassMeta(List meta);
+
+    /**
+     * get joined table metadatas excluded for polymorphic queries
+     */
+    public List getExcludedJoinedTableClassMeta();
+    
+    public DBDictionary getDictionary() ; 
 }
