@@ -64,9 +64,18 @@ public class Compatibility {
     private boolean _superclassDiscriminatorStrategyByDefault = true;
     private boolean _isAbstractMappingUniDirectional = false;
     private boolean _isNonDefaultMappingAllowed = false;
-    private boolean _reorderMetaDataResolution = true;
     private boolean _reloadOnDetach = false;
     private boolean _ignoreDetachedStateFieldForProxySerialization = false;
+    private boolean _checkDatabaseForCascadePersistToDetachedEntity = false;
+    private boolean _overrideContextClassloader = true;
+    private boolean _parseAnnotationsForQueryMode = true;
+    private boolean _convertPositionalParametersToNamed = false;
+    
+    private boolean _useListAttributeForArrays = false; 
+    private boolean _metaFactoriesAreStrict = false; 
+    private boolean _resetFlushFlagForCascadePersist = true;//OPENJPA-2051
+    private boolean _singletonLifecycleEventManager = false;
+    private boolean _filterPCRegistryClasses = false; // OPENJPA-2288
     
     /**
      * Whether to require exact identity value types when creating object
@@ -328,6 +337,14 @@ public class Compatibility {
     public boolean getIgnoreDetachedStateFieldForProxySerialization() {
         return _ignoreDetachedStateFieldForProxySerialization;
     }
+    
+    public boolean getConvertPositionalParametersToNamed() {
+        return _convertPositionalParametersToNamed;
+    }
+
+    public void setConvertPositionalParametersToNamed(boolean c) {
+        _convertPositionalParametersToNamed = c;
+    }    
 
     /**
      * Whether OpenJPA should flush changes before detaching or serializing an
@@ -508,26 +525,6 @@ public class Compatibility {
     public boolean isNonDefaultMappingAllowed() {
         return _isNonDefaultMappingAllowed;
     }
-    
-    /**
-     * Whether OpenJPA should reorder entities in MetaDataRepository.processBuffer() to ensure that the metadata for 
-     * entities with foreign keys in their identity are processed after the entities it depends on.
-     * 
-     * @return true if the reordering should be performed, false if not.
-     */
-    public boolean getReorderMetaDataResolution() {
-        return _reorderMetaDataResolution;
-    }
-    
-    /**
-     * Whether OpenJPA should reorder entities in MetaDataRepository.processBuffer() to ensure that the metadata for 
-     * entities with foreign keys in their identity are processed after the entities it depends on.
-     * 
-     * @param reorderProcessBuffer true if the reordering should be performed, false if not.
-     */
-    public void setReorderMetaDataResolution(boolean reorderProcessBuffer) {
-        _reorderMetaDataResolution = reorderProcessBuffer;
-    }
 
     /**
      * Whether OpenJPA should attempt to load fields when the DetachState
@@ -555,5 +552,180 @@ public class Compatibility {
      */
     public void setReloadOnDetach(boolean reloadOnDetach) {
         _reloadOnDetach = reloadOnDetach;
-    }       
+    }      
+    
+    /**
+     * Whether OpenJPA will check the database for an Entity when cascading a persist to another Entity. This property
+     * only applies for the case where we are trying to cascade a persist to an Entity which doesn't have a StateManager
+     * and we can't determine if it is detached. 
+     *   
+     * @since 2.1.1
+     */
+    public boolean getCheckDatabaseForCascadePersistToDetachedEntity(){
+        return _checkDatabaseForCascadePersistToDetachedEntity;
+    }
+    
+    /**
+     * Whether OpenJPA will check the database for an Entity when cascading a persist to another Entity. This property
+     * only applies for the case where we are trying to cascade a persist to an Entity which doesn't have a StateManager
+     * and we can't determine if it is detached. 
+     *   
+     * @since 2.1.1
+     */
+    public void setCheckDatabaseForCascadePersistToDetachedEntity(boolean b){
+        _checkDatabaseForCascadePersistToDetachedEntity = b;
+    }
+
+    /**
+     * Whether to temporally override the thread's Context Classloader when processing
+     * ORM XML documents to avoid deadlock potential with certain Classloader hierarchy
+     * configurations.  Defaults to false.
+     */
+    public boolean getOverrideContextClassloader() {
+        return _overrideContextClassloader;
+    }
+
+    /**
+     * Whether to temporally override the thread's Context Classloader when processing
+     * ORM XML documents to avoid deadlock potential with certain Classloader hierarchy
+     * configurations.  Defaults to false.
+     */
+    public void setOverrideContextClassloader(boolean overrideContextClassloader) {
+        _overrideContextClassloader = overrideContextClassloader;
+    }
+    
+    /**
+     * Whether OpenJPA will scan every persistent class in an XML mapping file for annotations prior to executing a 
+     * query. In practice this scan is rarely needed, but the option to enable it is present for compatibility with 
+     * prior releases.
+     * @since 2.0.2
+     * @return true if the annotations should be re-parsed when resolving MetaData in MODE_QUERY.  
+     */
+    public boolean getParseAnnotationsForQueryMode() {
+        return _parseAnnotationsForQueryMode;
+    }
+
+    /**
+     * Whether OpenJPA will scan every persistent class in an XML mapping file for annotations prior to executing a 
+     * query. In practice this scan is rarely needed, but the option to enable it is present for compatibility with 
+     * prior releases.
+     * @since 2.0.2
+     */
+    public void setParseAnnotationsForQueryMode(boolean parseAnnotationsForQueryMode) {
+        _parseAnnotationsForQueryMode = parseAnnotationsForQueryMode;
+    }
+
+    /**
+     * This property can be used to allow OpenJPA to use ListAttributes for all types of Arrays, not just those with the
+     * 
+     * @PersistentCollection annotation. If the canonical metamodel classes were generated in an early version of
+     *                       OpenJPA (e.g. 2.0.0, 2.0.1, or 2.1.0) it is recommended to set this property to true. If
+     *                       you have generated your metamodel classes on later versions of OpenJPA (e.g. 2.2.0) you may
+     *                       want to have this set to false.
+     * @since 2.2.0
+     * @return true if OpenJPA will use ListAttributes for <b>all</b> arrays, false if OpenJPA will use ListAttributes
+     *         for <b>only</b> arrays which use the @PersistentCollection annotation.
+     */
+    public boolean getUseListAttributeForArrays() {
+        return _useListAttributeForArrays;
+    }
+
+    /**
+     * This property can be used to allow OpenJPA to use ListAttributes for all types of Arrays, not just those with the
+     * 
+     * @PersistentCollection annotation. If the canonical metamodel classes were generated in an early version of
+     *                       OpenJPA (e.g. 2.0.0, 2.0.1, or 2.1.0) it is recommended to set this property to true. If
+     *                       you have generated your metamodel classes on later versions of OpenJPA (e.g. 2.2.0) you may
+     *                       want to have this set to false.
+     * @since 2.2.0
+     * @param useListAttribute
+     *            whether OpenJPA will use ListAttributes for all arrays.
+     */   
+    public void setUseListAttributeForArrays(boolean useListAttribute ) { 
+        _useListAttributeForArrays = useListAttribute;
+    }
+
+    /**
+     * Whether the MetaDataFactory and MappingFactory should be set to strict mode. If strict mode is used a
+     * MetaDataFactory will only resolve MetaData (no JDBC mappings), and a MappingFactory will only resolve Mapping
+     * information.
+     * 
+     * @since 2.2.0
+     * @return Whether strict mode should be used.
+     */
+    public boolean getMetaFactoriesAreStrict() { 
+        return _metaFactoriesAreStrict;
+    }
+    
+    /**
+     * Set whether MetaDataFactories and MappingFactories will be strictly separate.
+     * 
+     * @since 2.2.0
+     * 
+     * @param metaFactoriesAreStrict
+     *            True if the MetaDataFactory should only process metadata, and the MappingFactory should only process
+     *            mappings.
+     */
+    public void setMetaFactoriesAreStrict(boolean metaFactoriesAreStrict) {
+        _metaFactoriesAreStrict = metaFactoriesAreStrict;
+    }
+
+    /**
+     * Whether OpenJPA should reset the internal state (flush flag) when cascading a persist to another 
+     * Entity. That is, when a flush is performed, OpenJPA keep state to indicate the flush has been 
+     * performed.  In certain cascade persist scenarios the fact that a flush has been performed prior to 
+     * a cascade persist can cause certain entities to not be written to the database given the prior 
+     * flush.  This property, when set, will cause the flush flag to be reset in cascade scenarios. For more 
+     * details see JIRA OPENJPA-2051
+     *    
+     * @since 2.0.x
+     */
+    public boolean getResetFlushFlagForCascadePersist(){
+        return _resetFlushFlagForCascadePersist;
+    }
+    
+    /**
+     * Whether OpenJPA should reset the internal state (flush flag) when cascading a persist to another 
+     * Entity. That is, when a flush is performed, OpenJPA keep state to indicate the flush has been 
+     * performed.  In certain cascade persist scenarios the fact that a flush has been performed prior to 
+     * a cascade persist can cause certain entities to not be written to the database given the prior 
+     * flush.  This property, when set, will cause the flush flag to be reset in cascade scenarios. For more 
+     * details see JIRA OPENJPA-2051
+     *    
+     * @since 2.0.x
+     */
+    public void setResetFlushFlagForCascadePersist(boolean b){
+        _resetFlushFlagForCascadePersist = b;
+    }
+
+    /**
+     * Returns true if life cycle event manager is a singleton configuration.
+     */
+    public boolean isSingletonLifecycleEventManager() {
+        return _singletonLifecycleEventManager;
+    }
+
+    /**
+     * This property set whether each EntityManager has its own life cycle event manager.
+       By default, each EntityManager only fires events to the registered listeners to the entities
+       it manages. If the life cycle event manager is a singleton, events will be fired to listeners
+       registered to all instances of EntityManager in the same persistence unit.
+     */
+    public void setSingletonLifecycleEventManager(boolean singleton) {
+        _singletonLifecycleEventManager = singleton;
+    }
+    
+    /**
+     * Whether the metadata processor should filter classes dispatched by the PCRegistry listener system.
+     **/
+    public boolean  getFilterPCRegistryClasses() {
+        return _filterPCRegistryClasses; 
+    }
+     
+    /**
+     * Whether the metadata processor should filter classes dispatched by the PCRegistry listener system.
+     **/
+    public void setFilterPCRegistryClasses(boolean bool) {
+        _filterPCRegistryClasses = bool;
+    }
 }
