@@ -21,6 +21,7 @@ package org.apache.openjpa.conf;
 import java.util.Collection;
 import java.util.Map;
 
+import org.apache.openjpa.audit.Auditor;
 import org.apache.openjpa.datacache.CacheDistributionPolicy;
 import org.apache.openjpa.datacache.DataCache;
 import org.apache.openjpa.datacache.DataCacheManager;
@@ -32,6 +33,7 @@ import org.apache.openjpa.event.LifecycleEventManager;
 import org.apache.openjpa.event.OrphanedKeyAction;
 import org.apache.openjpa.event.RemoteCommitEventManager;
 import org.apache.openjpa.event.RemoteCommitProvider;
+import org.apache.openjpa.instrumentation.InstrumentationManager;
 import org.apache.openjpa.kernel.AutoClear;
 import org.apache.openjpa.kernel.AutoDetach;
 import org.apache.openjpa.kernel.BrokerFactory;
@@ -211,6 +213,12 @@ public interface OpenJPAConfiguration
     public static final String OPTION_JDBC_CONNECTION =
         "openjpa.option.JDBCConnection";
 
+    /**
+     * Option for enable fire &#064;PostLoad events on merge operations
+     */
+    public static final String OPTION_POSTLOAD_ON_MERGE =
+        "openjpa.option.PostLoadOnMerge";
+
     /**
      * Return the set of option strings supported by this runtime. This set
      * is mutable.
@@ -239,7 +247,7 @@ public interface OpenJPAConfiguration
      * This change will trigger all registered Product Derivations to mutate 
      * other configuration properties.
      *
-     * @param fullname of the specification that possibly encodes major and
+     * @param spec fullname of the specification that possibly encodes major and
      * minor version information. For encoding format
      * @see Specification
      * 
@@ -255,7 +263,7 @@ public interface OpenJPAConfiguration
      * This change will trigger all registered Product Derivations to mutate 
      * other configuration properties.
      *
-     * @param fullname of the specification that possibly encodes major and
+     * @param spec fullname of the specification that possibly encodes major and
      * minor version information. For encoding format
      * @see Specification
      * 
@@ -1836,6 +1844,111 @@ public interface OpenJPAConfiguration
      * @since 2.0.0
      */
     public void setCacheDistributionPolicy(String policyPlugin);
+    
+    /**
+     * Gets the plug-in string that defines instrumentation providers and what
+     * they instrument.
+     * @return a plug-in string for the instrumentation configuration
+     * @since 2.1.0
+     */
+    public String getInstrumentation();
+    
+    /**
+     * Sets the plug-in string that defines instrumentation providers and what
+     * they instrument.
+     * @param providers a plug-in string for the instrumentation configuration
+     * @since 2.1.0
+     */
+    public void setInstrumentation(String providers);
+    
+    /**
+     * Gets an instance of the instrumentation manager.  The instrumentation
+     * provides access to configured instrumentation providers and can be used
+     * to manage them at runtime.
+     * @return an instance of the instrumentation manager
+     * @since 2.1.0
+     */
+    public InstrumentationManager getInstrumentationManagerInstance();
+    
+    
+    /**
+     * Gets the singular instance of {@link Auditor} associated with this configuration.
+     * 
+     * @since 2.2.0
+     */
+    public Auditor getAuditorInstance();
+    
+    /**
+     * Sets the singular instance of {@link Auditor} associated with this configuration.
+     * 
+     * @since 2.2.0
+     */
 
+    public void setAuditorInstance(Auditor auditor);
+    
+    /**
+     * Gets the plug-in string of {@link Auditor} specified in this configuration.
+     * 
+     * @since 2.2.0
+     */
+    public String getAuditor();
+    
+    /**
+     * Sets the plug-in string of {@link Auditor} specified in this configuration.
+     * 
+     * @since 2.2.0
+     */
+    public void setAuditor(String s);
+
+     /**
+      * Whether to send &#064;PostLoad events on a merge operation.
+      * @since 2.2.0
+      */
+     public boolean getPostLoadOnMerge();
+
+     /**
+      * Whether to send &#064;PostLoad events on a merge operation.
+      * @since 2.2.0
+      */
+     public void setPostLoadOnMerge(boolean postLoadOnMerge);
+
+     /**
+      * Whether to send &#064;PostLoad events on a merge operation.
+      * @since 2.2.0
+      */
+     public void setPostLoadOnMerge(Boolean postLoadOnMerge);
+    
+     /**
+      * Whether to attempt to optimize id class copy operations during the
+      * enhancement process.  Optimization is only applicable for simple id classes
+      * that have a constructor with the proper construction parameters and
+      * direct assignments to fields within the id class during construction. 
+      * If the optimization cannot occur, the enhancer will fallback to the 
+      * noraml behavior.
+      * @since 2.2.0
+      */
+     public boolean getOptimizeIdCopy();
+     
+     /**
+      * Whether to attempt to optimize id class copy operations during the
+      * enhancement process.  Optimization is only applicable for simple id classes
+      * that have a constructor with the proper construction parameters and
+      * direct assignments to fields within the id class during construction. 
+      * If the optimization cannot occur, the enhancer will fallback to the 
+      * normal behavior.
+      * @since 2.2.0
+      */
+     public void setOptimizeIdCopy(boolean optimizeIds);
+
+     /**
+      * Whether to attempt to optimize id class copy operations during the
+      * enhancement process.  Optimization is only applicable for simple id classes
+      * that have a constructor with the proper construction parameters and
+      * direct assignments to fields within the id class during construction. 
+      * If the optimization cannot occur, the enhancer will fallback to the 
+      * normal behavior.
+      * @since 2.2.0
+      */
+      public void setOptimizeIdCopy(Boolean optimizeIds);
 }
 
