@@ -24,7 +24,6 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -118,7 +117,7 @@ public class DataCacheStoreManager
         DataCache cache;
         for (Class<?> cls : classes) {
             cache = mdr.getMetaData(cls, loader, false).getDataCache();
-            if (cache != null)
+            if (cache != null && cache.getEvictOnBulkUpdate())
                 cache.removeAll(cls, false);
         }
     }
@@ -144,9 +143,13 @@ public class DataCacheStoreManager
                         continue;
 
                     mods = getModifications(modMap, cache);
-                    data = newPCData(sm);
+                    data = newPCData(sm, cache);
                     data.store(sm);
                     mods.additions.add(new PCDataHolder(data, sm));
+                    CacheStatistics stats = cache.getStatistics();
+                    if (stats.isEnabled()) {
+                        ((CacheStatisticsSPI)stats).newPut(data.getType());
+                    }
                 }
             }
 
@@ -175,13 +178,17 @@ public class DataCacheStoreManager
                     // dirty, but maybe it got dropped from the cache in the
                     // interim
                     if (data == null) {
-                        data = newPCData(sm);
+                        data = newPCData(sm, cache);
                         data.store(sm);
                         mods.newUpdates.add(new PCDataHolder(data, sm));
                     } else {
                         data.store(sm, fields);
                         mods.existingUpdates.add(new PCDataHolder(data, sm));
                     }
+                    CacheStatistics stats = cache.getStatistics();
+                    if (stats.isEnabled()) {
+                        ((CacheStatisticsSPI)stats).newPut(data.getType());
+                    }
                 }
             }
 
@@ -272,9 +279,22 @@ public class DataCacheStoreManager
     }
 
     public boolean exists(OpenJPAStateManager sm, Object edata) {
-        DataCache cache = _mgr.selectCache(sm); 
-        if (cache != null && !isLocking(null) && cache.contains(sm.getObjectId()))
+        DataCache cache = _mgr.selectCache(sm);
+        CacheStatistics stats = (cache == null) ? null : cache.getStatistics();
+        if (cache != null && !isLocking(null) && cache.contains(sm.getObjectId())){
+            if (stats != null && stats.isEnabled()) {
+                // delay this call ONLY if stats collection is enabled
+                Class<?> cls = sm.getMetaData().getDescribedType();
+                ((CacheStatisticsSPI)stats).newGet(cls, false);
+            }
             return true;
+        }
+        // If isLocking(null)==true && cache.contains(..) == true... probably shouldn't count?
+        if (stats != null && stats.isEnabled()) {
+            // delay this call ONLY if stats collection is enabled
+            Class<?> cls = sm.getMetaData().getDescribedType();
+            ((CacheStatisticsSPI)stats).newGet(cls, false);
+        }
         return super.exists(sm, edata);
     }
 
@@ -290,7 +310,7 @@ public class DataCacheStoreManager
                     edata.set(i);
                 }
             }
-            if(edata.cardinality()==oids.size()){
+            if (edata.cardinality() == oids.size()){
                 return true;
             }
         }
@@ -300,17 +320,25 @@ public class DataCacheStoreManager
 
     public boolean syncVersion(OpenJPAStateManager sm, Object edata) {
         DataCache cache = _mgr.selectCache(sm);
-        if (cache == null || sm.isEmbedded())
+        FetchConfiguration fc = sm.getContext().getFetchConfiguration();
+        CacheStatistics stats = (cache == null) ? null : cache.getStatistics();
+        if (cache == null || sm.isEmbedded() || fc.getCacheRetrieveMode() == DataCacheRetrieveMode.BYPASS) {
+            if (stats != null && stats.isEnabled()) {
+                ((CacheStatisticsSPI) stats).newGet(sm.getMetaData().getDescribedType(), false);
+            }
             return super.syncVersion(sm, edata);
-
+        }
         DataCachePCData data;
         Object version = null;
         data = cache.get(sm.getObjectId());
         if (!isLocking(null) && data != null)
-            version = data.getVersion();
+            version = data.getVersion(); 
 
         // if we have a cached version update from there
         if (version != null) {
+            if (stats != null && stats.isEnabled()) {
+                ((CacheStatisticsSPI)stats).newGet(data.getType(), true);
+            }
             if (!version.equals(sm.getVersion())) {
                 sm.setVersion(version);
                 return false;
@@ -318,41 +346,62 @@ public class DataCacheStoreManager
             return true;
         }
 
+        if(stats.isEnabled()){
+            Class<?> cls = (data == null) ? sm.getMetaData().getDescribedType() : data.getType();
+            ((CacheStatisticsSPI) stats).newGet(cls, false);
+        }
         // use data store version
         return super.syncVersion(sm, edata);
     }
 
     public boolean initialize(OpenJPAStateManager sm, PCState state, FetchConfiguration fetch, Object edata) {
-        boolean fromDatabase; 
         DataCache cache = _mgr.selectCache(sm);
-        DataCachePCData data = null;
-        boolean updateCache = _ctx.getFetchConfiguration().getCacheRetrieveMode() != DataCacheRetrieveMode.BYPASS
-                            && _ctx.getPopulateDataCache();
-        if (cache == null || sm.isEmbedded() 
-            || _ctx.getFetchConfiguration().getCacheRetrieveMode() == DataCacheRetrieveMode.BYPASS
-            || _ctx.getFetchConfiguration().getCacheStoreMode() == DataCacheStoreMode.REFRESH) {
+        if (cache == null) {
+            return super.initialize(sm, state, fetch, edata);
+        }
+
+        DataCachePCData data = cache.get(sm.getObjectId());
+        CacheStatistics stats = cache.getStatistics();
+        boolean fromDatabase = false; 
+        boolean alreadyCached = data != null; 
+        if (sm.isEmbedded() 
+         || fetch.getCacheRetrieveMode() == DataCacheRetrieveMode.BYPASS
+         || fetch.getCacheStoreMode() == DataCacheStoreMode.REFRESH) {
+            // stats -- Skipped reading from the cache, noop
             fromDatabase = super.initialize(sm, state, fetch, edata);
         } else {
-            data = cache.get(sm.getObjectId());
-            if (data != null && !isLocking(fetch)) {                
-                //### the 'data.type' access here probably needs to be
-                //### addressed for bug 511
+            if (alreadyCached && !isLocking(fetch)) {
+                if (stats.isEnabled()) {
+                    ((CacheStatisticsSPI)stats).newGet(data.getType(), true);
+                }
                 sm.initialize(data.getType(), state);
                 data.load(sm, fetch, edata);
-                // no need to update the cache. 
-                updateCache = false;
-                fromDatabase = true;
             } else {
-                // initialize from store manager
+                if (!alreadyCached) {
+                    if (stats.isEnabled()) {
+                        // Get the classname from MetaData... but this won't be right in every case. 
+                        ((CacheStatisticsSPI)stats).newGet(sm.getMetaData().getDescribedType(), false);
+                    }
+                }
                 fromDatabase = super.initialize(sm, state, fetch, edata);
             }
         }
-
-        if (cache != null && (fromDatabase && updateCache)) {
-            // update cache if the result came from the database and configured to store or refresh the cache.
-            cacheStateManager(cache, sm, data);
+        // update cache if the result came from the database and configured to use or refresh the cache.
+        boolean updateCache = fromDatabase && _ctx.getPopulateDataCache()
+                           && ((fetch.getCacheStoreMode() == DataCacheStoreMode.USE && !alreadyCached)
+                            || (fetch.getCacheStoreMode() == DataCacheStoreMode.REFRESH));
+        if (updateCache) {
+            // It is possible that the "cacheability" of the provided SM changed after hitting the DB. This can happen
+            // when we are operating against an Entity that is in some sort of inheritance structure.
+            cache = _mgr.selectCache(sm);
+            if (cache != null) {
+                cacheStateManager(cache, sm, data);
+                if (stats.isEnabled()) {
+                    ((CacheStatisticsSPI) stats).newPut(sm.getMetaData().getDescribedType());
+                }
+            }
         }
-        return fromDatabase;
+        return fromDatabase || alreadyCached;
     }
     
     private void cacheStateManager(DataCache cache, OpenJPAStateManager sm, DataCachePCData data) {
@@ -371,7 +420,7 @@ public class DataCacheStoreManager
             // initial load of the data.
             boolean isNew = data == null;
             if (isNew) {
-                data = newPCData(sm);
+                data = newPCData(sm, cache);
             }
             data.store(sm);
             if (isNew) { 
@@ -387,31 +436,81 @@ public class DataCacheStoreManager
     public boolean load(OpenJPAStateManager sm, BitSet fields,
         FetchConfiguration fetch, int lockLevel, Object edata) {
         DataCache cache = _mgr.selectCache(sm);
-        if (cache == null || sm.isEmbedded() || bypass(_ctx.getFetchConfiguration(), StoreManager.FORCE_LOAD_NONE))
-            return super.load(sm, fields, fetch, lockLevel, edata);
 
+        boolean found = false;
+        if (cache == null || sm.isEmbedded() || bypass(fetch, StoreManager.FORCE_LOAD_NONE)) {
+            found = super.load(sm, fields, fetch, lockLevel, edata);
+            updateDataCache(found, sm, fetch);
+            return found;
+        }
+
+        CacheStatistics stats = cache.getStatistics();
         DataCachePCData data = cache.get(sm.getObjectId());
         if (lockLevel == LockLevels.LOCK_NONE && !isLocking(fetch) && data != null)
             data.load(sm, fields, fetch, edata);
-        if (fields.length() == 0)
+        if (fields.length() == 0){
+            if (stats.isEnabled()) {
+                Class<?> cls = (data == null) ? sm.getMetaData().getDescribedType() : data.getType();
+                ((CacheStatisticsSPI)stats).newGet(cls, true);
+            }
             return true;
+        }
 
         // load from store manager; clone the set of still-unloaded fields
         // so that if the store manager decides to modify it it won't affect us
-        if (!super.load(sm, (BitSet) fields.clone(), fetch, lockLevel, edata))
-            return false;
-        if (_ctx.getPopulateDataCache()) {
-            cacheStateManager(cache, sm, data);
-        }
-        return true;
+        found = super.load(sm, (BitSet) fields.clone(), fetch, lockLevel, edata);
 
+        // Get new instance of cache after DB load since it may have changed
+        updateDataCache(found, sm, fetch);
+
+        return found;
+    }
+
+    /**
+     * Updates or inserts and item into the data cache.  If storeMode=USE and not in the cache,
+     * the item is inserted.  If storeMode=REFRESH the item is inserted, updated, or if found=false, 
+     * removed from the cache.
+     * @param found whether the entity was found by the store manager
+     * @param sm the state manager
+     * @param fetch fetch configuration
+     */
+    private void updateDataCache(boolean found, OpenJPAStateManager sm, FetchConfiguration fetch) {
+
+        if (!_ctx.getPopulateDataCache() || sm == null || fetch.getCacheStoreMode() == DataCacheStoreMode.BYPASS) {
+            return;
+        }
+
+        DataCache cache = _mgr.selectCache(sm);
+        if (cache == null) {
+            return;
+        }
+
+        DataCachePCData data = cache.get(sm.getObjectId());
+        boolean alreadyCached = data != null;
+
+        if ((fetch.getCacheStoreMode() == DataCacheStoreMode.USE && !alreadyCached) ||
+             fetch.getCacheStoreMode() == DataCacheStoreMode.REFRESH) {
+            // If not found in the DB and the item is in the cache, and not locking remove the item
+            if (!found && data != null && !isLocking(fetch)) {
+                cache.remove(sm.getObjectId());
+                return;
+            }
+            // Update or insert the item into the cache
+            if (found) {
+                cacheStateManager(cache, sm, data);
+                CacheStatistics stats = cache.getStatistics();
+                if (stats.isEnabled()) {
+                    ((CacheStatisticsSPI) stats).newPut(sm.getMetaData().getDescribedType());
+                }
+            }
+        }
     }
 
     public Collection<Object> loadAll(Collection<OpenJPAStateManager> sms, PCState state, int load,
-    		FetchConfiguration fetch, Object edata) {
-    	if (bypass(fetch, load)) {
-    	    return super.loadAll(sms, state, load, fetch, edata);
-    	}
+        FetchConfiguration fetch, Object edata) {
+        if (bypass(fetch, load)) {
+            return super.loadAll(sms, state, load, fetch, edata);
+        }
 
         Map<OpenJPAStateManager, BitSet> unloaded = null;
         List<OpenJPAStateManager> smList = null;
@@ -440,9 +539,9 @@ public class DataCacheStoreManager
                 unloaded = addUnloaded(sm, null, unloaded);
         }
         
-        for (Iterator<DataCache> itr = caches.keySet().iterator(); itr.hasNext();) {
-            cache = itr.next();
-            smList = caches.get(cache);
+    for(Entry<DataCache,List<OpenJPAStateManager>> entry : caches.entrySet()){
+            cache = entry.getKey();
+            smList = entry.getValue();
             List<Object> oidList = new ArrayList<Object>(smList.size());
 
             for (OpenJPAStateManager sm : smList) {
@@ -453,15 +552,21 @@ public class DataCacheStoreManager
 
             for (OpenJPAStateManager sm : smList) {
                 data = dataMap.get(sm.getObjectId());
-
+                CacheStatistics stats = cache.getStatistics();
                 if (sm.getManagedInstance() == null) {
                     if (data != null) {
                         //### the 'data.type' access here probably needs
                         //### to be addressed for bug 511
+                        if (stats.isEnabled()) {
+                            ((CacheStatisticsSPI) stats).newGet(data.getType(), true);
+                        }
                         sm.initialize(data.getType(), state);
                         data.load(sm, fetch, edata);
                     } else {
                         unloaded = addUnloaded(sm, null, unloaded);
+                        if (stats.isEnabled()) {
+                            ((CacheStatisticsSPI)stats).newGet(sm.getMetaData().getDescribedType(), false);
+                        }
                     }
                 } else if (load != FORCE_LOAD_NONE
                         || sm.getPCState() == PCState.HOLLOW) {
@@ -470,10 +575,22 @@ public class DataCacheStoreManager
                         // load unloaded fields
                         fields = sm.getUnloaded(fetch);
                         data.load(sm, fields, fetch, edata);
-                        if (fields.length() > 0)
+                        if (fields.length() > 0){
                             unloaded = addUnloaded(sm, fields, unloaded);
-                    } else
+                            if (stats.isEnabled()) {
+                                ((CacheStatisticsSPI)stats).newGet(data.getType(), false);
+                            }
+                        }else{
+                            if (stats.isEnabled()) {
+                                ((CacheStatisticsSPI)stats).newGet(data.getType(), true);
+                            }
+                        }
+                    } else{
                         unloaded = addUnloaded(sm, null, unloaded);
+                        if (stats.isEnabled()) {
+                            ((CacheStatisticsSPI)stats).newGet(sm.getMetaData().getDescribedType(), false);
+                        }
+                    }
                 }
             }
         }
@@ -509,7 +626,7 @@ public class DataCacheStoreManager
 
                 isNew = data == null;
                 if (isNew)
-                    data = newPCData(sm);
+                    data = newPCData(sm, cache);
                 if (fields == null)
                     data.store(sm);
                 else
@@ -518,6 +635,10 @@ public class DataCacheStoreManager
                     cache.put(data);
                 else
                     cache.update(data);
+                CacheStatistics stats = cache.getStatistics();
+                if (stats.isEnabled()) {
+                    ((CacheStatisticsSPI)stats).newPut(data.getType());
+                }
             } finally {
                 cache.writeUnlock();
             }
@@ -659,15 +780,15 @@ public class DataCacheStoreManager
         _ctx.getConfiguration().getRemoteCommitEventManager()
             .fireLocalStaleNotification(oid);
     }
-
+   
     /**
      * Create a new cacheable instance for the given state manager.
      */
-    private DataCachePCData newPCData(OpenJPAStateManager sm) {
+    private DataCachePCData newPCData(OpenJPAStateManager sm, DataCache cache) {
         ClassMetaData meta = sm.getMetaData();
         if (_gen != null)
             return (DataCachePCData) _gen.generatePCData(sm.getObjectId(), meta);
-        return new DataCachePCDataImpl(sm.fetchObjectId(), meta, _mgr.selectCache(sm).getName());
+        return new DataCachePCDataImpl(sm.fetchObjectId(), meta, cache.getName());
     }
 
     /**
