@@ -124,11 +124,11 @@ public class QueryCacheStoreQuery
         // get the cached data
         QueryResult res = _cache.get(qk);
 
-        if (res == null)
-            return null;        
-        if (res.isEmpty())
-            return Collections.emptyList();
-
+        
+        if (res == null) {
+            return null;
+        }
+               
         // this if block is invoked if the evictOnTimestamp is set to true
         if (_cache instanceof AbstractQueryCache) {
             AbstractQueryCache qcache = (AbstractQueryCache) _cache;
@@ -144,7 +144,7 @@ public class QueryCacheStoreQuery
                     for (Long ts: timestamps) {
                         // if this is true we have to evict the query 
                         // from cache
-                        if (queryTS < ts) { 
+                        if (queryTS <= ts) { 
                             qcache.remove(qk);
                             return null;
                         }
@@ -152,6 +152,10 @@ public class QueryCacheStoreQuery
                 }
             }
         }
+      
+        if (res.isEmpty()) {
+            return Collections.emptyList();
+        }
 
         int projs = getContext().getProjectionAliases().length;
         if (projs == 0) {
@@ -258,6 +262,10 @@ public class QueryCacheStoreQuery
         return _query.newCompilation();
     }
 
+    public Object getCompilation() {
+        return _query.getCompilation();
+    }
+
     public void populateFromCompilation(Object comp) {
         _query.populateFromCompilation(comp);
     }
@@ -327,9 +335,10 @@ public class QueryCacheStoreQuery
         public ResultObjectProvider executeQuery(StoreQuery q, Object[] params,
             Range range) {
             QueryCacheStoreQuery cq = (QueryCacheStoreQuery) q;
+            Object parsed = cq.getDelegate().getCompilation();
             QueryKey key = QueryKey.newInstance(cq.getContext(),
                 _ex.isPacking(q), params, _candidate, _subs, range.start, 
-                range.end);
+                range.end, parsed);
             List<Object> cached = cq.checkCache(key);
             if (cached != null)
                 return new ListResultObjectProvider(cached);
@@ -371,7 +380,7 @@ public class QueryCacheStoreQuery
 
             // evict from the data cache
             for (int i = 0; i < cmd.length; i++) {
-                if (cmd[i].getDataCache() != null)
+                if (cmd[i].getDataCache() != null && cmd[i].getDataCache().getEvictOnBulkUpdate())
                     cmd[i].getDataCache().removeAll(
                         cmd[i].getDescribedType(), true);
             }
@@ -701,7 +710,7 @@ public class QueryCacheStoreQuery
     /**
      * Struct to recognize cached oids.
      */
-    private static class CachedObjectId {
+    private static class CachedObjectId implements java.io.Serializable {
 
         public final Object oid;
 
