@@ -19,7 +19,9 @@
 package org.apache.openjpa.kernel;
 
 import java.io.ObjectStreamException;
+import java.security.AccessController;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
@@ -31,7 +33,6 @@ import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.locks.ReentrantLock;
 
-import javax.naming.NamingException;
 import javax.transaction.Status;
 import javax.transaction.Synchronization;
 import javax.transaction.Transaction;
@@ -39,9 +40,9 @@ import javax.transaction.TransactionManager;
 
 import org.apache.commons.collections.set.MapBackedSet;
 import org.apache.commons.lang.StringUtils;
-import org.apache.commons.lang.exception.NestableRuntimeException;
+import org.apache.openjpa.kernel.AuditManager;
+import org.apache.openjpa.audit.Auditor;
 import org.apache.openjpa.conf.BrokerValue;
-import org.apache.openjpa.conf.MetaDataRepositoryValue;
 import org.apache.openjpa.conf.OpenJPAConfiguration;
 import org.apache.openjpa.conf.OpenJPAConfigurationImpl;
 import org.apache.openjpa.conf.OpenJPAVersion;
@@ -52,17 +53,19 @@ import org.apache.openjpa.enhance.PCRegistry;
 import org.apache.openjpa.enhance.PersistenceCapable;
 import org.apache.openjpa.event.BrokerFactoryEvent;
 import org.apache.openjpa.event.RemoteCommitEventManager;
+import org.apache.openjpa.instrumentation.InstrumentationManager;
 import org.apache.openjpa.lib.conf.Configuration;
 import org.apache.openjpa.lib.conf.Configurations;
+import org.apache.openjpa.lib.instrumentation.InstrumentationLevel;
 import org.apache.openjpa.lib.log.Log;
 import org.apache.openjpa.lib.util.J2DoPrivHelper;
 import org.apache.openjpa.lib.util.Localizer;
+import org.apache.openjpa.lib.util.Options;
 import org.apache.openjpa.lib.util.concurrent.ConcurrentReferenceHashSet;
 import org.apache.openjpa.meta.MetaDataModes;
 import org.apache.openjpa.meta.MetaDataRepository;
 import org.apache.openjpa.util.GeneralException;
 import org.apache.openjpa.util.InvalidStateException;
-import org.apache.openjpa.util.MetaDataException;
 import org.apache.openjpa.util.OpenJPAException;
 import org.apache.openjpa.util.UserException;
 
@@ -153,12 +156,6 @@ public abstract class AbstractBrokerFactory
         _conf = config;
         _brokers = newBrokerSet();
         getPcClassLoaders();
-        if (config.isInitializeEagerly()) {
-            newBroker(_conf.getConnectionUserName(), 
-            	_conf.getConnectionPassword(), 
-            	_conf.isConnectionFactoryModeManaged(),
-                _conf.getConnectionRetainModeConstant(), false).close(); 
-        }
     }
 
     /**
@@ -238,7 +235,7 @@ public abstract class AbstractBrokerFactory
         if (remote.areRemoteEventsEnabled())
             broker.addTransactionListener(remote);
 
-        loadPersistentTypes(broker.getClassLoader());
+       loadPersistentTypes(broker.getClassLoader());
         _brokers.add(broker);
         _conf.setReadOnly(Configuration.INIT_STATE_FROZEN);
     }
@@ -475,8 +472,14 @@ public abstract class AbstractBrokerFactory
         // reset these transient fields to empty values
         _transactional = new ConcurrentHashMap<Object,Collection<Broker>>();
         _brokers = newBrokerSet();
-
+        
+        // turn off logging while de-serializing BrokerFactory
+        String saveLogConfig = _conf.getLog();
+        _conf.setLog("none");
         makeReadOnly();
+        // re-enable any logging which was in effect
+        _conf.setLog(saveLogConfig);  
+        
         return this;
     }
 
@@ -487,8 +490,7 @@ public abstract class AbstractBrokerFactory
         else
             bv = (BrokerValue) _conf.getValue(BrokerValue.KEY);
 
-        if (FinalizingBrokerImpl.class.isAssignableFrom(
-            bv.getTemplateBrokerType(_conf))) {
+        if (FinalizingBrokerImpl.class.isAssignableFrom(bv.getTemplateBrokerType(_conf))) {
             return MapBackedSet.decorate(new ConcurrentHashMap(), new Object() { });
         } else {
             return new ConcurrentReferenceHashSet<Broker>(ConcurrentReferenceHashSet.WEAK);
@@ -598,6 +600,7 @@ public abstract class AbstractBrokerFactory
         broker.setMultithreaded(_conf.getMultithreaded());
         broker.setAutoDetach(_conf.getAutoDetachConstant());
         broker.setDetachState(_conf.getDetachStateInstance().getDetachState());
+        broker.setPostLoadOnMerge(_conf.getPostLoadOnMerge());
     }
 
     /**
@@ -624,7 +627,9 @@ public abstract class AbstractBrokerFactory
                 Map.Entry<?,?> entry;
                 for (Iterator<Map.Entry<String,Object>> itr = props.entrySet().iterator(); itr.hasNext();) {
                     entry = itr.next();
-                    buf.append(entry.getKey()).append(": ").append(entry.getValue());
+                    Object value = entry.getValue();
+                    buf.append(entry.getKey()).append(": ")
+                       .append(value != null && value.getClass().isArray() ? Arrays.toString((Object[])value) : value);
                     if (itr.hasNext())
                         buf.append(lineSep);
                 }
@@ -848,5 +853,40 @@ public abstract class AbstractBrokerFactory
         
         return dsm;
     }
+    
+    /**
+     * This method is invoked AFTER a BrokerFactory has been instantiated. 
+     */
+    public void postCreationCallback() {
+    	Auditor auditor = _conf.getAuditorInstance();
+    	if (auditor != null) {
+    		addTransactionListener(new AuditManager(auditor));
+    	}
+        if (_conf.isInitializeEagerly()) {
+            newBroker(_conf.getConnectionUserName(), _conf.getConnectionPassword(),
+                _conf.isConnectionFactoryModeManaged(), _conf.getConnectionRetainModeConstant(), false).close();
+        }
+        // Don't catch any exceptions here because we want to fail-fast if something bad happens when we're preloading.
+        Options o = Configurations.parseProperties(Configurations.getProperties(_conf.getMetaDataRepository()));
+        if (MetaDataRepository.needsPreload(o) == true) {
+            MetaDataRepository mdr = _conf.getMetaDataRepositoryInstance();
+            mdr.setValidate(MetaDataRepository.VALIDATE_RUNTIME, true);
+            mdr.setResolve(MetaDataRepository.MODE_MAPPING_INIT, true);
+
+            // Load persistent classes and hook in subclasser
+            loadPersistentTypes((ClassLoader) AccessController.doPrivileged(J2DoPrivHelper
+                .getContextClassLoaderAction()));
+            mdr.preload();
+        }
+
+        // Get a DataCacheManager instance up front to avoid threading concerns on first call.
+        // _conf.getDataCacheManagerInstance();
+
+        InstrumentationManager imgr = _conf.getInstrumentationManagerInstance();
+        if (imgr != null) {
+            // Start all factory level instrumentation
+            imgr.start(InstrumentationLevel.FACTORY, this);
+        }
+    }
 }
 
