@@ -281,6 +281,9 @@ public class DetachedValueStateManager
         if (sm != null) {
             if (sm instanceof DetachedStateManager)
                 return fetchFromDetachedSM((DetachedStateManager) sm, field);
+            if (_ctx.getAllowReferenceToSiblingContext() && sm instanceof StateManagerImpl) {
+                return ((StateManagerImpl) sm).fetch(field);
+            }
             throw new UnsupportedException(_loc.get("detach-val-badsm", _pc));
         }
         provideField(field);
@@ -309,19 +312,19 @@ public class DetachedValueStateManager
                 return (fm.fetchBooleanField(field)) ? Boolean.TRUE
                     : Boolean.FALSE;
             case JavaTypes.BYTE:
-                return new Byte(fm.fetchByteField(field));
+                return Byte.valueOf(fm.fetchByteField(field));
             case JavaTypes.CHAR:
-                return new Character(fm.fetchCharField(field));
+                return Character.valueOf(fm.fetchCharField(field));
             case JavaTypes.DOUBLE:
-                return new Double(fm.fetchDoubleField(field));
+                return Double.valueOf(fm.fetchDoubleField(field));
             case JavaTypes.FLOAT:
-                return new Float(fm.fetchFloatField(field));
+                return Float.valueOf(fm.fetchFloatField(field));
             case JavaTypes.INT:
                 return fm.fetchIntField(field);
             case JavaTypes.LONG:
                 return fm.fetchLongField(field);
             case JavaTypes.SHORT:
-                return new Short(fm.fetchShortField(field));
+                return Short.valueOf(fm.fetchShortField(field));
             default:
                 return fm.fetchObjectField(field);
         }
@@ -592,5 +595,20 @@ public class DetachedValueStateManager
     public Object replaceObjectField(PersistenceCapable pc, int idx) {
         throw new UnsupportedOperationException();
     }
+    
+    @Override
+    public boolean isDelayed(int field) {
+        return false;
+    }
+    
+    @Override
+    public void setDelayed(int field, boolean delay) {
+        throw new UnsupportedOperationException();
+    }
+    
+    @Override
+    public void loadDelayedField(int field) {
+        throw new UnsupportedOperationException();
+    }
 }
 
