@@ -28,14 +28,19 @@ import java.util.Collection;
 import java.util.Date;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.TimeZone;
 
+import org.apache.openjpa.conf.Compatibility;
+import org.apache.openjpa.enhance.PersistenceCapable;
 import org.apache.openjpa.lib.util.Localizer;
 import org.apache.openjpa.meta.FieldMetaData;
 import org.apache.openjpa.meta.JavaTypes;
 import org.apache.openjpa.meta.ValueMetaData;
 import org.apache.openjpa.util.ChangeTracker;
 import org.apache.openjpa.util.Exceptions;
+import org.apache.openjpa.util.ImplHelper;
 import org.apache.openjpa.util.InvalidStateException;
+import org.apache.openjpa.util.LRSProxy;
 import org.apache.openjpa.util.MapChangeTracker;
 import org.apache.openjpa.util.ObjectId;
 import org.apache.openjpa.util.Proxies;
@@ -47,22 +52,19 @@ import org.apache.openjpa.util.UserException;
  * FieldManager type used to hold onto a single field value and then
  * dispense it via the fetch methods. The manager can also perform actions
  * on the held field.
- *
- * @author Abe White
  */
-class SingleFieldManager
-    extends TransferFieldManager
-    implements Serializable {
-
-    private static final Localizer _loc = Localizer.forPackage
-        (SingleFieldManager.class);
+class SingleFieldManager extends TransferFieldManager implements Serializable {
+    private static final Localizer _loc = Localizer.forPackage(SingleFieldManager.class);
 
     private final StateManagerImpl _sm;
     private final BrokerImpl _broker;
-
+    private final boolean _checkDbOnCascadePersist; 
+    
     public SingleFieldManager(StateManagerImpl sm, BrokerImpl broker) {
         _sm = sm;
         _broker = broker;
+        _checkDbOnCascadePersist =
+            _broker.getConfiguration().getCompatibilityInstance().getCheckDatabaseForCascadePersistToDetachedEntity();
     }
 
     /**
@@ -77,31 +79,34 @@ class SingleFieldManager
             case JavaTypes.DATE:
                 if (objval == null)
                     return false;
-                proxy = checkProxy();
+                proxy = checkProxy(fmd);
                 if (proxy == null) {
                     proxy = (Proxy) _sm.newFieldProxy(field);
                     ((Date) proxy).setTime(((Date) objval).getTime());
-                    if (proxy instanceof Timestamp 
-                        && objval instanceof Timestamp)
-                        ((Timestamp) proxy).setNanos(((Timestamp) objval).
-                            getNanos());
+                    if (proxy instanceof Timestamp && objval instanceof Timestamp)
+                        ((Timestamp) proxy).setNanos(((Timestamp) objval).getNanos());
                     ret = true;
                 }
                 break;
             case JavaTypes.CALENDAR:
                 if (objval == null)
                     return false;
-                proxy = checkProxy();
+                proxy = checkProxy(fmd);
                 if (proxy == null) {
                     proxy = (Proxy) _sm.newFieldProxy(field);
                     ((Calendar) proxy).setTime(((Calendar) objval).getTime());
                     ret = true;
+                } else {
+                    Object init = fmd.getInitializer();
+                    if (init != null && init instanceof TimeZone) {
+                        ((Calendar) proxy).setTimeZone((TimeZone)init);
+                    }
                 }
                 break;
             case JavaTypes.COLLECTION:
                 if (objval == null && !replaceNull)
                     return false;
-                proxy = checkProxy();
+                proxy = checkProxy(fmd);
                 if (proxy == null) {
                     proxy = (Proxy) _sm.newFieldProxy(field);
                     if (objval != null)
@@ -112,7 +117,7 @@ class SingleFieldManager
             case JavaTypes.MAP:
                 if (objval == null && !replaceNull)
                     return false;
-                proxy = checkProxy();
+                proxy = checkProxy(fmd);
                 if (proxy == null) {
                     proxy = (Proxy) _sm.newFieldProxy(field);
                     if (objval != null)
@@ -123,7 +128,7 @@ class SingleFieldManager
             case JavaTypes.OBJECT:
                 if (objval == null)
                     return false;
-                proxy = checkProxy();
+                proxy = checkProxy(fmd);
                 if (proxy == null) {
                     proxy = getProxyManager().newCustomProxy(objval,
                         _sm.getBroker().getConfiguration().
@@ -150,15 +155,21 @@ class SingleFieldManager
     }
 
     /**
-     * If the current field is a usable proxy, return it; else return null.
+     * If the current field is a usable proxy and it should be a proxy, return it; else return null.
+     * 
+     * This method will skim out Calendar instances that were proxied before we knew if they need to be proxied.
      */
-    private Proxy checkProxy() {
+    private Proxy checkProxy(FieldMetaData fmd) {
         if (!(objval instanceof Proxy))
             return null;
 
         Proxy proxy = (Proxy) objval;
-        if (proxy.getOwner() == null || Proxies.isOwner(proxy, _sm, field))
-            return proxy;
+        if (proxy.getOwner() == null || Proxies.isOwner(proxy, _sm, field)) {
+            if (fmd.getProxyType().isAssignableFrom(proxy.getClass()) 
+                    || (fmd.isLRS() && (objval instanceof LRSProxy))) {
+                return proxy;
+            }
+        }
         return null;
     }
 
@@ -309,8 +320,11 @@ class SingleFieldManager
             // works on external value
             if ((immediate || fmd.isEmbeddedPC())
                 && fmd.getCascadeDelete() == ValueMetaData.CASCADE_IMMEDIATE) {
-                if (fmd.isEmbeddedPC() && deref)
-                    dereferenceEmbedDependent(_broker.getStateManagerImpl(objval, false));
+                if (fmd.isEmbeddedPC() && deref) {
+                    StateManagerImpl sm = _broker.getStateManagerImpl(objval, false);
+                    if (sm != null) 
+                        dereferenceEmbedDependent(sm);
+                }
                 delete(fmd, objval, call);
             }
             else if (fmd.getCascadeDelete() == ValueMetaData.CASCADE_AUTO)
@@ -751,26 +765,35 @@ class SingleFieldManager
     /**
      * Perform pre flush operations on the given object.
      */
-    private void preFlushPC(ValueMetaData vmd, Object obj, boolean logical,
-        OpCallbacks call) {
+    private void preFlushPC(ValueMetaData vmd, Object obj, boolean logical, OpCallbacks call) {
         if (obj == null)
             return;
 
         OpenJPAStateManager sm;        
 
         if (vmd.getCascadePersist() == ValueMetaData.CASCADE_NONE) {
-            if (!_broker.isDetachedNew() && _broker.isDetached(obj))
+            if (!_broker.isDetachedNew() && _broker.isDetached(obj, _checkDbOnCascadePersist)) {
                 return; // allow but ignore
+            }
 
             sm = _broker.getStateManager(obj);
-            if (sm == null || !sm.isPersistent())
-                throw new InvalidStateException(
-                    _loc.get("cant-cascade-persist", vmd))
+            if (sm == null || !sm.isPersistent()) {
+                if (((StoreContext)_broker).getAllowReferenceToSiblingContext() 
+                 && ImplHelper.isManageable(obj) 
+                 && ((PersistenceCapable)obj).pcGetStateManager() != null) {
+                    return; 
+                } else {
+                    throw new InvalidStateException(_loc.get("cant-cascade-persist", 
+                            vmd.toString(), Exceptions.toString(obj),
+                            sm == null ? " unmanaged" : sm.getPCState().getClass().getSimpleName()))
                     .setFailedObject(obj);
+                }
+            }
         } else {
             if (vmd.getCascadePersist() == ValueMetaData.CASCADE_IMMEDIATE) {
-                if (!_broker.isDetachedNew() && _broker.isDetached(obj))
+                if (!_broker.isDetachedNew() && _broker.isDetached(obj, _checkDbOnCascadePersist)) {
                     return; // allow but ignore
+                }
             }        	
             sm = _broker.getStateManager(obj);
             if (sm == null || !sm.isProvisional()) { 
@@ -778,6 +801,12 @@ class SingleFieldManager
                 // ensure generated IDs get assigned properly
                 if (!logical)
                     ((StateManagerImpl)sm).assignObjectId(false, true);
+                
+                // Call preFetch on this and any related persistent fields.
+                // This will ensure IDs get assigned to those that need them.
+                if (_broker.isFlushing()) {
+                    ((StateManagerImpl)sm).preFlush(logical, call);
+                }
             }
         }
 
