@@ -73,6 +73,15 @@ public interface StoreContext {
      */
     public FetchConfiguration pushFetchConfiguration();
 
+    /**
+     * Pushes the fetch configuration argument onto a stack, and makes the new configuration
+     * the active one.
+     *
+     * @since 2.1.1
+     * @return the new fetch configuration
+     */
+    public FetchConfiguration pushFetchConfiguration(FetchConfiguration fc);
+
     /**
      * Pops the fetch configuration from the top of the stack, making the
      * next one down the active one. This returns void to avoid confusion,
@@ -500,4 +509,44 @@ public interface StoreContext {
      * @since 2.0.0. 
      */
     public boolean isCached(List<Object> oid);
+    
+    /**
+     * Affirms if this context will allow its managed instances to refer instances 
+     * that are managed by other contexts. 
+     * <B>Note</B>: Some specification (such as JPA) does not warranty predictable
+     * behavior when strict group-like property of a persistent context (where managed 
+     * instances can only refer to instances managed by the <em>same</em> context).
+     * Please be aware of consequences when the flag is set to true.
+     * 
+     * @since 2.1
+     */
+    public void setAllowReferenceToSiblingContext(boolean flag);
+    
+    /**
+     * Affirms if this context will allow its managed instances to refer instances 
+     * that are managed by other contexts. 
+     * 
+     * @return false by default.
+     * 
+     * @since 2.1
+     */
+    public boolean getAllowReferenceToSiblingContext();
+
+
+    /**
+     * Set to <code>true</code> if the merge operation should trigger
+     * a &#064;PostLoad lifecycle event.
+     * @param allow PostLoad lifecycle events to be triggered on a merge operation
+     */
+    public void setPostLoadOnMerge(boolean allow);
+
+    /**
+     * Force sending a &#064;PostLoad lifecycle event while merging.
+     *
+     * @return <code>false</code> by default
+     *
+     * @since 2.2
+     */
+    public boolean getPostLoadOnMerge();
+
 }
