@@ -18,6 +18,7 @@
  */
 package org.apache.openjpa.meta;
 
+import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
@@ -44,6 +45,7 @@ import org.apache.openjpa.conf.OpenJPAConfigurationImpl;
 import org.apache.openjpa.lib.meta.ClassArgParser;
 import org.apache.openjpa.lib.meta.ClasspathMetaDataIterator;
 import org.apache.openjpa.lib.meta.FileMetaDataIterator;
+import org.apache.openjpa.lib.meta.JarFileURLMetaDataIterator;
 import org.apache.openjpa.lib.meta.MetaDataFilter;
 import org.apache.openjpa.lib.meta.MetaDataIterator;
 import org.apache.openjpa.lib.meta.MetaDataParser;
@@ -381,10 +383,11 @@ public abstract class AbstractCFMetaDataFactory
             if (queries[i].getSourceMode() == MODE_QUERY
                 || (mode & queries[i].getSourceMode()) == 0)
                 continue;
-            if (queries[i].getSourceFile() == null)
-                queries[i].setSource(defaultSourceFile(queries[i],
-                    clsNames), queries[i].getSourceScope(),
-                    queries[i].getSourceType());
+            if (queries[i].getSourceFile() == null) {
+                File defaultFile = defaultSourceFile(queries[i], clsNames);
+                queries[i].setSource(defaultFile, queries[i].getSourceScope(), queries[i].getSourceType(),
+                    defaultFile == null ? "" : defaultFile.getPath());
+            }
             if ((AccessController.doPrivileged(
                 J2DoPrivHelper.existsAction(queries[i].getSourceFile())))
                 .booleanValue()) {
@@ -423,9 +426,11 @@ public abstract class AbstractCFMetaDataFactory
         for (int i = 0; i < queries.length; i++) {
             if (queries[i].getSourceMode() != MODE_QUERY)
                 continue;
-            if (queries[i].getSourceFile() == null)
-                queries[i].setSource(defaultSourceFile(queries[i], clsNames),
-                    queries[i].getSourceScope(), queries[i].getSourceType());
+            if (queries[i].getSourceFile() == null) {
+                File defaultFile = defaultSourceFile(queries[i], clsNames);
+                queries[i].setSource(defaultFile, queries[i].getSourceScope(), queries[i].getSourceType(),
+                    defaultFile == null ? "" : defaultFile.getPath());
+            }
             if ((AccessController.doPrivileged(
                 J2DoPrivHelper.existsAction(queries[i].getSourceFile())))
                 .booleanValue()) {
@@ -522,7 +527,8 @@ public abstract class AbstractCFMetaDataFactory
      * Set the current source file of the given metadata.
      */
     protected void setSourceFile(ClassMetaData meta, File sourceFile) {
-        meta.setSource(sourceFile, meta.getSourceType());
+        meta.setSource(sourceFile, meta.getSourceType(), sourceFile != null ? 
+            sourceFile.getPath() : "");
     }
 
     /**
@@ -699,34 +705,55 @@ public abstract class AbstractCFMetaDataFactory
                         continue;
                     }
                 }
-                if ("jar".equals(url.getProtocol())
-                    && url.getPath().endsWith("!/")) {
-                    if (log.isTraceEnabled())
-                        log.trace(_loc.get("scanning-jar-url", url));
-                    scan(new ZipFileMetaDataIterator(url,
-                        newMetaDataFilter()), cparser, names, true, url);
-                } else if (url.getPath().endsWith(".jar")) {
-                    if (log.isTraceEnabled())
-                        log.trace(_loc.get("scanning-jar-at-url", url));
-                    try {
-                        InputStream is = (InputStream)
-                            AccessController.doPrivileged(
-                                J2DoPrivHelper.openStreamAction(url));
-                        scan(new ZipStreamMetaDataIterator(
-                            new ZipInputStream(is),
+                if ("jar".equals(url.getProtocol())) {
+                    if (url.getPath().endsWith("!/")) {
+                        if (log.isTraceEnabled())
+                            log.trace(_loc.get("scanning-jar-url", url));
+                        scan(new ZipFileMetaDataIterator(url,
                             newMetaDataFilter()), cparser, names, true, url);
+                    } else {
+                        if (log.isTraceEnabled())
+                            log.trace(_loc.get("scanning-jar-url", url));
+                        scan(new JarFileURLMetaDataIterator(url,
+                            newMetaDataFilter()), cparser, names, true, url);
+                    }                   
+                } else {
+                    // Open an InputStream from the URL and sniff for a zip header.  If it is, then this is
+                    // a URL with a jar-formated InputStream, as per the JPA specification.  Otherwise, fall back
+                    // to URLMetaDataIterator.
+                    BufferedInputStream is = null;                    
+                    try {
+                        is = new BufferedInputStream((InputStream) AccessController.
+                                doPrivileged(J2DoPrivHelper.openStreamAction(url)));
                     } catch (PrivilegedActionException pae) {
                         throw (IOException) pae.getException();
                     }
-                } else {
-                    if (log.isTraceEnabled())
-                        log.trace(_loc.get("scanning-url", url));
-                    clss = cparser.parseTypeNames(new URLMetaDataIterator(url));
-                    List<String> newNames = Arrays.asList(clss);
-                    if (log.isTraceEnabled())
-                        log.trace(_loc.get("scan-found-names", newNames, url));
-                    names.addAll(newNames);
-                    mapPersistentTypeNames(url, clss);
+                    
+                    // Check for zip header magic 0x50 0x4b 0x03 0x04
+                    is.mark(0);
+                    boolean zipHeaderMatch = is.read() == 0x50 && is.read() == 0x4b && is.read() == 0x03 && 
+                            is.read() == 0x04;
+                    is.reset();
+                    
+                    if (zipHeaderMatch) {
+                        // The URL provides a Jar-formatted InputStream, consume it with ZipStreamMetaDataIterator
+                        if (log.isTraceEnabled())
+                            log.trace(_loc.get("scanning-jar-at-url", url));
+                        scan(new ZipStreamMetaDataIterator(new ZipInputStream(is), newMetaDataFilter()), 
+                                cparser, names, true, url);
+                    } else {
+                        // Fall back to URLMetaDataIterator
+                        if (log.isTraceEnabled())
+                            log.trace(_loc.get("scanning-url", url));
+                        
+                        is.close();
+                        clss = cparser.parseTypeNames(new URLMetaDataIterator(url));
+                        List<String> newNames = Arrays.asList(clss);
+                        if (log.isTraceEnabled())
+                            log.trace(_loc.get("scan-found-names", newNames, url));
+                        names.addAll(newNames);
+                        mapPersistentTypeNames(url, clss);
+                    }
                 }
             }
         }
@@ -768,7 +795,18 @@ public abstract class AbstractCFMetaDataFactory
                     String puUrlString = puUrl == null ? null : puUrl.toString();
                     if (log.isTraceEnabled())
                         log.trace(_loc.get("pu-root-url", puUrlString));
-
+					
+                    URL puORMUrl = null;
+                    try {
+                        if (puUrlString != null) {
+                            String puORMUrlStr = puUrlString + (puUrlString.endsWith("/") ? "" : "/") + rsrc;
+                            puORMUrl = AccessController.doPrivileged(J2DoPrivHelper.createURL(puORMUrlStr));
+                        }
+                    } catch (PrivilegedActionException e) {
+                        throw new IOException("Error generating puORMUrlStr.",
+                                e.getCause());
+                    }
+					
                     List<URL> urls = new ArrayList<URL>(3);
                     while (mitr.hasNext()) {
                         url = (URL) mitr.next();
@@ -776,8 +814,14 @@ public abstract class AbstractCFMetaDataFactory
                         if (log.isTraceEnabled())
                             log.trace(_loc.get("resource-url", urlString));
                         if (peMap != null) {
-                            if (puUrlString != null && urlString.indexOf(puUrlString) != -1) 
+                            //OPENJPA-2102: decode the URL to remove such things a spaces (' ') encoded as '%20'
+                            if (puUrlString != null && decode(urlString).indexOf(decode(puUrlString)) != -1) {
                                 urls.add(url);
+                            } else if (puORMUrl != null && puORMUrl.equals(url)) {
+                                // Check URL equality to support encapsulating URL protocols
+                                urls.add(url);
+                            }
+
                             if (mappingFileNames != null && mappingFileNames.size() != 0) {
                                 for (String mappingFileName : mappingFileNames) {
                                     if (log.isTraceEnabled())
@@ -854,6 +898,66 @@ public abstract class AbstractCFMetaDataFactory
             names.addAll(newNames);
         }
     }
+    
+    /**
+     * Decodes a URL-encoded path string.  For example, an encoded
+     * space (%20) is decoded into a normal space (' ') character.
+     * Added via OPENJPA-2102.
+     * @param String encoded - the encoded URL string
+     * @return String decoded - the decoded string.
+     */
+    public static String decode(String s) {
+       if (s == null) {
+          return null;
+       }
+
+       int i = s.indexOf('%');
+       if (i == -1) {
+          return s;
+       }
+
+       StringBuilder builder = new StringBuilder();
+       int begin = 0;
+
+       do {
+          builder.append(s, begin, i);
+          begin = i + 3;
+
+          char ch = (char) Integer.parseInt(s.substring(i + 1, begin), 16);
+
+          if ((ch & 0x80) != 0) {
+             // Decode "modified UTF-8".
+
+             if (s.charAt(begin++) != '%') {
+                throw new IllegalArgumentException();
+             }
+
+             char ch2 = (char) Integer.parseInt(s.substring(begin, begin + 2), 16);
+             begin += 2;
+
+             if ((ch & 0xe0) == 0xc0) {
+                ch = (char) (((ch & 0x1f) << 6) | (ch2 & 0x3f));
+             } else if ((ch & 0xf0) == 0xe0) {
+                if (s.charAt(begin++) != '%') {
+                   throw new IllegalArgumentException();
+                }
+
+                char ch3 = (char) Integer.parseInt(s.substring(begin, begin + 2), 16);
+                begin += 2;
+
+                ch = (char) (((ch & 0x0f) << 12) | ((ch2 & 0x3f) << 6) | (ch3 & 0x3f));
+             } else {
+                throw new IllegalArgumentException();
+             }
+          }
+
+          builder.append(ch);
+       } while ((i = s.indexOf('%', begin)) != -1);
+
+       builder.append(s, begin, s.length());
+
+       return builder.toString();
+    }
 
     /**
      * Implement this method to map metadata resources to the persistent
