@@ -23,14 +23,11 @@ import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.security.AccessController;
-import java.sql.Timestamp;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Calendar;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.Date;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -41,6 +38,7 @@ import java.util.TreeMap;
 
 import org.apache.commons.lang.StringUtils;
 import org.apache.openjpa.conf.OpenJPAConfiguration;
+import org.apache.openjpa.datacache.CacheDistributionPolicy;
 import org.apache.openjpa.datacache.DataCache;
 import org.apache.openjpa.enhance.PCRegistry;
 import org.apache.openjpa.enhance.PersistenceCapable;
@@ -54,6 +52,7 @@ import org.apache.openjpa.lib.util.Localizer;
 import org.apache.openjpa.lib.xml.Commentable;
 import org.apache.openjpa.util.BigDecimalId;
 import org.apache.openjpa.util.BigIntegerId;
+import org.apache.openjpa.util.BooleanId;
 import org.apache.openjpa.util.ByteId;
 import org.apache.openjpa.util.CharId;
 import org.apache.openjpa.util.DateId;
@@ -67,7 +66,6 @@ import org.apache.openjpa.util.LongId;
 import org.apache.openjpa.util.MetaDataException;
 import org.apache.openjpa.util.ObjectId;
 import org.apache.openjpa.util.OpenJPAId;
-import org.apache.openjpa.util.Proxy;
 import org.apache.openjpa.util.ShortId;
 import org.apache.openjpa.util.StringId;
 import org.apache.openjpa.util.UnsupportedException;
@@ -147,6 +145,7 @@ public class ClassMetaData
     private final ValueMetaData _owner;
     private final LifecycleMetaData _lifeMeta = new LifecycleMetaData(this);
     private File _srcFile = null;
+    private String _srcName = null;
     private int _srcType = SRC_OTHER;
     private int _lineNum = 0;  
     private int _colNum = 0;  
@@ -157,6 +156,7 @@ public class ClassMetaData
 
     private Class<?> _type = Object.class;
     private int _hashCode = Object.class.getName().hashCode();
+    private String _typeString = Object.class.getName();
     private final Map<String,FieldMetaData> _fieldMap = new TreeMap<String,FieldMetaData>();
     private Map<String,FieldMetaData> _supFieldMap = null;
     private boolean _defSupFields = false;
@@ -182,7 +182,6 @@ public class ClassMetaData
     private int _identity = ID_UNKNOWN;
     private int _idStrategy = ValueStrategies.NONE;
     private int _accessType = AccessCode.UNKNOWN;
-    private boolean _replicated = false;
     
     private String _seqName = DEFAULT_STRING;
     private SequenceMetaData _seqMeta = null;
@@ -220,7 +219,11 @@ public class ClassMetaData
     private boolean _abstract = false;
     private Boolean _hasAbstractPKField = null;
     private Boolean _hasPKFieldsFromAbstractClass = null;
-    
+    private int[] _pkAndNonPersistentManagedFmdIndexes = null;
+    private Boolean inverseManagedFields = null;
+    private List<FieldMetaData> _mappedByIdFields;
+    private boolean _mappedByIdFieldsSet = false;
+
     /**
      * Constructor. Supply described type and repository.
      */
@@ -262,7 +265,14 @@ public class ClassMetaData
     public Class<?> getDescribedType() {
         return _type;
     }
-
+    
+    /**
+     * The persistence capable stringified class described by this metadata.
+     */
+    public String getDescribedTypeString(){
+        return _typeString;
+    }
+    
     /**
      * Set the class described by this metadata. The type may be reset when
      * an embedded value changes its declared type.
@@ -272,7 +282,8 @@ public class ClassMetaData
             (type.getSuperclass().getName()))
             throw new MetaDataException(_loc.get("enum", type));
         _type = type;
-        _hashCode = _type.getName().hashCode();
+        _typeString = _type.getName();
+        _hashCode = _typeString.hashCode();
         if (PersistenceCapable.class.isAssignableFrom(type))
             setIntercepting(true);
     }
@@ -535,6 +546,10 @@ public class ClassMetaData
             case JavaTypes.BIGINTEGER:
                 _objectId = BigIntegerId.class;
                 break;
+            case JavaTypes.BOOLEAN:
+            case JavaTypes.BOOLEAN_OBJ:
+                _objectId = BooleanId.class;
+                break;
         }
         return _objectId;
     }
@@ -2285,36 +2300,36 @@ public class ClassMetaData
      * Return the fetch groups declared explicitly in this type.
      */
     public FetchGroup[] getDeclaredFetchGroups() {
-        if (_fgs == null)
-            _fgs = (_fgMap == null) ? EMPTY_FETCH_GROUP_ARRAY : (FetchGroup[])
-                _fgMap.values().toArray(new FetchGroup[_fgMap.size()]); 
+        if (_fgs == null) {
+            _fgs = (_fgMap == null) ? EMPTY_FETCH_GROUP_ARRAY : _fgMap.values().toArray(new FetchGroup[_fgMap.size()]); 
+        }
         return _fgs;
     }
 
     /**
-     * Return all fetch groups for this type, including superclass groups.
+     * Return all fetch groups for this type, including superclass groups but excluding the standard groups
+     * such as "default" or "all".
      */
     public FetchGroup[] getCustomFetchGroups() {
         if (_customFGs == null) {
             // map fetch groups to names, allowing our groups to override super
             Map<String,FetchGroup> fgs = new HashMap<String,FetchGroup>();
             ClassMetaData sup = getPCSuperclassMetaData();
-            if (sup != null)
-            {
+            if (sup != null) {
                 FetchGroup[] supFGs = sup.getCustomFetchGroups();
-                for (int i = 0; i < supFGs.length; i++)
+                for (int i = 0; i < supFGs.length; i++) {
                     fgs.put(supFGs[i].getName(), supFGs[i]);
+                }
             }
             FetchGroup[] decs = getDeclaredFetchGroups();
-            for (int i = 0; i < decs.length; i++)
+            for (int i = 0; i < decs.length; i++) {
                 fgs.put(decs[i].getName(), decs[i]);
-            
+            }
             // remove standard groups
             fgs.remove(FetchGroup.NAME_DEFAULT);
             fgs.remove(FetchGroup.NAME_ALL);
 
-            _customFGs = (FetchGroup[]) fgs.values().toArray
-                (new FetchGroup[fgs.size()]);
+            _customFGs = fgs.values().toArray(new FetchGroup[fgs.size()]);
         }
         return _customFGs;
     }
@@ -2328,7 +2343,7 @@ public class ClassMetaData
      * receiver or any of its superclasses. Otherwise null.
      */
     public FetchGroup getFetchGroup(String name) {
-        FetchGroup fg = (_fgMap == null) ? null : (FetchGroup) _fgMap.get(name);
+        FetchGroup fg = (_fgMap == null) ? null : _fgMap.get(name);
         if (fg != null)
             return fg;
         ClassMetaData sup = getPCSuperclassMetaData();
@@ -2353,7 +2368,7 @@ public class ClassMetaData
     		throw new MetaDataException(_loc.get("empty-fg-name", this));
         if (_fgMap == null)
             _fgMap = new HashMap<String,FetchGroup>();
-        FetchGroup fg = (FetchGroup) _fgMap.get(name);
+        FetchGroup fg = _fgMap.get(name);
         if (fg == null) {
         	fg = new FetchGroup(this, name);
         	_fgMap.put(name, fg);
@@ -2393,9 +2408,10 @@ public class ClassMetaData
         return _srcType;
     }
 
-    public void setSource(File file, int srcType) {
+    public void setSource(File file, int srcType, String srcName) {
         _srcFile = file;
         _srcType = srcType;
+        _srcName = srcName;
     }
 
     public String getResourceName() {
@@ -2646,21 +2662,6 @@ public class ClassMetaData
     	return result.toArray(new String[result.size()]);
     }
     
-    /**
-     * Affirms the persistence instances of this receiver is replicated across
-     * multiple databases.
-     */
-    public boolean isReplicated() {
-    	return _replicated;
-    }
-    
-    /**
-     * Sets the persistence instances of this receiver to be replicated across
-     * multiple databases.
-     */
-    public void setReplicated(boolean flag) {
-    	_replicated = flag;
-    }
 
     public boolean isAbstract() {
         return _abstract;
@@ -2762,5 +2763,59 @@ public class ClassMetaData
             return _cacheEnabled;
         return getPCSuperclassMetaData() != null ?  getPCSuperclassMetaData().getCacheEnabled() : null; 
     }
-}
+    
+    public String getSourceName(){
+        return _srcName; 
+    }
+    
+    public int[] getPkAndNonPersistentManagedFmdIndexes() {
+        if (_pkAndNonPersistentManagedFmdIndexes == null) {
+            List<Integer> ids = new ArrayList<Integer>();
+            for (FieldMetaData fmd : getFields()) {
+                if (fmd.isPrimaryKey() || fmd.getManagement() != FieldMetaData.MANAGE_PERSISTENT) {
+                    ids.add(fmd.getIndex());
+                }
+            }
+            int idsSize = ids.size();
+            _pkAndNonPersistentManagedFmdIndexes = new int[idsSize];
+            for(int i = 0; i<idsSize; i++){
+                _pkAndNonPersistentManagedFmdIndexes[i] = ids.get(i).intValue();
+            }
+        }
+        return _pkAndNonPersistentManagedFmdIndexes;
+    }
 
+    public boolean hasInverseManagedFields() {
+        if (inverseManagedFields == null) {
+            Boolean res = Boolean.FALSE;
+            for (FieldMetaData fmd : getFields()) {
+                if (fmd.getInverseMetaDatas().length > 0) {
+                    res = Boolean.TRUE;
+                    break;
+                }
+            }
+            inverseManagedFields = res;
+        }
+        return inverseManagedFields.booleanValue();
+    }
+    
+    public List<FieldMetaData> getMappyedByIdFields() {
+        if (!_mappedByIdFieldsSet) {
+            List<FieldMetaData> fmdArray = null;
+            for (FieldMetaData fmd : getFields()) {
+                if (getIdentityType() == ClassMetaData.ID_APPLICATION) {
+                    String mappedByIdValue = fmd.getMappedByIdValue();
+                    if (mappedByIdValue != null) {
+                        if (fmdArray == null) {
+                            fmdArray = new ArrayList<FieldMetaData>();
+                        }
+                        fmdArray.add(fmd);
+                    }
+                }
+            }
+            _mappedByIdFields = fmdArray;
+            _mappedByIdFieldsSet = true;
+        }
+        return _mappedByIdFields;
+    }
+}
