@@ -98,7 +98,7 @@ public class ApplicationIds {
         // default to reflection
         if (meta.isObjectIdTypeShared())
             oid = ((ObjectId) oid).getId();
-        Class oidType = oid.getClass();
+        Class<?> oidType = oid.getClass();
         for (int i = 0; i < fmds.length; i++) {
             if (AccessCode.isField(meta.getAccessType()))
                 pks[i] = Reflection.get(oid, Reflection.findField(oidType, 
@@ -203,6 +203,12 @@ public class ApplicationIds {
                             "!(x instanceof BigInteger)");
                     return new BigIntegerId(meta.getDescribedType(), 
                         (BigInteger)val);
+                case JavaTypes.BOOLEAN:
+                case JavaTypes.BOOLEAN_OBJ:
+                    if (!convert && !(val instanceof Boolean))
+                        throw new ClassCastException("!(x instanceof Boolean)");
+                    return new BooleanId(meta.getDescribedType(), 
+                        val == null ? false : (Boolean)val);
                 default:
                     throw new InternalException();
             }
@@ -221,7 +227,7 @@ public class ApplicationIds {
         }
 
         // default to reflection
-        Class oidType = meta.getObjectIdType();
+        Class<?> oidType = meta.getObjectIdType();
         if (Modifier.isAbstract(oidType.getModifiers()))
             throw new UserException(_loc.get("objectid-abstract", meta));
         Object copy = null;
@@ -261,7 +267,7 @@ public class ApplicationIds {
 
         if (meta.isOpenJPAIdentity()) {
             // use meta type instead of oid type in case it's a subclass
-            Class cls = meta.getDescribedType();
+            Class<?> cls = meta.getDescribedType();
             OpenJPAId koid = (OpenJPAId) oid;
             FieldMetaData pk = meta.getPrimaryKeyFields()[0];
             switch (pk.getObjectIdFieldTypeCode()) {
@@ -324,7 +330,7 @@ public class ApplicationIds {
         // oid instance
         if (!Modifier.isAbstract(meta.getDescribedType().getModifiers())
             && !hasPCPrimaryKeyFields(meta)) {
-            Class type = meta.getDescribedType();
+            Class<?> type = meta.getDescribedType();
             PersistenceCapable pc = PCRegistry.newInstance(type, null, oid, 
                  false);
             Object copy = pc.pcNewObjectIdInstance();
@@ -361,7 +367,7 @@ public class ApplicationIds {
         if (oid == null)
             return null;
 
-        Class oidType = oid.getClass();
+        Class<?> oidType = oid.getClass();
         Object copy = null;
         try {
             copy = AccessController.doPrivileged(
@@ -402,7 +408,7 @@ public class ApplicationIds {
             return ((OpenJPAId) oid).getIdObject();
 
         ClassMetaData meta = fmd.getDefiningMetaData();
-        Class oidType = oid.getClass();
+        Class<?> oidType = oid.getClass();
         if (AccessCode.isField(meta.getAccessType()))
             return Reflection.get(oid, Reflection.findField(oidType, 
                 fmd.getName(), true));
@@ -480,9 +486,9 @@ public class ApplicationIds {
                 // If a value already exists on this field, throw exception.
                 // This is considered an application coding error.
                 if (!sm.isDefaultValue(pks[i].getIndex()))
-                    throw new InvalidStateException(_loc2.get(
-                            "existing-value-override-excep", pks[i]
-                                    .getFullName(false)));
+                    throw new InvalidStateException(_loc2.get("existing-value-override-excep", 
+                            pks[i].getFullName(false), Exceptions.toString(sm.getPersistenceCapable()),
+                            sm.getPCState().getClass().getSimpleName()));
                 // Assign the generated value
                 if (store.assignField(sm, pks[i].getIndex(), preFlush))
                     pks[i].setValueGenerated(true);
@@ -506,7 +512,7 @@ public class ApplicationIds {
         if (mappedByIdFieldName.length() != 0) {
             if (((ObjectId)id).getId() == null)
                 return false;        	
-            Class idClass = ((ObjectId)id).getId().getClass();
+            Class<?> idClass = ((ObjectId)id).getId().getClass();
             val = Reflection.get(key, 
                     Reflection.findField(idClass, mappedByIdFieldName, true)); 
         } else
@@ -569,6 +575,14 @@ public class ApplicationIds {
 
     }
 
+    /**
+     * Sets the underlying id of an ObjectId.  Should only
+     * be used with simple (idclass) types.
+     */
+    public static void setAppId(ObjectId id, Object newId) {
+        id.setId(newId);
+    }
+
     /**
      * Helper class used to transfer pk values to/from application oids.
      */
@@ -596,15 +610,15 @@ public class ApplicationIds {
         }
 
         public void storeByteField(int field, byte val) {
-            store(new Byte(val));
+            store(Byte.valueOf(val));
         }
 
         public void storeCharField(int field, char val) {
-            store(new Character(val));
+            store(Character.valueOf(val));
         }
 
         public void storeShortField(int field, short val) {
-            store(new Short(val));
+            store(Short.valueOf(val));
         }
 
         public void storeIntField(int field, int val) {
@@ -616,11 +630,11 @@ public class ApplicationIds {
         }
 
         public void storeFloatField(int field, float val) {
-            store(new Float(val));
+            store(Float.valueOf(val));
         }
 
         public void storeDoubleField(int field, double val) {
-            store(new Double(val));
+            store(Double.valueOf(val));
         }
 
         public void storeStringField(int field, String val) {
