@@ -50,15 +50,16 @@ import java.util.TreeSet;
 import java.util.Queue;
 
 import org.apache.commons.lang.StringUtils;
+import org.apache.openjpa.enhance.AsmAdaptor;
 import org.apache.openjpa.kernel.OpenJPAStateManager;
 import org.apache.openjpa.lib.util.Files;
 import org.apache.openjpa.lib.util.J2DoPrivHelper;
-import org.apache.openjpa.lib.util.JavaVersions;
 import org.apache.openjpa.lib.util.Localizer;
 import org.apache.openjpa.lib.util.Options;
 import org.apache.openjpa.lib.util.concurrent.NullSafeConcurrentHashMap;
 
-import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
+
 import serp.bytecode.BCClass;
 import serp.bytecode.BCField;
 import serp.bytecode.BCMethod;
@@ -97,6 +98,7 @@ public class ProxyManagerImpl
     private final Map _proxies = new NullSafeConcurrentHashMap();
     private boolean _trackChanges = true;
     private boolean _assertType = false;
+    private boolean _delayedCollectionLoading = false;
 
     public ProxyManagerImpl() {
         _unproxyable.add(TimeZone.class.getName());
@@ -137,7 +139,26 @@ public class ProxyManagerImpl
     public void setAssertAllowedType(boolean assertType) {
         _assertType = assertType;
     }
-
+    
+    /**
+     * Whether loading of collections should be delayed until an operation
+     * is performed that requires them to be loaded.  This property only
+     * applies to proxies that implement java.util.Collection (ie. not arrays
+     * or maps).  Defaults to false.
+     * @return 
+     */
+    public boolean getDelayCollectionLoading() {
+        return _delayedCollectionLoading;
+    }
+    
+    /**
+     * Whether loading of collections should be delayed until an operation
+     * is performed that requires them to be loaded.  Defaults to false.
+     */
+    public void setDelayCollectionLoading(boolean delay) {
+        _delayedCollectionLoading = delay;
+    }
+       
     /**
      * Return a mutable view of class names we know cannot be proxied  
      * correctly by this manager.
@@ -476,11 +497,43 @@ public class ProxyManagerImpl
      */
     protected Class loadBuildTimeProxy(Class type, ClassLoader loader) {
         try {
+            Class<?> proxyClass = null;
+            if (_delayedCollectionLoading) {
+                proxyClass = loadDelayedProxy(type);
+                if (proxyClass != null) {
+                    return proxyClass;
+                }
+            }
             return Class.forName(getProxyClassName(type, false), true, loader);
         } catch (Throwable t) {
             return null;
         }
     }
+    
+    protected Class<?> loadDelayedProxy(Class<?> type) {
+        if (type.equals(java.util.ArrayList.class)) {
+            return org.apache.openjpa.util.DelayedArrayListProxy.class;
+        }
+        if (type.equals(java.util.HashSet.class)) {
+            return org.apache.openjpa.util.DelayedHashSetProxy.class;
+        }
+        if (type.equals(java.util.LinkedList.class)) {
+            return org.apache.openjpa.util.DelayedLinkedListProxy.class;
+        }
+        if (type.equals(java.util.Vector.class)) {
+            return org.apache.openjpa.util.DelayedVectorProxy.class;
+        }
+        if (type.equals(java.util.LinkedHashSet.class)) {
+            return org.apache.openjpa.util.DelayedLinkedHashSetProxy.class;
+        }
+        if (type.equals(java.util.SortedSet.class) || type.equals(java.util.TreeSet.class)) {
+            return org.apache.openjpa.util.DelayedTreeSetProxy.class;
+        }
+        if (type.equals(java.util.PriorityQueue.class)) {
+            return org.apache.openjpa.util.DelayedPriorityQueueProxy.class;
+        }
+        return null;
+    }
 
     /**
      * Instantiate the given proxy class.
@@ -560,7 +613,9 @@ public class ProxyManagerImpl
         delegateConstructors(bc, type);
         addProxyMethods(bc, false);
         addProxyMapMethods(bc, type);
-        proxyRecognizedMethods(bc, type, ProxyMaps.class, ProxyMap.class);
+        Class<? extends ProxyMaps> mapProxyClassType =
+            ConcurrentMap.class.isAssignableFrom(type) ? ProxyConcurrentMaps.class : ProxyMaps.class;
+        proxyRecognizedMethods(bc, type, mapProxyClassType, ProxyMap.class);
         proxySetters(bc, type);
         addWriteReplaceMethod(bc, runtime);
         return bc;
@@ -1660,7 +1715,7 @@ public class ProxyManagerImpl
             }
 
             System.out.println(bc.getName());
-            bc.write(new File(dir, bc.getClassName() + ".class"));
+            AsmAdaptor.write(bc, new File(dir, bc.getClassName() + ".class"));
         }
     }
 }
