@@ -118,7 +118,7 @@ public class ConfigurationImpl
     private boolean _globals = false;
     private String _auto = null;
     private final List<Value> _vals = new ArrayList<Value>();
-    private Set<String> _supportedKeys;
+    private Set<String> _supportedKeys = new TreeSet<String>();
     
     // property listener helper
     private PropertyChangeSupport _changeSupport = null;
@@ -126,6 +126,9 @@ public class ConfigurationImpl
     // cache descriptors
     private PropertyDescriptor[] _pds = null;
     private MethodDescriptor[] _mds = null;
+    //Ant task needs to defer the resource loading 
+    //until the classpath setting is loaded properly
+    private boolean _deferResourceLoading = false; 
 
     /**
      * Default constructor. Attempts to load default properties through
@@ -149,6 +152,7 @@ public class ConfigurationImpl
             "openjpa", LogFactoryImpl.class.getName(),
             "commons", "org.apache.openjpa.lib.log.CommonsLogFactory",
             "log4j", "org.apache.openjpa.lib.log.Log4JLogFactory",
+            "slf4j", "org.apache.openjpa.lib.log.SLF4JLogFactory",
             "none", NoneLogFactory.class.getName(),
             "false", NoneLogFactory.class.getName(),
         };
@@ -271,6 +275,14 @@ public class ConfigurationImpl
         }
     }
 
+    public boolean isDeferResourceLoading() {
+        return _deferResourceLoading;
+    }
+ 
+    public void setDeferResourceLoading(boolean deferResourceLoading) {
+        this._deferResourceLoading = deferResourceLoading;
+    }
+
     public void instantiateAll() {
         StringWriter errs = null;
         PrintWriter stack = null;
@@ -584,9 +596,7 @@ public class ConfigurationImpl
      * 
      * @param storeDefaults
      *            whether or not to retrieve a property if its value is the
-     *            default value. This parameter is irrelevant if getAll is true.
-     * @param getAll
-     *            whether or not to get all of the properties
+     *            default value.
      * @return
      */
     public Map toProperties(boolean storeDefaults) {
@@ -605,10 +615,11 @@ public class ConfigurationImpl
         // with default values, add values to properties
         if (_props == null || storeDefaults) {
             String str;
-            for (Value val : _vals) { 
+            for (Value val : _vals) {
+                // NOTE: Following was removed to hide Value.INVISIBLE properties, like connectionPassword
                 // if key in existing properties, we already know value is up to date
-//                if (_props != null && Configurations.containsProperty(val, _props) && val.isVisible())
-//                    continue;
+                //if (_props != null && Configurations.containsProperty(val, _props) && val.isVisible())
+                //    continue;
                 str = val.getString();
                 if ((str != null && (storeDefaults || !str.equals(val.getDefault()))))
                     setValue(clone, val);
@@ -650,7 +661,9 @@ public class ConfigurationImpl
             if (o == null)
                 continue;
             if (o instanceof String) {
-                if (!StringUtils.equals((String) o, val.getString()))
+                // OPENJPA-1830 Do not overwrite existing string values with "******"
+                if ((!StringUtils.equals((String) o, val.getString())) &&
+                        (!StringUtils.equals((String) o, Value.INVISIBLE)))
                     val.setString((String) o);
             } else {
                 ser &= o instanceof Serializable;
@@ -663,6 +676,7 @@ public class ConfigurationImpl
         // <prefix>.properties System property; remove that property so we
         // we don't warn about it
         Configurations.removeProperty("properties", remaining);
+        Configurations.removeProperty("Id", remaining, map);
 
         // now warn if there are any remaining properties that there
         // is an unhandled prop, and remove the unknown properties
@@ -694,19 +708,21 @@ public class ConfigurationImpl
      * The Values that are {@linkplain Value#makePrivate() marked private} are filtered out. 
      */
     public Set<String> getPropertyKeys() {
-        if (_supportedKeys != null) 
-            return _supportedKeys;
-        
-        _supportedKeys = new TreeSet<String>();
-        for (Value val : _vals) {
-            if (val.isPrivate())
-                continue;
-            List<String> keys = val.getPropertyKeys();
-            for (String key : keys) {
-                _supportedKeys.add(fixPrefix(key));
+        synchronized (_supportedKeys) {
+            if (_supportedKeys.size() == 0) {
+                for (Value val : _vals) {
+                    if (val.isPrivate())
+                        continue;
+                    List<String> keys = val.getPropertyKeys();
+                    for (String key : keys) {
+                        _supportedKeys.add(fixPrefix(key));
+                    }
+                }
             }
         }
-        return _supportedKeys;
+        //OJ2257: Return a copy of _supportedKeys as calls to this method (e.g. 
+        //BrokerImpl.getSupportedProperties()) may add to this set.
+        return new TreeSet<String>(_supportedKeys);
     }
     
     /**
@@ -840,8 +856,17 @@ public class ConfigurationImpl
      * <code>properties</code> value with the name of a resource.
      */
     public void setProperties(String resourceName) throws IOException {
-        ProductDerivations.load(resourceName, null, 
-            getClass().getClassLoader()).setInto(this);
+
+        if (!_deferResourceLoading) {
+            String anchor = null;
+            if (resourceName.indexOf("#") != -1) {
+                anchor = resourceName.substring(resourceName.lastIndexOf("#") + 1);
+                resourceName = resourceName.substring(0, resourceName.length() - anchor.length() - 1);
+            }
+
+            ProductDerivations.load(resourceName, anchor, getClass().getClassLoader()).setInto(this);
+        }
+
         _auto = resourceName;
     }
 
@@ -853,6 +878,7 @@ public class ConfigurationImpl
     public void setPropertiesFile(File file) throws IOException {
         ProductDerivations.load(file, null, getClass().getClassLoader()).
             setInto(this);
+        setDeferResourceLoading(false);
         _auto = file.toString();
     }
 
