@@ -18,24 +18,27 @@
  */
 package org.apache.openjpa.lib.jdbc;
 
-import java.lang.reflect.Constructor;
-import java.lang.reflect.Method;
+import java.sql.Array;
+import java.sql.Blob;
 import java.sql.CallableStatement;
+import java.sql.Clob;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
+import java.sql.NClob;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
+import java.sql.SQLClientInfoException;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
+import java.sql.SQLXML;
 import java.sql.Savepoint;
 import java.sql.Statement;
-import java.util.HashMap;
+import java.sql.Struct;
 import java.util.Map;
+import java.util.Properties;
+import java.util.concurrent.Executor;
 
-import org.apache.commons.lang.exception.NestableRuntimeException;
 import org.apache.openjpa.lib.util.Closeable;
-import org.apache.openjpa.lib.util.ConcreteClassGenerator;
-import org.apache.openjpa.lib.util.Localizer;
 
 
 /**
@@ -46,51 +49,7 @@ import org.apache.openjpa.lib.util.Localizer;
  *
  * @author Abe White
  */
-public abstract class DelegatingConnection implements Connection, Closeable {
-
-    static final Constructor<DelegatingConnection> concreteImpl;
-    static {
-        try {
-            concreteImpl = ConcreteClassGenerator.getConcreteConstructor(DelegatingConnection.class, Connection.class);
-        } catch (Exception e) {
-            throw new ExceptionInInitializerError(e);
-        }
-    }
-
-    // jdbc 3 method keys
-    private static final Object SET_HOLDABILITY = new Object();
-    private static final Object GET_HOLDABILITY = new Object();
-    private static final Object SET_SAVEPOINT_NONAME = new Object();
-    private static final Object SET_SAVEPOINT = new Object();
-    private static final Object ROLLBACK_SAVEPOINT = new Object();
-    private static final Object RELEASE_SAVEPOINT = new Object();
-    private static final Object CREATE_STATEMENT = new Object();
-    private static final Object PREPARE_STATEMENT = new Object();
-    private static final Object PREPARE_CALL = new Object();
-    private static final Object PREPARE_WITH_KEYS = new Object();
-    private static final Object PREPARE_WITH_INDEX = new Object();
-    private static final Object PREPARE_WITH_NAMES = new Object();
-
-    private static final Localizer _loc = Localizer.forPackage
-        (DelegatingConnection.class);
-
-    private static final Map<Object, Method> _jdbc3;
-
-    static {
-        boolean jdbc3 = false;
-        Method m = null;
-        try {
-            m = Connection.class.getMethod("setSavepoint", new Class[]{ String.class });
-            jdbc3 = true;
-        } catch (Throwable t) {
-        }
-
-        if (jdbc3) {
-            _jdbc3 = new HashMap<Object,Method>();
-            _jdbc3.put(SET_SAVEPOINT, m);
-        } else
-            _jdbc3 = null;
-    }
+public class DelegatingConnection implements Connection, Closeable {
 
     private final Connection _conn;
     private final DelegatingConnection _del;
@@ -103,18 +62,6 @@ public abstract class DelegatingConnection implements Connection, Closeable {
             _del = null;
     }
 
-    /** 
-     *  Constructor for the concrete implementation of this abstract class.
-     */
-    public static DelegatingConnection newInstance(Connection conn) {
-        return ConcreteClassGenerator.newInstance(concreteImpl, conn);
-    }
-
-    /** 
-     *  Marker to enforce that subclasses of this class are abstract.
-     */
-    protected abstract void enforceAbstract();
-
     /**
      * Return the wrapped connection.
      */
@@ -167,7 +114,7 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         else
             stmnt = _conn.createStatement();
         if (wrap)
-            stmnt = DelegatingStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingStatement(stmnt, this);
         return stmnt;
     }
 
@@ -188,7 +135,7 @@ public abstract class DelegatingConnection implements Connection, Closeable {
             stmnt = _conn.prepareStatement(str, ResultSet.TYPE_FORWARD_ONLY, 
                 ResultSet.CONCUR_READ_ONLY);
         if (wrap)
-            stmnt = DelegatingPreparedStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingPreparedStatement(stmnt, this);
         return stmnt;
     }
 
@@ -208,7 +155,7 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         else
             stmnt = _conn.prepareCall(str);
         if (wrap)
-            stmnt = DelegatingCallableStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingCallableStatement(stmnt, this);
         return stmnt;
     }
 
@@ -255,7 +202,7 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         else
             meta = _conn.getMetaData();
         if (wrap)
-            meta = DelegatingDatabaseMetaData.newInstance(meta, this);
+            meta = new DelegatingDatabaseMetaData(meta, this);
         return meta;
     }
 
@@ -307,7 +254,7 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         else
             stmnt = _conn.createStatement(type, concur);
         if (wrap)
-            stmnt = DelegatingStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingStatement(stmnt, this);
         return stmnt;
     }
 
@@ -328,7 +275,7 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         else
             stmnt = _conn.prepareStatement(str, type, concur);
         if (wrap)
-            stmnt = DelegatingPreparedStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingPreparedStatement(stmnt, this);
         return stmnt;
     }
 
@@ -349,7 +296,7 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         else
             stmnt = _conn.prepareCall(str, type, concur);
         if (wrap)
-            stmnt = DelegatingCallableStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingCallableStatement(stmnt, this);
         return stmnt;
     }
 
@@ -361,66 +308,35 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         _conn.setTypeMap(map);
     }
 
-    // JDBC 3.0 methods follow; these are required to be able to
-    // compile against JDK 1.4; these methods will not work on
-    // previous JVMs
+    // JDBC 3.0 methods follow.
 
     public void setHoldability(int holdability) throws SQLException {
-        assertJDBC3();
-        Method m = (Method) _jdbc3.get(SET_HOLDABILITY);
-        if (m == null)
-            m = createJDBC3Method(SET_HOLDABILITY, "setHoldability",
-                new Class[]{ int.class });
-        invokeJDBC3(m, new Object[]{ holdability });
+        _conn.setHoldability(holdability);
     }
 
     public int getHoldability() throws SQLException {
-        assertJDBC3();
-        Method m = (Method) _jdbc3.get(GET_HOLDABILITY);
-        if (m == null)
-            m = createJDBC3Method(GET_HOLDABILITY, "getHoldability", null);
-        return ((Number) invokeJDBC3(m, null)).intValue();
+        return _conn.getHoldability();
     }
 
     public Savepoint setSavepoint() throws SQLException {
-        assertJDBC3();
-        Method m = (Method) _jdbc3.get(SET_SAVEPOINT_NONAME);
-        if (m == null)
-            m = createJDBC3Method(SET_SAVEPOINT_NONAME, "setSavepoint", null);
-        return (Savepoint) invokeJDBC3(m, null);
+        return _conn.setSavepoint();
     }
 
     public Savepoint setSavepoint(String savepoint) throws SQLException {
-        assertJDBC3();
-        Method m = (Method) _jdbc3.get(SET_SAVEPOINT);
-        if (m == null)
-            m = createJDBC3Method(SET_SAVEPOINT, "setSavepoint",
-                new Class[]{ String.class });
-        return (Savepoint) invokeJDBC3(m, new Object[]{ savepoint });
+        return _conn.setSavepoint(savepoint);
     }
 
     public void rollback(Savepoint savepoint) throws SQLException {
-        assertJDBC3();
-        Method m = (Method) _jdbc3.get(ROLLBACK_SAVEPOINT);
-        if (m == null)
-            m = createJDBC3Method(ROLLBACK_SAVEPOINT, "rollback",
-                new Class[]{ Savepoint.class });
-        invokeJDBC3(m, new Object[]{ savepoint });
+        _conn.rollback(savepoint);
     }
 
     public void releaseSavepoint(Savepoint savepoint) throws SQLException {
-        assertJDBC3();
-        Method m = (Method) _jdbc3.get(RELEASE_SAVEPOINT);
-        if (m == null)
-            m = createJDBC3Method(RELEASE_SAVEPOINT, "releaseSavepoint",
-                new Class[]{ Savepoint.class });
-        invokeJDBC3(m, new Object[]{ savepoint });
+        _conn.releaseSavepoint(savepoint);
     }
 
     public Statement createStatement(int resultSetType,
         int resultSetConcurrency, int resultSetHoldability)
         throws SQLException {
-        assertJDBC3();
         return createStatement(resultSetType, resultSetConcurrency,
             resultSetHoldability, true);
     }
@@ -433,24 +349,16 @@ public abstract class DelegatingConnection implements Connection, Closeable {
             stmnt = _del.createStatement(resultSetType, resultSetConcurrency,
                 resultSetHoldability, false);
         else {
-            Method m = (Method) _jdbc3.get(CREATE_STATEMENT);
-            if (m == null)
-                m = createJDBC3Method(CREATE_STATEMENT, "createStatement",
-                    new Class[]{ int.class, int.class, int.class });
-            stmnt = (Statement) invokeJDBC3(m, new Object[]{
-                resultSetType,
-                resultSetConcurrency,
-                resultSetHoldability });
+            stmnt = _conn.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
         }
         if (wrap)
-            stmnt = DelegatingStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingStatement(stmnt, this);
         return stmnt;
     }
 
     public PreparedStatement prepareStatement(String sql,
         int resultSetType, int resultSetConcurrency, int resultSetHoldability)
         throws SQLException {
-        assertJDBC3();
         return prepareStatement(sql, resultSetType, resultSetConcurrency,
             resultSetHoldability, true);
     }
@@ -463,25 +371,16 @@ public abstract class DelegatingConnection implements Connection, Closeable {
             stmnt = _del.prepareStatement(sql, resultSetType,
                 resultSetConcurrency, resultSetHoldability, false);
         else {
-            Method m = (Method) _jdbc3.get(PREPARE_STATEMENT);
-            if (m == null)
-                m = createJDBC3Method(PREPARE_STATEMENT, "prepareStatement",
-                    new Class[]{ String.class, int.class, int.class,
-                        int.class });
-            stmnt = (PreparedStatement) invokeJDBC3(m, new Object[]{ sql,
-                resultSetType,
-                resultSetConcurrency,
-                resultSetHoldability });
+            stmnt = _conn.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
         }
         if (wrap)
-            stmnt = DelegatingPreparedStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingPreparedStatement(stmnt, this);
         return stmnt;
     }
 
     public CallableStatement prepareCall(String sql,
         int resultSetType, int resultSetConcurrency, int resultSetHoldability)
         throws SQLException {
-        assertJDBC3();
         return prepareCall(sql, resultSetType, resultSetConcurrency,
             resultSetHoldability, true);
     }
@@ -494,24 +393,15 @@ public abstract class DelegatingConnection implements Connection, Closeable {
             stmnt = _del.prepareCall(sql, resultSetType,
                 resultSetConcurrency, resultSetHoldability, false);
         else {
-            Method m = (Method) _jdbc3.get(PREPARE_CALL);
-            if (m == null)
-                m = createJDBC3Method(PREPARE_CALL, "prepareCall",
-                    new Class[]{ String.class, int.class, int.class,
-                        int.class });
-            stmnt = (CallableStatement) invokeJDBC3(m, new Object[]{ sql,
-                resultSetType,
-                resultSetConcurrency,
-                resultSetHoldability });
+            stmnt = _conn.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
         }
         if (wrap)
-            stmnt = DelegatingCallableStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingCallableStatement(stmnt, this);
         return stmnt;
     }
 
     public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
         throws SQLException {
-        assertJDBC3();
         return prepareStatement(sql, autoGeneratedKeys, true);
     }
 
@@ -521,21 +411,15 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         if (_del != null)
             stmnt = _del.prepareStatement(sql, autoGeneratedKeys);
         else {
-            Method m = (Method) _jdbc3.get(PREPARE_WITH_KEYS);
-            if (m == null)
-                m = createJDBC3Method(PREPARE_WITH_KEYS, "prepareStatement",
-                    new Class[]{ String.class, int.class });
-            stmnt = (PreparedStatement) invokeJDBC3(m, new Object[]{ sql,
-                autoGeneratedKeys });
+            stmnt = _conn.prepareStatement(sql, autoGeneratedKeys);
         }
         if (wrap)
-            stmnt = DelegatingPreparedStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingPreparedStatement(stmnt, this);
         return stmnt;
     }
 
     public PreparedStatement prepareStatement(String sql, int[] columnIndexes)
         throws SQLException {
-        assertJDBC3();
         return prepareStatement(sql, columnIndexes, true);
     }
 
@@ -545,81 +429,108 @@ public abstract class DelegatingConnection implements Connection, Closeable {
         if (_del != null)
             stmnt = _del.prepareStatement(sql, columnIndexes, wrap);
         else {
-            Method m = (Method) _jdbc3.get(PREPARE_WITH_INDEX);
-            if (m == null)
-                m = createJDBC3Method(PREPARE_WITH_INDEX, "prepareStatement",
-                    new Class[]{ String.class, int[].class });
-            stmnt = (PreparedStatement) invokeJDBC3(m, new Object[]{ sql,
-                columnIndexes });
+            stmnt = _conn.prepareStatement(sql, columnIndexes);
         }
         if (wrap)
-            stmnt = DelegatingPreparedStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingPreparedStatement(stmnt, this);
         return stmnt;
     }
 
     public PreparedStatement prepareStatement(String sql, String[] columnNames)
         throws SQLException {
-        assertJDBC3();
         return prepareStatement(sql, columnNames, true);
     }
 
     protected PreparedStatement prepareStatement(String sql,
         String[] columnNames, boolean wrap) throws SQLException {
-        assertJDBC3();
         PreparedStatement stmnt;
         if (_del != null)
             stmnt = _del.prepareStatement(sql, columnNames, wrap);
         else {
-            Method m = (Method) _jdbc3.get(PREPARE_WITH_NAMES);
-            if (m == null)
-                m = createJDBC3Method(PREPARE_WITH_NAMES, "prepareStatement",
-                    new Class[]{ String.class, String[].class });
-            stmnt = (PreparedStatement) invokeJDBC3(m, new Object[]{ sql,
-                columnNames });
+            stmnt = _conn.prepareStatement(sql, columnNames);
         }
         if (wrap)
-            stmnt = DelegatingPreparedStatement.newInstance(stmnt, this);
+            stmnt = new DelegatingPreparedStatement(stmnt, this);
         return stmnt;
     }
 
-    private static void assertJDBC3() {
-        if (_jdbc3 == null)
-            throw new UnsupportedOperationException(_loc.get("not-jdbc3")
-                .getMessage());
-    }
+    //  JDBC 4.0 methods follow.
 
-    private Object invokeJDBC3(Method m, Object[] args) throws SQLException {
-        try {
-            return m.invoke(_conn, args);
-        } catch (Throwable t) {
-            if (t instanceof SQLException)
-                throw(SQLException) t;
-            throw new NestableRuntimeException(_loc.get("invoke-jdbc3")
-                .getMessage(), t);
-        }
-    }
-
-    private static Method createJDBC3Method(Object key, String name,
-        Class<?>[] args) {
-        try {
-            Method m = Connection.class.getMethod(name, args);
-            _jdbc3.put(key, m);
-            return m;
-        } catch (Throwable t) {
-            throw new NestableRuntimeException(_loc.get("error-jdbc3")
-                .getMessage(), t);
-        }
-    }
-
-    // java.sql.Wrapper implementation (JDBC 4)
-    public boolean isWrapperFor(Class iface) {
+    public boolean isWrapperFor(Class<?> iface) throws SQLException {
         return iface.isAssignableFrom(getDelegate().getClass());
     }
 
-    public Object unwrap(Class iface) {
+    @Override
+    public <T> T unwrap(Class<T> iface) throws SQLException {
         if (isWrapperFor(iface))
-            return getDelegate();
+            return (T) getDelegate();
         else
             return null;
     }
+
+    public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
+        return _conn.createArrayOf(typeName, elements);
+    }
+
+    public Blob createBlob() throws SQLException {
+        return _conn.createBlob();
+    }
+
+    public Clob createClob() throws SQLException {
+        return _conn.createClob();
+    }
+
+    public NClob createNClob() throws SQLException {
+        return _conn.createNClob();
+    }
+
+    public SQLXML createSQLXML() throws SQLException {
+        return _conn.createSQLXML();
+    }
+
+    public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
+        return _conn.createStruct(typeName, attributes);
+    }
+
+    public Properties getClientInfo() throws SQLException {
+        return _conn.getClientInfo();
+    }
+
+    public String getClientInfo(String name) throws SQLException {
+        return _conn.getClientInfo(name);
+    }
+
+    public boolean isValid(int timeout) throws SQLException {
+        return _conn.isValid(timeout);
+    }
+
+    public void setClientInfo(Properties properties) throws SQLClientInfoException {
+        _conn.setClientInfo(properties);
+    }
+
+    public void setClientInfo(String name, String value) throws SQLClientInfoException {
+        _conn.setClientInfo(name, value);
+    }
+    
+    // Java 7 methods follow
+    
+    public void abort(Executor executor) throws SQLException {
+    	throw new UnsupportedOperationException();
+    }
+    
+    public int getNetworkTimeout() throws SQLException{
+    	throw new UnsupportedOperationException();
+    }
+    
+    public void setNetworkTimeout(Executor executor, int milliseconds) throws SQLException{
+    	throw new UnsupportedOperationException();
+    }
+    
+    public String getSchema() throws SQLException {
+    	throw new UnsupportedOperationException(); 
+    }
+    
+    public void setSchema(String schema)throws SQLException {
+    	throw new UnsupportedOperationException();
+    }
 }
