@@ -20,8 +20,8 @@ package org.apache.openjpa.lib.jdbc;
 
 import java.io.InputStream;
 import java.io.Reader;
-import java.lang.reflect.Constructor;
 import java.math.BigDecimal;
+import java.net.URL;
 import java.sql.Array;
 import java.sql.BatchUpdateException;
 import java.sql.Blob;
@@ -46,18 +46,7 @@ import java.util.Calendar;
 import java.util.Iterator;
 import java.util.List;
 
-import org.apache.openjpa.lib.jdbc.LoggingConnectionDecorator.LoggingConnection.
-        LoggingCallableStatement;
-import org.apache.openjpa.lib.jdbc.LoggingConnectionDecorator.LoggingConnection.
-        LoggingDatabaseMetaData;
-import org.apache.openjpa.lib.jdbc.LoggingConnectionDecorator.LoggingConnection.
-        LoggingPreparedStatement;
-import org.apache.openjpa.lib.jdbc.LoggingConnectionDecorator.LoggingConnection.
-        LoggingResultSet;
-import org.apache.openjpa.lib.jdbc.LoggingConnectionDecorator.LoggingConnection.
-        LoggingStatement;
 import org.apache.openjpa.lib.log.Log;
-import org.apache.openjpa.lib.util.ConcreteClassGenerator;
 import org.apache.openjpa.lib.util.J2DoPrivHelper;
 
 /**
@@ -70,12 +59,6 @@ import org.apache.openjpa.lib.util.J2DoPrivHelper;
 public class LoggingConnectionDecorator implements ConnectionDecorator {
 
     private static final String SEP = J2DoPrivHelper.getLineSeparator();
-    static final Constructor<LoggingConnection> loggingConnectionImpl;
-    static final Constructor<LoggingResultSet> loggingResultSetImpl;
-    static final Constructor<LoggingStatement> loggingStatementImpl;
-    static final Constructor<LoggingPreparedStatement> loggingPreparedStatementImpl;
-    static final Constructor<LoggingCallableStatement> loggingCallableStatementImpl;
-    static final Constructor<LoggingDatabaseMetaData> loggingDatabaseMetaDataImpl;
 
     private static final int WARN_IGNORE = 0;
     private static final int WARN_LOG_TRACE = 1;
@@ -94,28 +77,6 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
         WARNING_ACTIONS[WARN_LOG_ERROR] = "error";
         WARNING_ACTIONS[WARN_THROW] = "throw";
         WARNING_ACTIONS[WARN_HANDLE] = "handle";
-
-        try {
-            loggingConnectionImpl = ConcreteClassGenerator.getConcreteConstructor(LoggingConnection.class,
-                LoggingConnectionDecorator.class, Connection.class);
-            loggingResultSetImpl = ConcreteClassGenerator.getConcreteConstructor(
-                LoggingConnection.LoggingResultSet.class, 
-                LoggingConnection.class, ResultSet.class, Statement.class);
-            loggingStatementImpl = ConcreteClassGenerator.getConcreteConstructor(
-                LoggingConnection.LoggingStatement.class,
-                LoggingConnection.class, Statement.class);
-            loggingPreparedStatementImpl = ConcreteClassGenerator.getConcreteConstructor(
-                LoggingConnection.LoggingPreparedStatement.class, 
-                LoggingConnection.class, PreparedStatement.class, String.class);
-            loggingCallableStatementImpl = ConcreteClassGenerator.getConcreteConstructor(
-                LoggingConnection.LoggingCallableStatement.class,
-                LoggingConnection.class, CallableStatement.class, String.class);
-            loggingDatabaseMetaDataImpl = ConcreteClassGenerator.getConcreteConstructor(
-                LoggingConnection.LoggingDatabaseMetaData.class,
-                LoggingConnection.class, DatabaseMetaData.class);
-        } catch (Exception e) {
-            throw new ExceptionInInitializerError(e);
-        }
     }
 
     private final DataSourceLogs _logs = new DataSourceLogs();
@@ -124,7 +85,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
     private int _prettyPrintLineLength = 60;
     private int _warningAction = WARN_IGNORE;
     private SQLWarningHandler _warningHandler;
-    private boolean _trackParameters = true;
+    private boolean _printParameters = false;
 
     /**
      * If set to <code>true</code>, pretty-print SQL by running it
@@ -169,17 +130,18 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
     }
 
     /**
-     * Whether to track parameters for the purposes of reporting exceptions.
+     * <p>
+     * Whether parameter values will be printed in exception messages or in trace. This is different from
+     * trackParameters which controls whether OpenJPA will track parameters internally (visible while debugging and used
+     * in batching).
+     * </p>
      */
-    public void setTrackParameters(boolean trackParameters) {
-        _trackParameters = trackParameters;
+    public boolean getPrintParameters() {
+        return _printParameters;
     }
 
-    /**
-     * Whether to track parameters for the purposes of reporting exceptions.
-     */
-    public boolean getTrackParameters() {
-        return _trackParameters;
+    public void setPrintParameters(boolean printParameters) {
+        _printParameters = printParameters;
     }
 
     /**
@@ -226,7 +188,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
          
     private LoggingConnection newLoggingConnection(Connection conn)
         throws SQLException {
-        return ConcreteClassGenerator.newInstance(loggingConnectionImpl, LoggingConnectionDecorator.this, conn);
+        return new LoggingConnection(conn);
     }
 
     private SQLException wrap(SQLException sqle, Statement stmnt) {
@@ -273,7 +235,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
     /**
      * Logging connection.
      */
-    protected abstract class LoggingConnection extends DelegatingConnection {
+    protected class LoggingConnection extends DelegatingConnection {
 
         public LoggingConnection(Connection conn) throws SQLException {
             super(conn);
@@ -352,21 +314,22 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
 
         private LoggingPreparedStatement newLoggingPreparedStatement
             (PreparedStatement stmnt, String sql) throws SQLException {
-            return ConcreteClassGenerator.newInstance(loggingPreparedStatementImpl, LoggingConnection.this, stmnt, sql);
+            return new LoggingPreparedStatement(stmnt, sql);
         }
         
-        private CallableStatement newLoggingCallableStatement(CallableStatement stmnt, String sql) throws SQLException {
-            return ConcreteClassGenerator.newInstance(loggingCallableStatementImpl, LoggingConnection.this, stmnt, sql);
+        private CallableStatement newLoggingCallableStatement(
+            CallableStatement stmnt, String sql) throws SQLException {
+            return new LoggingCallableStatement(stmnt, sql);
         }
         
         private LoggingStatement newLoggingStatement(Statement stmnt)
             throws SQLException {
-            return ConcreteClassGenerator.newInstance(loggingStatementImpl, LoggingConnection.this, stmnt);
+            return new LoggingStatement(stmnt);
         }
         
         private LoggingDatabaseMetaData newLoggingDatabaseMetaData
             (DatabaseMetaData meta) throws SQLException {
-            return ConcreteClassGenerator.newInstance(loggingDatabaseMetaDataImpl, LoggingConnection.this, meta);
+            return new LoggingDatabaseMetaData(meta);
         }
 
 
@@ -707,7 +670,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
         /**
          * Metadata wrapper that logs actions.
          */
-        protected abstract class LoggingDatabaseMetaData
+        protected class LoggingDatabaseMetaData
             extends DelegatingDatabaseMetaData {
 
             public LoggingDatabaseMetaData(DatabaseMetaData meta) {
@@ -887,7 +850,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
          * Statement wrapper that logs SQL to the parent data source and
          * remembers the last piece of SQL to be executed on it.
          */
-        protected abstract class LoggingStatement extends DelegatingStatement {
+        protected class LoggingStatement extends DelegatingStatement {
 
             private String _sql = null;
 
@@ -896,7 +859,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
             }
 
             private LoggingResultSet newLoggingResultSet(ResultSet rs, Statement stmnt) {
-                return ConcreteClassGenerator.newInstance(loggingResultSetImpl, LoggingConnection.this, rs, stmnt);
+                return new LoggingResultSet(rs, stmnt);
             }
 
             public void appendInfo(StringBuffer buf) {
@@ -971,9 +934,105 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
                     handleSQLErrors(LoggingStatement.this, err);
                 }
             }
+
+            public int executeUpdate(String sql, int i) throws SQLException {
+                _sql = sql;
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.executeUpdate(sql, i);
+                } catch (SQLException se) {                 
+                    err = wrap(se, LoggingStatement.this, sql);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    handleSQLErrors(LoggingStatement.this, err);
+                }
+            }
+
+            public int executeUpdate(String sql, int[] ia) throws SQLException {
+                _sql = sql;
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.executeUpdate(sql, ia);
+                } catch (SQLException se) {                 
+                    err = wrap(se, LoggingStatement.this, sql);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    handleSQLErrors(LoggingStatement.this, err);
+                }
+            }
+
+            public int executeUpdate(String sql, String[] sa) throws SQLException {
+                _sql = sql;
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.executeUpdate(sql, sa);
+                } catch (SQLException se) {                 
+                    err = wrap(se, LoggingStatement.this, sql);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    handleSQLErrors(LoggingStatement.this, err);
+                }
+            }
+
+            public boolean execute(String sql, int i) throws SQLException {
+                _sql = sql;
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.execute(sql, i);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingStatement.this, sql);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    handleSQLErrors(LoggingStatement.this, err);
+                }
+            }
+
+            public boolean execute(String sql, int[] ia) throws SQLException {
+                _sql = sql;
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.execute(sql, ia);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingStatement.this, sql);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    handleSQLErrors(LoggingStatement.this, err);
+                }
+            }
+
+            public boolean execute(String sql, String[] sa) throws SQLException {
+                _sql = sql;
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.execute(sql, sa);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingStatement.this, sql);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    handleSQLErrors(LoggingStatement.this, err);
+                }
+            }
         }
 
-        protected abstract class LoggingPreparedStatement
+        protected class LoggingPreparedStatement
             extends DelegatingPreparedStatement {
 
             private final String _sql;
@@ -991,11 +1050,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
 
             private LoggingResultSet newLoggingResultSet(ResultSet rs,
                 PreparedStatement stmnt) {
-                return ConcreteClassGenerator.
-                    newInstance(loggingResultSetImpl,
-                    LoggingConnection.class, LoggingConnection.this,
-                    ResultSet.class, rs,
-                    PreparedStatement.class, stmnt);
+                return new LoggingResultSet(rs, stmnt);
             }
 
             protected ResultSet wrapResult(ResultSet rs, boolean wrap) {
@@ -1154,7 +1209,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
 
                             // set the current params to the saved values
                             if (indexOfFirstFailedObject < _paramBatch.size())
-                                _params = (List) _paramBatch.get(indexOfFirstFailedObject);
+                                _params = (List<String>) _paramBatch.get(indexOfFirstFailedObject);
                         }
                     }
                     err = wrap(se, LoggingPreparedStatement.this, indexOfFirstFailedObject);
@@ -1181,6 +1236,102 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
                 }
             }
 
+            public int executeUpdate(String s, int i) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.executeUpdate(s, i);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingPreparedStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingPreparedStatement.this, err);
+                }
+            }
+
+            public int executeUpdate(String s, int[] ia) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.executeUpdate(s, ia);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingPreparedStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingPreparedStatement.this, err);
+                }
+            }
+
+            public int executeUpdate(String s, String[] sa) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.executeUpdate(s, sa);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingPreparedStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingPreparedStatement.this, err);
+                }
+            }
+
+            public boolean execute(String s, int i) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.execute(s, i);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingPreparedStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingPreparedStatement.this, err);
+                }
+            }
+
+            public boolean execute(String s, int[] ia) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.execute(s, ia);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingPreparedStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingPreparedStatement.this, err);
+                }
+            }
+
+            public boolean execute(String s, String[] sa) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;
+                try {
+                    return super.execute(s, sa);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingPreparedStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingPreparedStatement.this, err);
+                }
+            }
+
             public void cancel() throws SQLException {
                 if (_logs.isJDBCEnabled())
                     _logs.logJDBC("cancel " + this + ": " + _sql,
@@ -1278,7 +1429,13 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
                 setLogParameter(i1, "InputStream", is);
                 super.setBinaryStream(i1, is, i2);
             }
-
+            
+            public void setBinaryStream(int i1, InputStream is)
+            throws SQLException {
+            	setLogParameter(i1, "InputStream", is);
+            	super.setBinaryStream(i1, is);            	
+            }
+            
             public void clearParameters() throws SQLException {
                 clearLogParameters(false);
                 super.clearParameters();
@@ -1377,6 +1534,11 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
                 super.setNull(i1, i2, s);
             }
 
+            public void setURL(int i, URL u) throws SQLException {
+                setLogParameter(i, "URL", u);
+                super.setURL(i, u);
+            }
+
             protected void appendInfo(StringBuffer buf) {
                 buf.append(" ");
                 if (_formatter != null) {
@@ -1392,17 +1554,23 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
                     paramBuf = new StringBuilder();
                     for (Iterator<String> itr = _params.iterator(); itr
                         .hasNext();) {
-                        paramBuf.append(itr.next());
-                        if (itr.hasNext())
+                        if (_printParameters) {
+                            paramBuf.append(itr.next());
+                        } else {
+                            paramBuf.append("?");
+                            itr.next();
+                        }
+                        if (itr.hasNext()) {
                             paramBuf.append(", ");
+                        }
                     }
                 }
 
                 if (paramBuf != null) {
-                    if (!_prettyPrint)
+                    if (!_prettyPrint) {
                         buf.append(" ");
-                    buf.append("[params=").
-                        append(paramBuf.toString()).append("]");
+                    }
+                    buf.append("[params=").append(paramBuf.toString()).append("]");
                 }
                 super.appendInfo(buf);
             }
@@ -1418,7 +1586,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
             }
 
             private boolean shouldTrackParameters() {
-                return _trackParameters || _logs.isSQLEnabled();
+                return _printParameters || _logs.isSQLEnabled();
             }
 
             private void setLogParameter(int index, boolean val) {
@@ -1475,7 +1643,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
         /**
          * Warning-handling result set.
          */
-        protected abstract class LoggingResultSet extends DelegatingResultSet {
+        protected class LoggingResultSet extends DelegatingResultSet {
 
             public LoggingResultSet(ResultSet rs, Statement stmnt) {
                 super(rs, stmnt);
@@ -1595,7 +1763,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
          * Similar to {@link LoggingPreparedStatement} but can not be extended
          * due to the existing delegation hierarchy.
          */
-        protected abstract class LoggingCallableStatement extends 
+        protected class LoggingCallableStatement extends 
             DelegatingCallableStatement {
             private final String _sql;
             private List<String> _params = null;
@@ -1612,11 +1780,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
         	
             private LoggingResultSet newLoggingResultSet(ResultSet rs,
                 CallableStatement stmnt) {
-                return ConcreteClassGenerator.
-                    newInstance(loggingResultSetImpl,
-                    LoggingConnection.class, LoggingConnection.this,
-                    ResultSet.class, rs,
-                    CallableStatement.class, stmnt);
+                return new LoggingResultSet(rs, stmnt);
             }
             
             protected ResultSet wrapResult(ResultSet rs, boolean wrap) {
@@ -1803,6 +1967,102 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
                 }
             }
 
+            public int executeUpdate(String s, int i) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;                
+                try {
+                    return super.executeUpdate(s, i);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingCallableStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingCallableStatement.this, err);
+                }
+            }
+
+            public int executeUpdate(String s, int[] ia) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;                
+                try {
+                    return super.executeUpdate(s, ia);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingCallableStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingCallableStatement.this, err);
+                }
+            }
+
+            public int executeUpdate(String s, String[] sa) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;                
+                try {
+                    return super.executeUpdate(s, sa);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingCallableStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingCallableStatement.this, err);
+                }
+            }
+
+            public boolean execute(String s, int i) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;                
+                try {
+                    return super.execute(s, i);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingCallableStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingCallableStatement.this, err);
+                }
+            }
+
+            public boolean execute(String s, int[] ia) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;                
+                try {
+                    return super.execute(s, ia);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingCallableStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingCallableStatement.this, err);
+                }
+            }
+
+            public boolean execute(String s, String[] sa) throws SQLException {
+                logSQL(this);
+                long start = System.currentTimeMillis();
+                SQLException err = null;                
+                try {
+                    return super.execute(s, sa);
+                } catch (SQLException se) {
+                    err = wrap(se, LoggingCallableStatement.this);
+                    throw err;
+                } finally {
+                    logTime(start);
+                    clearLogParameters(true);
+                    handleSQLErrors(LoggingCallableStatement.this, err);
+                }
+            }
+
             public void cancel() throws SQLException {
                 if (_logs.isJDBCEnabled())
                     _logs.logJDBC("cancel " + this + ": " + _sql,
@@ -1999,6 +2259,11 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
                 super.setNull(i1, i2, s);
             }
 
+            public void setURL(int i, URL u) throws SQLException {
+                setLogParameter(i, "URL", u);
+                super.setURL(i, u);
+            }
+
             protected void appendInfo(StringBuffer buf) {
                 buf.append(" ");
                 if (_formatter != null) {
@@ -2037,7 +2302,7 @@ public class LoggingConnectionDecorator implements ConnectionDecorator {
             }
 
             private boolean shouldTrackParameters() {
-                return _trackParameters || _logs.isSQLEnabled();
+                return _printParameters || _logs.isSQLEnabled();
             }
 
             private void setLogParameter(int index, boolean val) {
