@@ -44,7 +44,6 @@ import org.xml.sax.ext.LexicalHandler;
 import org.xml.sax.helpers.DefaultHandler;
 import org.apache.openjpa.lib.log.Log;
 import org.apache.openjpa.lib.util.J2DoPrivHelper;
-import org.apache.openjpa.lib.util.JavaVersions;
 import org.apache.openjpa.lib.util.Localizer.Message;
 import org.apache.openjpa.lib.util.Localizer;
 import org.apache.openjpa.lib.xml.Commentable;
@@ -65,6 +64,10 @@ public abstract class XMLMetaDataParser extends DefaultHandler
     private static final Localizer _loc = Localizer.forPackage
         (XMLMetaDataParser.class);
     private static boolean _schemaBug;
+    
+    private static final String OPENJPA_NAMESPACE = "http://openjpa.apache.org/ns/orm";
+    protected int _extendedNamespace = 0;
+    protected int _openjpaNamespace = 0;
 
     static {
         try {
@@ -107,6 +110,16 @@ public abstract class XMLMetaDataParser extends DefaultHandler
     private int _ignore = Integer.MAX_VALUE;
 
     private boolean _parsing = false;
+    
+    private boolean _overrideContextClassloader = false;
+    
+    public boolean getOverrideContextClassloader() {
+        return _overrideContextClassloader;
+    }
+
+    public void setOverrideContextClassloader(boolean overrideCCL) {
+        _overrideContextClassloader = overrideCCL;
+    }
 
     /*
      * Whether the parser is currently parsing.
@@ -366,36 +379,69 @@ public abstract class XMLMetaDataParser extends DefaultHandler
         try {
             setParsing(true);
             _sourceName = sourceName;
-            SAXParser parser = XMLFactory.getSAXParser(validating, true);
-            Object schema = null;
-            if (validating) {
-                schema = schemaSource;
-                if (schema == null && getDocType() != null)
-                    xml = new DocTypeReader(xml, getDocType());
+            
+            SAXParser parser = null;
+            boolean overrideCL = _overrideContextClassloader;
+            ClassLoader oldLoader = null;
+            ClassLoader newLoader = null;           
+            
+            try {
+                if (overrideCL == true) {
+                    oldLoader =
+                        (ClassLoader) AccessController.doPrivileged(J2DoPrivHelper.getContextClassLoaderAction());
+                    newLoader = XMLMetaDataParser.class.getClassLoader();
+                    AccessController.doPrivileged(J2DoPrivHelper.setContextClassLoaderAction(newLoader));
+                    
+                    if (_log != null && _log.isTraceEnabled()) {
+                        _log.trace(_loc.get("override-contextclassloader-begin", oldLoader, newLoader));
+                    }                   
+                }
+                
+                parser = XMLFactory.getSAXParser(validating, true);
+                Object schema = null;
+                if (validating) {
+                    schema = schemaSource;
+                    if (schema == null && getDocType() != null)
+                        xml = new DocTypeReader(xml, getDocType());
+                }
+
+                if (_parseComments || _lh != null)
+                    parser.setProperty
+                        ("http://xml.org/sax/properties/lexical-handler", this);
+
+                if (schema != null) {
+                    parser.setProperty
+                        ("http://java.sun.com/xml/jaxp/properties/schemaLanguage",
+                            "http://www.w3.org/2001/XMLSchema");
+                    parser.setProperty
+                        ("http://java.sun.com/xml/jaxp/properties/schemaSource",
+                            schema);
+                }
+
+                InputSource is = new InputSource(xml);
+                if (_systemId && sourceName != null)
+                    is.setSystemId(sourceName);
+                parser.parse(is, this);
+                finish();
+            } catch (SAXException se) {
+                IOException ioe = new IOException(se.toString());
+                ioe.initCause(se);
+                throw ioe;
+            } finally {
+                if (overrideCL == true) {
+                    // Restore the old ContextClassloader
+                    try {
+                        if (_log != null && _log.isTraceEnabled()) {
+                            _log.trace(_loc.get("override-contextclassloader-end", newLoader, oldLoader));
+                        }
+                        AccessController.doPrivileged(J2DoPrivHelper.setContextClassLoaderAction(oldLoader));
+                    } catch (Throwable t) {
+                        if (_log != null && _log.isWarnEnabled()) {
+                            _log.warn(_loc.get("restore-contextclassloader-failed"));
+                        }
+                    }
+                }
             }
-
-            if (_parseComments || _lh != null)
-                parser.setProperty
-                    ("http://xml.org/sax/properties/lexical-handler", this);
-
-            if (schema != null) {
-                parser.setProperty
-                    ("http://java.sun.com/xml/jaxp/properties/schemaLanguage",
-                        "http://www.w3.org/2001/XMLSchema");
-                parser.setProperty
-                    ("http://java.sun.com/xml/jaxp/properties/schemaSource",
-                        schema);
-            }
-
-            InputSource is = new InputSource(xml);
-            if (_systemId && sourceName != null)
-                is.setSystemId(sourceName);
-            parser.parse(is, this);
-            finish();
-        } catch (SAXException se) {
-            IOException ioe = new IOException(se.toString());
-            JavaVersions.initCause(ioe, se);
-            throw ioe;
         } finally {
             reset();
         }
@@ -445,15 +491,28 @@ public abstract class XMLMetaDataParser extends DefaultHandler
     public void startElement(String uri, String name, String qName,
         Attributes attrs) throws SAXException {
         _depth++;
-        if (_depth <= _ignore)
+        if (_depth <= _ignore){
+            if (uri.equals(OPENJPA_NAMESPACE)) {
+                _extendedNamespace++;
+                _openjpaNamespace++;
+            }
             if (!startElement(qName, attrs))
                 ignoreContent(true);
+        }
     }
 
     public void endElement(String uri, String name, String qName)
         throws SAXException {
-        if (_depth < _ignore)
+        if (_depth < _ignore) {
             endElement(qName);
+            _extendedNamespace = (_extendedNamespace > 0) ? _extendedNamespace - 1 : 0;
+            _openjpaNamespace = (_openjpaNamespace > 0) ? _openjpaNamespace - 1 : 0;
+        }
+        else if (_depth ==_ignore) {
+            _extendedNamespace = (_extendedNamespace > 0) ? _extendedNamespace - 1 : 0;
+            _openjpaNamespace = (_openjpaNamespace > 0) ? _openjpaNamespace - 1 : 0;
+        }
+        
         _text = null;
         if (_comments != null)
             _comments.clear();
