@@ -24,11 +24,13 @@ import javax.persistence.Cache;
 import javax.persistence.CacheRetrieveMode;
 import javax.persistence.CacheStoreMode;
 import javax.persistence.EntityManager;
+import javax.persistence.LockModeType;
 
 import org.apache.openjpa.jdbc.meta.ClassMapping;
 import org.apache.openjpa.lib.jdbc.AbstractJDBCListener;
 import org.apache.openjpa.lib.jdbc.JDBCEvent;
 import org.apache.openjpa.persistence.OpenJPAEntityManagerFactorySPI;
+import org.apache.openjpa.persistence.OpenJPAEntityManagerSPI;
 import org.apache.openjpa.persistence.cache.jpa.model.CacheEntity;
 import org.apache.openjpa.persistence.cache.jpa.model.CacheableEntity;
 import org.apache.openjpa.persistence.cache.jpa.model.NegatedCachableEntity;
@@ -154,20 +156,6 @@ public abstract class AbstractCacheModeTestCase extends AbstractCacheTestCase {
         public void run();
     }
 
-    /**
-     * Simple JDBCListener which stores the executed sql in a List. The List is
-     * provided by the getSql() method so that subclasses may use separate
-     * lists.
-     */
-    public class Listener extends AbstractJDBCListener {
-        @Override
-        public void beforeExecuteStatement(JDBCEvent event) {
-            if (event.getSQL() != null && getSql() != null) {
-                getSql().add(event.getSQL());
-            }
-        }
-    }
-
     // =======================================================================
     // Test utilities
     // =======================================================================
@@ -218,28 +206,30 @@ public abstract class AbstractCacheModeTestCase extends AbstractCacheTestCase {
      * 
      */
     public void testRetrieveModeUse() {
-        assertNoSql(new Action() {
-            public void run() {
-                EntityManager em = getEntityManagerFactory().createEntityManager();
-                em.setProperty(RETRIEVE_MODE_PROP, CacheRetrieveMode.USE);
-                for (Class<?> cls : getExpectedInCache()) {
-                    assertCached(getEntityManagerFactory().getCache(), cls, 1, true);
-                    assertNotNull(em.find(cls, 1));
+        if (getCacheEnabled()) {
+            assertNoSql(new Action() {
+                public void run() {
+                    EntityManager em = getEntityManagerFactory().createEntityManager();
+                    em.setProperty(RETRIEVE_MODE_PROP, CacheRetrieveMode.USE);
+                    for (Class<?> cls : getExpectedInCache()) {
+                        assertCached(getEntityManagerFactory().getCache(), cls, 1, true);
+                        assertNotNull(em.find(cls, 1));
+                    }
+                    em.close();
                 }
-                em.close();
-            }
-        });
-        assertSqlInc(new Action() {
-            public void run() {
-                EntityManager em = getEntityManagerFactory().createEntityManager();
-                em.setProperty(RETRIEVE_MODE_PROP, CacheRetrieveMode.USE);
-                for (Class<?> cls : getExpectedNotInCache()) {
-                    assertCached(getEntityManagerFactory().getCache(), cls, 1, false);
-                    assertNotNull(em.find(cls, 1));
+            });
+            assertSqlInc(new Action() {
+                public void run() {
+                    EntityManager em = getEntityManagerFactory().createEntityManager();
+                    em.setProperty(RETRIEVE_MODE_PROP, CacheRetrieveMode.USE);
+                    for (Class<?> cls : getExpectedNotInCache()) {
+                        assertCached(getEntityManagerFactory().getCache(), cls, 1, false);
+                        assertNotNull(em.find(cls, 1));
+                    }
+                    em.close();
                 }
-                em.close();
-            }
-        }, getExpectedNotInCache().length);
+            }, getExpectedNotInCache().length);
+        }
     }
 
     public void updateAndFind(Class<? extends CacheEntity> classToUpdate, int idToUpdate,
@@ -321,11 +311,11 @@ public abstract class AbstractCacheModeTestCase extends AbstractCacheTestCase {
         em = getEntityManagerFactory().createEntityManager();
         em.setProperty(RETRIEVE_MODE_PROP, CacheRetrieveMode.BYPASS);
         CacheableEntity ceFromDB =
-            (CacheableEntity) em.createNativeQuery("Select * from CacheableEntity where id = 1", CacheableEntity.class)
+            (CacheableEntity) em.createNativeQuery("Select * from CacheableEntity where ID = 1", CacheableEntity.class)
                 .getSingleResult();
 
         XmlCacheableEntity xceFromDB =
-            (XmlCacheableEntity) em.createNativeQuery("Select * from XmlCacheableEntity where id = 1",
+            (XmlCacheableEntity) em.createNativeQuery("Select * from XmlCacheableEntity where ID = 1",
                 XmlCacheableEntity.class).getSingleResult();
 
         assertEquals(version + 1, ceFromDB.getVersion());
@@ -347,6 +337,32 @@ public abstract class AbstractCacheModeTestCase extends AbstractCacheTestCase {
             entityManagerStoreModeTest(CacheStoreMode.USE, CacheStoreMode.USE, true, true, 1);
         }
     }
+    
+    public void testRefresh() {
+        if (getCacheEnabled()) {
+            OpenJPAEntityManagerSPI em = getEntityManagerFactory().createEntityManager();
+            CacheableEntity e1 = em.find(CacheableEntity.class, 1);
+            XmlCacheableEntity e2 = em.find(XmlCacheableEntity.class, 1);
+            assertNotNull(e1);
+            assertNotNull(e2);
+            int e1Version = e1.getVersion();
+            int e2Version = e2.getVersion();
+
+            String e1Sql = "UPDATE CacheableEntity SET VERSN=?1 WHERE ID=?2";
+            String e2Sql = "UPDATE XmlCacheableEntity SET VERSN=?1 WHERE ID=?2";
+            em.getTransaction().begin();
+            assertEquals(1, em.createNativeQuery(e1Sql).setParameter(1, e1Version + 1).setParameter(2, e1.getId())
+                .executeUpdate());
+            assertEquals(1, em.createNativeQuery(e2Sql).setParameter(1, e2Version + 1).setParameter(2, e2.getId())
+                .executeUpdate());
+            em.getTransaction().commit();
+            em.refresh(e1);
+            em.refresh(e2);
+            assertEquals(e1Version + 1, e1.getVersion());
+            assertEquals(e2Version + 1, e2.getVersion());
+            em.close();
+        }
+    }
 
     public void testStoreModeUseRefresh() {
         if (getCacheEnabled()) {
@@ -393,45 +409,50 @@ public abstract class AbstractCacheModeTestCase extends AbstractCacheTestCase {
     }
     
     public void testResultsFromQueryAreInCache() { 
-        // clear cache
-        getEntityManagerFactory().getStoreCache().evictAll();
-        getEntityManagerFactory().getQueryResultCache().evictAll();
+        if (getCacheEnabled()) {
+            // clear cache
+            getEntityManagerFactory().getStoreCache().evictAll();
+            getEntityManagerFactory().getQueryResultCache().evictAll();
 
-        EntityManager em = getEntityManagerFactory().createEntityManager();
-        String query; 
-        for(Class<?> cls : persistentTypes) {
-            query = "Select e from " + getAlias(cls) + " e";
-            List<?> res = em.createQuery(query).getResultList();
-            assertNotNull(String.format("Expected to find some results when running query %s",query), res);
-            assertTrue(String.format("Expected more than 0 results running query %s",query),res.size() != 0 ) ;
+            EntityManager em = getEntityManagerFactory().createEntityManager();
+            String query; 
+            for(Class<?> cls : persistentTypes) {
+                query = "Select e from " + getAlias(cls) + " e";
+                List<?> res = em.createQuery(query).getResultList();
+                assertNotNull(String.format("Expected to find some results when running query %s",query), res);
+                assertTrue(String.format("Expected more than 0 results running query %s",query),res.size() != 0 ) ;
+            }
+            for(Class<?> cls : getExpectedInCache()) { 
+                assertCached(getEntityManagerFactory().getCache(), cls, 1, true);
+            }
+            
+            for(Class<?> cls : getExpectedNotInCache()) { 
+                assertCached(getEntityManagerFactory().getCache(), cls, 1, false);
+            }
+            em.close();
         }
-        for(Class<?> cls : getExpectedInCache()) { 
-            assertCached(getEntityManagerFactory().getCache(), cls, 1, true);
-        }
-        
-        for(Class<?> cls : getExpectedNotInCache()) { 
-            assertCached(getEntityManagerFactory().getCache(), cls, 1, false);
-        }
-        em.close();
     }
     
     public void testResultsFromFindAreInCache() { 
-        // clear cache
-        getEntityManagerFactory().getStoreCache().evictAll();
-        getEntityManagerFactory().getQueryResultCache().evictAll();
+        if (getCacheEnabled()) {
+            // clear cache
+            getEntityManagerFactory().getStoreCache().evictAll();
+            getEntityManagerFactory().getQueryResultCache().evictAll();
 
-        EntityManager em = getEntityManagerFactory().createEntityManager();
-        for(Class<?> cls : persistentTypes) {
-            assertNotNull(String.format("Expected to find %s::%d from database or from cache", cls, 1),em.find(cls, 1));
+            EntityManager em = getEntityManagerFactory().createEntityManager();
+            for(Class<?> cls : persistentTypes) {
+                assertNotNull(String.format("Expected to find %s::%d from database or from cache", cls, 1),
+                		em.find(cls, 1));
+            }
+            for(Class<?> cls : getExpectedInCache()) { 
+                assertCached(getEntityManagerFactory().getCache(), cls, 1, true);
+            }
+
+            for(Class<?> cls : getExpectedNotInCache()) { 
+                assertCached(getEntityManagerFactory().getCache(), cls, 1, false);
+            }
+            em.close();
         }
-        for(Class<?> cls : getExpectedInCache()) { 
-            assertCached(getEntityManagerFactory().getCache(), cls, 1, true);
-        }
-        
-        for(Class<?> cls : getExpectedNotInCache()) { 
-            assertCached(getEntityManagerFactory().getCache(), cls, 1, false);
-        }
-        em.close();
     }
     
     private String getAlias(Class<?> cls) {
