@@ -64,7 +64,7 @@ public class TestDataCacheBehavesIdentical extends AbstractTestCase {
     private static final String MARKER_CACHE     = "in Object Cache";
     private static final String MARKER_DIRTY_CACHE = "in Object Cache (dirty)";
     private static long ID_COUNTER = System.currentTimeMillis();
-    
+    private static int TEST_COUNT = 0;
 
     /**
      * Sets up two EntityManagerFactory: one with DataCache another without.
@@ -76,7 +76,6 @@ public class TestDataCacheBehavesIdentical extends AbstractTestCase {
                     "openjpa.jdbc.SynchronizeMappings",    "buildSchema", 
                     "openjpa.RuntimeUnenhancedClasses",    "unsupported", 
                     "openjpa.DataCache", "true",
-                    "openjpa.RemoteCommitProvider", "sjvm",
                     "openjpa.jdbc.UpdateManager", "constraint",
                     PObject.class,
                     BidirectionalOne2OneOwner.class,
@@ -100,6 +99,18 @@ public class TestDataCacheBehavesIdentical extends AbstractTestCase {
             assertTrue(isDataCacheActive(emfWithDataCache));
             assertFalse(isDataCacheActive(emfWithoutDataCache));
         }
+        TEST_COUNT++;
+    }
+    
+    public void tearDown() throws Exception {
+        // HACK - need to manually close EMFs after all tests have run
+        if (TEST_COUNT >= 21) {
+            closeEMF(emfWithDataCache);
+            emfWithDataCache = null;
+            closeEMF(emfWithoutDataCache);
+            emfWithoutDataCache = null;
+            super.tearDown();            
+        }
     }
     
     /**
@@ -522,7 +533,7 @@ public class TestDataCacheBehavesIdentical extends AbstractTestCase {
     }
 
     public void testDeleteIsNotDetectedOnCleanRefreshWithoutLockWithDataCache() {
-        verifyDeleteDetectionOnRefresh(WITH_DATACACHE, !DIRTY, NOLOCK, NO_ERROR);
+        verifyDeleteDetectionOnRefresh(WITH_DATACACHE, !DIRTY, NOLOCK, ENTITY_NOT_FOUND_ERROR);
     }
     
     public void testDeleteIsDetectedOnCleanRefreshWithLockWithDataCache() {
