@@ -31,6 +31,7 @@ import org.apache.openjpa.persistence.datacache.common.apps.CacheObjectAChild1;
 import org.apache.openjpa.persistence.datacache.common.apps.CacheObjectB;
 import org.apache.openjpa.persistence.datacache.common.apps.
         CacheObjectWithExternalizedFields;
+import org.apache.openjpa.persistence.test.AllowFailure;
 import org.apache.openjpa.persistence.common.utils.AbstractTestCase;
 
 import org.apache.openjpa.kernel.Broker;
@@ -39,6 +40,7 @@ import org.apache.openjpa.kernel.Query;
 import org.apache.openjpa.kernel.jpql.JPQLParser;
 import org.apache.openjpa.persistence.JPAFacadeHelper;
 
+@AllowFailure(message="surefire excluded")
 public class TestJPQL2ResultsAndResultClasses
     extends AbstractTestCase {
 
@@ -104,7 +106,7 @@ public class TestJPQL2ResultsAndResultClasses
     public void tearDown()
         throws Exception {
         _factory.close();
-
+        _factory = null;
         super.tearDown();
     }
 
