@@ -19,8 +19,10 @@
 package org.apache.openjpa.persistence.kernel.common.apps;
 
 import javax.persistence.Entity;
+import javax.persistence.Table;
 
 @Entity
+@Table(name="RuntimeTest5k")
 public class RuntimeTest5 {
 
     private String name;
