@@ -46,10 +46,12 @@ import org.apache.openjpa.kernel.Broker;
 import org.apache.openjpa.meta.ClassMetaData;
 import org.apache.openjpa.persistence.JPAFacadeHelper;
 import org.apache.openjpa.persistence.OpenJPAEntityManagerFactorySPI;
+import org.apache.openjpa.persistence.OpenJPAPersistence;
 
 /**
  * Base class for Persistence TestCases. This class contains utility methods but does not maintain an EntityManager or
- * EntityManagerFactory - these tasks are left for subclasses to handle.
+ * EntityManagerFactory - these tasks are left for subclasses to handle.  Extends junit.framework.TestCase and
+ * performs NO automatic clean up of EMFs created by createEMF() or createNamedEMF().
  */
 public abstract class AbstractPersistenceTestCase extends TestCase {
     public static final String FRESH_EMF = "Creates new EntityManagerFactory";
@@ -82,6 +84,7 @@ public abstract class AbstractPersistenceTestCase extends TestCase {
     /**
      * Create an entity manager factory. Put {@link #CLEAR_TABLES} in this list to tell the test framework to delete all
      * table contents before running the tests.
+     * NOTE: Caller must close the returned EMF.
      * 
      * @param props
      *            list of persistent types used in testing and/or configuration values in the form
@@ -101,6 +104,7 @@ public abstract class AbstractPersistenceTestCase extends TestCase {
     /**
      * Create an entity manager factory for persistence unit <code>pu</code>. Put {@link #CLEAR_TABLES} in this list to
      * tell the test framework to delete all table contents before running the tests.
+     * NOTE: Caller must close the returned EMF.
      * 
      * @param props
      *            list of persistent types used in testing and/or configuration values in the form
@@ -119,6 +123,29 @@ public abstract class AbstractPersistenceTestCase extends TestCase {
         return oemf;
     }
 
+    /**
+     * Create an entity manager factory for persistence unit <code>pu</code>. Put {@link #CLEAR_TABLES} in this list to
+     * tell the test framework to delete all table contents before running the tests.
+     * NOTE: Caller must close the returned EMF.
+     * 
+     * @param props
+     *            list of persistent types used in testing and/or configuration values in the form
+     *            key,value,key,value...
+     */
+    protected OpenJPAEntityManagerFactorySPI createNamedOpenJPAEMF(final String pu,
+            String res, Map<String,Object> props) {
+        OpenJPAEntityManagerFactorySPI oemf = null;
+        Map<Object, Object> config = new HashMap<Object, Object>(System.getProperties());
+        if (props != null)
+            config.putAll(props);
+        oemf = (OpenJPAEntityManagerFactorySPI) OpenJPAPersistence.createEntityManagerFactory(pu, res, props);
+        if (oemf == null) {
+            throw new NullPointerException("Expected an OpenJPA entity manager factory " +
+                "for the persistence unit named: \"" + pu + "\"");
+        }
+        return oemf;
+    }
+
     protected Map<String, Object> getPropertiesMap(Object... props) {
         Map<String, Object> map = new HashMap<String, Object>();
         map.put("openjpa.DynamicEnhancementAgent", "false");
@@ -196,15 +223,43 @@ public abstract class AbstractPersistenceTestCase extends TestCase {
      * Safely close the given factory.
      */
     protected boolean closeEMF(EntityManagerFactory emf) {
+        boolean brc = false;
         if (emf == null || !emf.isOpen()) {
-            return false;
+            return brc;
         }
-
-        closeAllOpenEMs(emf);
-        emf.close();
-        return !emf.isOpen();
+        try {
+            closeAllOpenEMs(emf);
+        } finally {
+            emf.close();
+            brc = !emf.isOpen();
+            if (!brc) {
+                System.err.println("AbstractPersistenceTestCase().closeEMF() - EMF is still open.");
+            }
+        }
+        return brc;
     }
 
+    /**
+     * Safely close the given EM
+     * 
+     * @param em
+     * @return
+     */
+    protected boolean closeEM(EntityManager em) {
+        if (em == null || !em.isOpen()) {
+            return false;
+        }
+        if (em.getTransaction().isActive()) {
+            em.getTransaction().rollback();
+        }
+        em.close();
+        boolean brc = !em.isOpen();
+        if (!brc) {
+            System.err.println("AbstractPersistenceTestCase().closeEM() - EM is still open.");
+        }
+        return brc;
+    }
+    
     /**
      * Closes all open entity managers after first rolling back any open transactions.
      */
@@ -216,10 +271,13 @@ public abstract class AbstractPersistenceTestCase extends TestCase {
         for (Broker b : ((AbstractBrokerFactory) JPAFacadeHelper.toBrokerFactory(emf)).getOpenBrokers()) {
             if (b != null && !b.isClosed()) {
                 EntityManager em = JPAFacadeHelper.toEntityManager(b);
-                if (em.getTransaction().isActive()) {
-                    em.getTransaction().rollback();
+                if( em.getTransaction().isActive() ) {
+                	try {
+						em.getTransaction().rollback();
+					} catch (Exception e) {
+					}
                 }
-                em.close();
+                closeEM(em);
             }
         }
     }
@@ -280,17 +338,22 @@ public abstract class AbstractPersistenceTestCase extends TestCase {
             return;
         }
         EntityManager em = emf.createEntityManager();
-        em.getTransaction().begin();
-        for (ClassMetaData meta : types) {
-            if (!meta.isMapped() || meta.isEmbeddedOnly()
-                || Modifier.isAbstract(meta.getDescribedType().getModifiers()) 
-                && !isBaseManagedInterface(meta, types)) {
-                continue;
+        try {
+            em.getTransaction().begin();
+            for (ClassMetaData meta : types) {
+                if (!meta.isMapped() || meta.isEmbeddedOnly()
+                    || Modifier.isAbstract(meta.getDescribedType().getModifiers()) 
+                    && !isBaseManagedInterface(meta, types)) {
+                    continue;
+                }
+                em.createQuery("DELETE FROM " + meta.getTypeAlias() + " o").executeUpdate();
             }
-            em.createQuery("DELETE FROM " + meta.getTypeAlias() + " o").executeUpdate();
+            em.getTransaction().commit();
+        } catch (Exception e) {
+            // ignore
+        } finally {
+            closeEM(em);
         }
-        em.getTransaction().commit();
-        em.close();
     }
 
     /**
