@@ -31,6 +31,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import javax.persistence.Cache;
 import javax.persistence.EntityManager;
 import javax.persistence.LockModeType;
 import javax.persistence.Query;
@@ -171,14 +172,22 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
     }
 
     protected Log getDumpStackLog() {
-        return emf.getConfiguration().getLog("DumpStack");
+        if (emf != null) {
+            return emf.getConfiguration().getLog("DumpStack");
+        } else {
+            return null;
+        }
     }
 
     protected void logStack(Throwable t) {
         StringWriter str = new StringWriter();
         PrintWriter print = new PrintWriter(str);
         t.printStackTrace(print);
-        getDumpStackLog().trace(str.toString());
+        if (getDumpStackLog() != null) {
+            getDumpStackLog().trace(str.toString());
+        } else {
+            System.err.println(str.toString());
+        }
     }
 
     private void notifyParent() {
@@ -216,6 +225,9 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
         // OpenJPA entity manager API
         Detach,            // ();
 
+        // OpenJPA entity manager factory API
+        ClearCache,        // ()
+
         // Transaction API
         StartTx,           // ()
         CommitTx,          // ()
@@ -247,6 +259,7 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
         SaveVersion,       // ([int id])
         TestVersion,       // (int id, int increment)
         TestLockMode,      // (int id, LockModeType lockMode)
+        TestInCache,       // ([int id[, boolean expectedInCache]])
 
         Test,              // Open-ended testing actions
     };
@@ -530,6 +543,13 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
                     log.trace("Employee (after)  :" + detEmployee);
                     break;
 
+                case ClearCache:
+                    Cache cache = em.getEntityManagerFactory().getCache();
+                    if (cache != null) {
+                        cache.evictAll();
+                    }      
+                    break;
+
                 case StartTx:
                     em.getTransaction().begin();
                     break;
@@ -577,7 +597,7 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
                     }
                     if (waitTime < MinThreadWaitInMs / 2)
                         waitTime = MinThreadWaitInMs / 2;
-                    log.trace(">> Started wait for " + waitTime + " ms");
+                    log.trace(">> Started thread " + waitThreadid + " wait for " + waitTime + " ms");
                     if( waitThreadid != 0) {
                         thisThread.wait(waitTime);
                     } else {
@@ -684,41 +704,113 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
                             }
                         }
                     }
-                    String testExClass = null;
-                    Throwable curThrowable = null;
                     int threadId = threadToRun;
                     if (args.length > 1) {
                         threadId = (Integer) args[1];
                     }
-                    TestThread exThread = threads.get(threadId);
-                    curThrowable = exThread.throwable;
-                    testExClass = processException(curAction, curThrowable);
+                    if( threadId != -1 ) {
+                    	// test exception on a specific thread
+                        String testExClass = null;
+                        Throwable curThrowable = null;
+                        boolean exMatched = false;
+                        TestThread exThread = threads.get(threadId);
+                        curThrowable = exThread.throwable;
+                        testExClass = processException(exThread, curAction, curThrowable);
 
-                    boolean exMatched = false;
-                    if (expectedExceptions != null
-                        && expectedExceptions.size() > 0) {
-                        for (Class<?> expectedException :
-                            expectedExceptions) {
-                            if (matchExpectedException(curAct, expectedException,
-                                curThrowable)) {
+                        if (expectedExceptions != null
+                            && expectedExceptions.size() > 0) {
+                            for (Class<?> expectedException :
+                                expectedExceptions) {
+                                if (matchExpectedException(curAct, expectedException,
+                                    curThrowable)) {
+                                    exMatched = true;
+                                    break;
+                                }
+                            }
+                        } else {
+                            if (curThrowable == null) {
                                 exMatched = true;
-                                break;
                             }
                         }
+                        if (!exMatched) {
+                            log.trace(testExClass);
+                            if (curThrowable != null) {
+                                logStack(curThrowable);
+                            }
+                        }
+                        assertTrue(curAct + ":Expecting=" + expectedExceptions
+                            + ", Testing=" + testExClass, exMatched);
+                        exThread.throwable = null;
                     } else {
-                        if (curThrowable == null) {
-                            exMatched = true;
-                        }
+                    	// test exception in any thread; used for deadlock exception testing since db server
+                    	// decides on which thread to terminate if deadlock is detected.
+                        if (expectedExceptions == null || expectedExceptions.size() == 0) {
+                        	// Expecting no exception in all threads.
+                        	boolean noExMatched = true;
+							String aTestExClass = "[";
+							for (TestThread aThread : threads) {
+								Throwable aThrowable = aThread.throwable;
+								aTestExClass += processException(aThread, curAction, aThrowable) + ", ";
+							    if (aThrowable != null) {
+							    	noExMatched = false;
+		                            log.trace(aTestExClass);
+	                                logStack(aThrowable);
+		                            aThread.throwable = null;
+							    }
+							}
+	                        assertTrue(curAct + ":Expecting=[no exception]"
+	                                + ", Testing=" + aTestExClass + ']', noExMatched);
+						} else {
+                        	// Expecting any exception in any threads.
+							boolean aMatched = false;
+							String aTestExClass = "[";
+							for (TestThread aThread : threads) {
+								Throwable aThrowable = aThread.throwable;
+								aTestExClass += processException(aThread, curAction, aThrowable) + ", ";
+
+								for (Class<?> anExpectedException : expectedExceptions) {
+									if (matchExpectedException(curAct,
+											anExpectedException, aThrowable)) {
+										aMatched = true;
+										break;
+									}
+								}
+								if (aMatched) {
+									break;
+								} else {
+		                            if (aThrowable != null) {
+		                                logStack(aThrowable);
+			                            aThread.throwable = null;
+		                            }
+								}
+							}
+	                        if (!aMatched) {
+	                            log.trace(aTestExClass);
+	                        }
+	                        assertTrue(curAct + ":Expecting=" + expectedExceptions
+	                            + ", Testing=" + aTestExClass + "]", aMatched);
+						}
                     }
-                    if (!exMatched) {
-                        log.trace(testExClass);
-                        if (curThrowable != null) {
-                            logStack(curThrowable);
-                        }
+                    break;
+
+                case TestInCache:
+                    id = 1;
+                    boolean expectedInCache = false;
+                    if (args.length > 1) {
+                        id = (Integer)args[1];
+                    }
+                    if (args.length > 2) {
+                        expectedInCache = (Boolean)args[2];
+                    }
+                    Cache testCache = em.getEntityManagerFactory().getCache();
+                    if (testCache != null) {
+                        boolean inCache = testCache.contains(LockEmployee.class, id);                        
+                        log.trace("cache.contains(Employee("+id+")) = " + inCache);
+                        assertTrue(curAct + ":TestInCache Expecting=" + expectedInCache
+                                + ", Testing=" + inCache + "]", inCache == expectedInCache);
+                    } else {
+                        log.info(curAct+":TestInCache - No Cache found.");
                     }
-                    assertTrue(curAct + ":Expecting=" + expectedExceptions
-                        + ", Testing=" + testExClass, exMatched);
-                    exThread.throwable = null;
                     break;
 
                 case WaitAllChildren:
@@ -781,7 +873,11 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
                     break;
 
                 case Sleep:
-                    Thread.sleep((Integer) args[1]);
+                    int sleepMS = 5000;
+                    if (args.length > 1) {
+                        sleepMS = (Integer) args[1];
+                    }
+                    Thread.sleep(sleepMS);
                     break;
                 case DetachSerialize:
                     id = 1;
@@ -855,7 +951,7 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
                     log.trace("finally: commit completed");
                 }
                 } catch(Exception finalEx) {
-                    String failStr = processException(curAction, finalEx);
+                    String failStr = processException(thisThread, curAction, finalEx);
                     log.trace("Fincally:" + failStr);
                 }
             }
@@ -881,11 +977,11 @@ public abstract class SequencedActionsTest extends SQLListenerTestCase {
         return lockMode;
     }
 
-    private String processException(Act curAction, Throwable t) {
-        String failStr = "Caught exception: none";
+    private String processException(TestThread thread, Act curAction, Throwable t) {
+        String failStr = "[" + thread.threadToRun + "] Caught exception: none";
         if (t != null) {
             getLog().trace(
-                "Caught exception: " + t.getClass().getName() + ":" + t);
+            		"[" + thread.threadToRun + "] Caught exception: " + t.getClass().getName() + ":" + t);
             logStack(t);
             Throwable rootCause = t.getCause();
             failStr = "Failed on action '" + curAction + "' with exception "
