@@ -21,6 +21,11 @@ package org.apache.openjpa.persistence.lockmgr;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Random;
+import java.util.concurrent.Callable;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
 
 import javax.persistence.EntityManager;
 import javax.persistence.LockModeType;
@@ -32,7 +37,11 @@ import javax.persistence.TypedQuery;
 import junit.framework.AssertionFailedError;
 
 import org.apache.openjpa.jdbc.conf.JDBCConfiguration;
+import org.apache.openjpa.jdbc.sql.DB2Dictionary;
 import org.apache.openjpa.jdbc.sql.DBDictionary;
+import org.apache.openjpa.jdbc.sql.DerbyDictionary;
+import org.apache.openjpa.jdbc.sql.InformixDictionary;
+import org.apache.openjpa.lib.log.Log;
 import org.apache.openjpa.persistence.LockTimeoutException;
 import org.apache.openjpa.persistence.OpenJPAEntityManagerFactorySPI;
 import org.apache.openjpa.persistence.test.SQLListenerTestCase;
@@ -65,7 +74,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
         if (isTestsDisabled())
             return;
         
-        setUp(CLEAR_TABLES, Employee.class, Department.class, "openjpa.LockManager", "mixed");
+        setUp(CLEAR_TABLES, Employee.class, Department.class, VersionEntity.class, "openjpa.LockManager", "mixed");
 
         EntityManager em = null;
         em = emf.createEntityManager();
@@ -98,6 +107,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
         em.persist(e1);
         em.persist(e2);
         em.getTransaction().commit();
+        em.close();
     }
 
     /*
@@ -127,10 +137,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
             em2.find(Employee.class, 2, LockModeType.PESSIMISTIC_READ, hints);
             fail("Unexcpected find succeeded. Should throw a PessimisticLockException.");
         } catch (Throwable e) {            
-            if (!dict.supportsLockingWithMultipleTables)
-                assertError(e, PessimisticLockException.class);
-            else 
-                assertError(e, LockTimeoutException.class);
+            assertError(e, PessimisticLockException.class, LockTimeoutException.class);
         } finally {
             if (em1.getTransaction().isActive())
                 em1.getTransaction().rollback();
@@ -196,10 +203,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
             em2.find(Employee.class, 2, LockModeType.PESSIMISTIC_READ, map);
             fail("Unexcpected find succeeded. Should throw a PessimisticLockException.");
         } catch (Exception e) {
-            if (!dict.supportsLockingWithMultipleTables)
-                assertError(e, PessimisticLockException.class);
-            else
-                assertError(e, LockTimeoutException.class);
+            assertError(e, PessimisticLockException.class, LockTimeoutException.class);
         } finally {
             if (em1.getTransaction().isActive())
                 em1.getTransaction().rollback();
@@ -266,10 +270,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
             assertTrue("Test department name = 'D20'", q.get(0).getName().equals("D10")
                     || q.get(0).getName().equals("D20"));
         } catch (Exception ex) {
-            if (!dict.supportsLockingWithMultipleTables)
-                fail("Caught unexpected " + ex.getClass().getName() + ":" + ex.getMessage());
-            else
-                assertError(ex, QueryTimeoutException.class);
+            assertError(ex, QueryTimeoutException.class);
         } finally {
             if (em1.getTransaction().isActive())
                 em1.getTransaction().rollback();
@@ -294,10 +295,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
             List<Employee> q = query.getResultList();
             fail("Unexcpected find succeeded. Should throw a PessimisticLockException.");
         } catch (Exception e) {
-            if (!dict.supportsLockingWithMultipleTables)
-                assertError(e, PessimisticLockException.class);
-            else
-                assertError(e, QueryTimeoutException.class);
+            assertError(e, PessimisticLockException.class, QueryTimeoutException.class);
         } finally {
             if (em1.getTransaction().isActive())
                 em1.getTransaction().rollback();
@@ -332,10 +330,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
             assertEquals("Expected 1 element with department id=20", q.size(), 1);
             assertEquals("Test department name = 'D20'", q.get(0).getName(), "D20");
         } catch (Exception ex) {
-            if (!dict.supportsLockingWithMultipleTables)
-                fail("Caught unexpected " + ex.getClass().getName() + ":" + ex.getMessage());
-            else 
-                assertError(ex, QueryTimeoutException.class);
+            assertError(ex, QueryTimeoutException.class);
         } finally {
             if (em1.getTransaction().isActive())
                 em1.getTransaction().rollback();
@@ -360,10 +355,7 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
             List<?> q = query.getResultList();
             fail("Unexcpected find succeeded. Should throw a PessimisticLockException.");
         } catch (Exception e) {
-            if (!dict.supportsLockingWithMultipleTables)
-                assertError(e, PessimisticLockException.class);
-            else 
-                assertError(e, QueryTimeoutException.class);
+            assertError(e, PessimisticLockException.class, QueryTimeoutException.class);
         } finally {
             if (em1.getTransaction().isActive())
                 em1.getTransaction().rollback();
@@ -387,18 +379,105 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
         String firstName1 = (String) q1.getSingleResult();
         //Expected sql for Derby is:
         //SELECT t0.firstName FROM Employee t0 WHERE (t0.id = CAST(? AS BIGINT)) FOR UPDATE WITH RR
-        String SQL1 = toString(sql);
+        String SQL1 = getLastSQL(sql);
         
         // run the second time
         resetSQL();
         Query q2 = em.createQuery(jpql);
         q2.setLockMode(LockModeType.PESSIMISTIC_WRITE);
         String firstName2 = (String) q2.getSingleResult();
-        String SQL2 = toString(sql);
+        String SQL2 = getLastSQL(sql);
         assertEquals(SQL1, SQL2);
         em.getTransaction().commit();
     }
+    
+    protected Log getLog() {
+        return emf.getConfiguration().getLog("Tests");
+    }
 
+    /**
+     * This variation introduces a row level write lock in a secondary thread,
+     * issues a refresh in the main thread with a lock timeout, and expects a 
+     * LockTimeoutException.
+     */
+    public void testRefreshLockTimeout() {
+
+        // Only run this test on DB2 and Derby for now.  It could cause
+        // the test to hang on other platforms.
+        if (!(dict instanceof DerbyDictionary ||
+              dict instanceof DB2Dictionary ||
+              dict instanceof InformixDictionary)) {
+            return;
+        }
+        
+        // Informix currently requires the lock timeout to be set directly on the dictionary
+        if (dict instanceof InformixDictionary) {
+            InformixDictionary ifxDict = (InformixDictionary)((JDBCConfiguration)emf.getConfiguration()).getDBDictionaryInstance();
+            ifxDict.lockModeEnabled = true;
+            ifxDict.lockWaitSeconds = 5;
+        }
+
+        EntityManager em = emf.createEntityManager();
+        
+        resetSQL();
+        VersionEntity ve = new VersionEntity();
+        int veid = new Random().nextInt();
+        ve.setId(veid);
+        ve.setName("Versioned Entity");
+
+        em.getTransaction().begin();
+        em.persist(ve);
+        em.getTransaction().commit();
+                
+        em.getTransaction().begin();
+        // Assert that the department can be found and no lock mode is set
+        ve = em.find(VersionEntity.class, veid);
+        assertTrue(em.contains(ve));        
+        assertTrue(em.getLockMode(ve) == LockModeType.NONE);
+        em.getTransaction().commit();
+        
+        // Kick of a thread to lock the DB for update
+        ExecutorService executor = Executors.newFixedThreadPool(1);
+        Future<Boolean> result = executor.submit(new RefreshWithLock(veid, this));
+        try {
+            // Wait for the thread to lock the row
+            getLog().trace("Main: waiting");
+            synchronized (this) {
+                // The derby lock timeout is configured for 60 seconds, by default.
+                wait(70000);
+            }
+            getLog().trace("Main: done waiting");
+            Map<String,Object> props = new HashMap<String,Object>();
+            // This property does not have any effect on Derby for the locking
+            // condition produced by this test.  Instead, Derby uses the 
+            // lock timeout value specified in the config (pom.xml).  On Informix,
+            // the dictionary level timeout (set above) will be used.
+            if (!(dict instanceof InformixDictionary)) {
+                props.put("javax.persistence.lock.timeout", 5000);
+            }
+            em.getTransaction().begin();
+            getLog().trace("Main: refresh with force increment");
+            em.refresh(ve, LockModeType.PESSIMISTIC_FORCE_INCREMENT, props);  
+            getLog().trace("Main: commit");
+            em.getTransaction().commit();
+            getLog().trace("Main: done commit");
+            fail("Expected LockTimeoutException");
+        } catch (Throwable t) {
+            getLog().trace("Main: exception - " + t.getMessage(), t);
+            assertTrue( t instanceof LockTimeoutException);
+        } finally {
+            try {
+                // Wake the thread and wait for the thread to finish
+                synchronized(this) {
+                    this.notify();
+                }
+                result.get();
+            } catch (Throwable t) { 
+                fail("Caught throwable waiting for thread finish: " + t);
+            }
+        }
+    }
+        
     /**
      * Assert that an exception of proper type has been thrown. Also checks that
      * that the exception has populated the failed object.
@@ -408,12 +487,22 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
      * @param expeceted
      *            type of the exception
      */
-    void assertError(Throwable actual, Class<? extends Throwable> expected) {
-        if (!expected.isAssignableFrom(actual.getClass())) {
-            actual.printStackTrace();
-            throw new AssertionFailedError(actual.getClass().getName() + " was raised but expected "
-                    + expected.getName());
-        }
+    void assertError(Throwable actual, Class<? extends Throwable> ... expected) {
+		boolean matched = false;
+		String expectedNames = "";
+		for (Class<? extends Throwable> aExpected : expected) {
+			expectedNames += aExpected.getName() + ", ";
+			if (aExpected.isAssignableFrom(actual.getClass())) {
+				matched = true;
+			}
+		}
+		if (!matched) {
+			actual.printStackTrace();
+			throw new AssertionFailedError(actual.getClass().getName()
+					+ " was raised but expecting one of the following: ["
+					+ expectedNames.substring(0, expectedNames.length() - 2) + "]");
+		}
+
         Object failed = getFailedObject(actual);
         assertNotNull("Failed object is null", failed);
         assertNotEquals("null", failed);
@@ -435,4 +524,51 @@ public class TestPessimisticLocks extends SQLListenerTestCase {
         return null;
     }
 
+    /**
+     * Separate execution thread used to forcing a lock condition on 
+     * a row in the VersionEntity table.
+     */
+    public class RefreshWithLock implements Callable<Boolean> {
+
+        private int _id;
+        private Object _monitor;
+        
+        public RefreshWithLock(int id, Object monitor) {
+            _id = id;
+            _monitor = monitor;
+        }
+        
+        public Boolean call() throws Exception {
+            try {
+                EntityManager em = emf.createEntityManager();
+                
+                em.getTransaction().begin();
+                // Find with pessimistic force increment.  Will lock row for duration of TX.
+                VersionEntity ve = em.find(VersionEntity.class, _id, LockModeType.PESSIMISTIC_FORCE_INCREMENT);
+                assertTrue(em.getLockMode(ve) == LockModeType.PESSIMISTIC_FORCE_INCREMENT);
+                // Wake up the main thread
+                getLog().trace("Thread: wake up main thread");
+                synchronized(_monitor) {
+                    _monitor.notify();
+                }
+                // Wait up to 120 seconds for main thread to complete.  The default derby timeout is 60 seconds. 
+                try {
+                    getLog().trace("Thread: waiting up to 120 secs for notify");
+                    synchronized(_monitor) {
+                        _monitor.wait(120000);
+                    }
+                    getLog().trace("Thread: done waiting");
+                } catch (Throwable t) {
+                    getLog().trace("Unexpected thread interrupt",t);
+                }
+                
+                em.getTransaction().commit();
+                em.close();
+                getLog().trace("Thread: done");
+            } catch (Throwable t) {
+                getLog().trace("Thread: caught - " + t.getMessage(), t);
+            }
+            return Boolean.TRUE;
+        }
+    }
 }
