@@ -62,7 +62,6 @@ import static org.apache.openjpa.persistence.MetaDataTag.PRE_UPDATE;
 import static org.apache.openjpa.persistence.MetaDataTag.QUERIES;
 import static org.apache.openjpa.persistence.MetaDataTag.QUERY;
 import static org.apache.openjpa.persistence.MetaDataTag.READ_ONLY;
-import static org.apache.openjpa.persistence.MetaDataTag.REPLICATED;
 import static org.apache.openjpa.persistence.MetaDataTag.SEQ_GENERATOR;
 import static org.apache.openjpa.persistence.MetaDataTag.TYPE;
 import static org.apache.openjpa.persistence.MetaDataTag.VERSION;
@@ -85,6 +84,7 @@ import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Map;
+import java.util.Properties;
 import java.util.Set;
 import java.util.Stack;
 import java.util.TreeSet;
@@ -235,7 +235,6 @@ public class AnnotationPersistenceMetaDataParser
         _tags.put(ManagedInterface.class, MANAGED_INTERFACE);
         _tags.put(ReadOnly.class, READ_ONLY);
         _tags.put(Type.class, TYPE);
-        _tags.put(Replicated.class, REPLICATED);
     }
 
     private final OpenJPAConfiguration _conf;
@@ -647,9 +646,6 @@ public class AnnotationPersistenceMetaDataParser
                     if (isMetaDataMode())
                         parseManagedInterface(meta, (ManagedInterface) anno);
                     break;
-                case REPLICATED:
-                	meta.setReplicated(true);
-                	break;
                 case ACCESS:
                     if (isMetaDataMode())
                         parseAccess(meta, (Access)anno);
@@ -737,12 +733,18 @@ public class AnnotationPersistenceMetaDataParser
      */
     private ClassMetaData getMetaData() {
         ClassMetaData meta = getRepository().getCachedMetaData(_cls);
-        if (meta != null &&
-            ((isMetaDataMode() && (meta.getSourceMode() & MODE_META) != 0) ||
-                (isMappingMode() &&
-                    (meta.getSourceMode() & MODE_MAPPING) != 0))) {
-            if (_log.isWarnEnabled())
+        if (meta != null
+            && ((isMetaDataMode() 
+                && (meta.getSourceMode() & MODE_META) != 0) 
+                || (isMappingMode() && (meta.getSourceMode() & MODE_MAPPING) != 0) ) ) {
+            if (_log.isWarnEnabled()) {
                 _log.warn(_loc.get("dup-metadata", _cls.getName()));
+            }
+            if(_log.isTraceEnabled()) { 
+                _log.trace(String.format(
+                    "MetaData originally obtained from file: %s under mode :%d with scope %s, and type :%d",
+                    meta.getSourceName(), meta.getSourceMode(), meta.getSourceScope(), meta.getSourceType()));
+            }
             return null;
         }
 
@@ -750,7 +752,8 @@ public class AnnotationPersistenceMetaDataParser
             meta = getRepository().addMetaData(_cls, getAccessCode(_cls));
             meta.setEnvClassLoader(_envLoader);
             meta.setSourceMode(MODE_NONE);
-            meta.setSource(getSourceFile(), SourceTracker.SRC_ANNOTATIONS);
+            meta.setSource(getSourceFile(), SourceTracker.SRC_ANNOTATIONS, getSourceFile() == null ? ""
+                : getSourceFile().getPath());
         }
         return meta;
     }
@@ -812,24 +815,29 @@ public class AnnotationPersistenceMetaDataParser
      * Parse @DataStoreId.
      */
     private void parseDataStoreId(ClassMetaData meta, DataStoreId id) {
+        parseDataStoreId(meta, id.strategy(), id.generator());
+    }
+    
+    static void parseDataStoreId(ClassMetaData meta, GenerationType strategy, 
+        String generator) {
         meta.setIdentityType(ClassMetaData.ID_DATASTORE);
 
-        int strat = getGeneratedValueStrategy(meta, id.strategy(),
-            id.generator());
+        int strat = getGeneratedValueStrategy(meta, strategy,
+            generator);
         if (strat != -1)
             meta.setIdentityStrategy(strat);
         else {
-            switch (id.strategy()) {
+            switch (strategy) {
                 case TABLE:
                 case SEQUENCE:
                     // technically we should have separate system table and
                     // sequence generators, but it's easier to just rely on
                     // the system org.apache.openjpa.Sequence setting for both
-                    if (StringUtils.isEmpty(id.generator()))
+                    if (StringUtils.isEmpty(generator))
                         meta.setIdentitySequenceName(
                             SequenceMetaData.NAME_SYSTEM);
                     else
-                        meta.setIdentitySequenceName(id.generator());
+                        meta.setIdentitySequenceName(generator);
                     break;
                 case AUTO:
                     meta.setIdentityStrategy(ValueStrategies.NATIVE);
@@ -838,11 +846,11 @@ public class AnnotationPersistenceMetaDataParser
                     meta.setIdentityStrategy(ValueStrategies.AUTOASSIGN);
                     break;
                 default:
-                    throw new UnsupportedException(id.strategy().toString());
+                    throw new UnsupportedException(strategy.toString());
             }
         }
     }
-
+    
     /**
      * Warn that @FlushMode is not supported.
      */
@@ -856,14 +864,21 @@ public class AnnotationPersistenceMetaDataParser
      * 
      */
     private void parseDataCache(ClassMetaData meta, DataCache cache) {
-        if (cache.timeout() != Integer.MIN_VALUE) {
-            meta.setDataCacheTimeout(cache.timeout());
+        parseDataCache(meta, cache.enabled(), cache.name(), cache.timeout());
+    }
+    
+    static void parseDataCache(ClassMetaData meta,
+            boolean enabled,
+            String name,
+            int timeout) {
+        if (timeout != Integer.MIN_VALUE) {
+            meta.setDataCacheTimeout(timeout);
         }
-        String cacheName = cache.name();
+        String cacheName = name;
         if (StringUtils.isEmpty(cacheName)) {
             cacheName = org.apache.openjpa.datacache.DataCache.NAME_DEFAULT;
         }
-        meta.setDataCacheName(cache.enabled() ? cacheName : null);
+        meta.setDataCacheName(enabled ? cacheName : null);
     }
 
     private void parseManagedInterface(ClassMetaData meta,
@@ -1025,53 +1040,83 @@ public class AnnotationPersistenceMetaDataParser
      * A and B. Both A and B must be declared in the same Class.
      * <br>
      * Call {@link #parseFetchAttribute(ClassMetaData,
-     * org.apache.openjpa.meta.FetchGroup, FetchAttribute) only after the
+     * org.apache.openjpa.meta.FetchGroup, FetchAttribute)} only after the
      * bi-links have been established, because a field f will not only add the
-     * fetch group A which explictly includes f to its custom fetch groups but
+     * fetch group A which explicitly includes f to its custom fetch groups but
      * also will also add any fetch group B that includes A.
      */
-    private void parseFetchGroups(ClassMetaData meta, FetchGroup... groups) {
-        org.apache.openjpa.meta.FetchGroup fg;
-        for (FetchGroup group : groups) {
-            if (StringUtils.isEmpty(group.name()))
-                throw new MetaDataException(_loc.get("unnamed-fg", meta));
-
-            fg = meta.addDeclaredFetchGroup(group.name());
-            if (group.postLoad())
-                fg.setPostLoad(true);
-            for (String s : group.fetchGroups()) {
-                fg.addDeclaredInclude(s);
-            }
-        }
-        // Add the parent-child style bi-links between fetch groups in a
-        // separate pass.
-        for (FetchGroup group:groups) {
-        	fg = meta.getFetchGroup(group.name());
-        	String[] includedFetchGropNames = fg.getDeclaredIncludes();
-        	for (String includedFectchGroupName:includedFetchGropNames) {
-        		org.apache.openjpa.meta.FetchGroup child =
-        	    meta.getFetchGroup(includedFectchGroupName);
-        		if (child == null)
-                    throw new UserException(_loc.get("missing-included-fg",
-                            meta.getDescribedType().getName(), fg.getName(),
-                            includedFectchGroupName));
-        		child.addContainedBy(fg);
-        	}
-        }
-
-        for (FetchGroup group : groups) {
-            fg = meta.getFetchGroup(group.name());
-            for (FetchAttribute attr : group.attributes())
-                parseFetchAttribute(meta, fg, attr);
-        }
+    static void parseFetchGroups(ClassMetaData meta, FetchGroup... groups) {
+       FetchGroupImpl[] fetchGroupImpls = new FetchGroupImpl[groups.length];
+       for (int i = 0; i < groups.length; i++) {
+           FetchAttribute[] fetchAttributes = groups[i].attributes();
+           FetchAttributeImpl[] fetchAttributeImpls= null;
+           if (fetchAttributes != null && fetchAttributes.length > 0) {
+               fetchAttributeImpls = new FetchAttributeImpl[fetchAttributes.length];
+               for (int j = 0; j < fetchAttributes.length; j++) {
+                   fetchAttributeImpls[j] = 
+                           new FetchAttributeImpl(fetchAttributes[j].name(), fetchAttributes[j].recursionDepth()); 
+               }
+           }
+           fetchGroupImpls[i] = new FetchGroupImpl(groups[i].name(), groups[i].postLoad());
+           if (fetchAttributeImpls != null) {
+               fetchGroupImpls[i].setAttributes(fetchAttributeImpls);
+           }
+           if (groups[i].fetchGroups() != null) {
+               fetchGroupImpls[i].setFetchGroups(groups[i].fetchGroups());
+           }
+       }
+       
+       parseFetchGroups(meta, fetchGroupImpls);
     }
+   
+   /**
+    * Parse fetch group input for the FetchGroup and FetchGroups annotations
+    * as well as for the fetch-group and fetch-groups XML metadata
+    * @param meta
+    * @param groups
+    */
+   static void parseFetchGroups(ClassMetaData meta, FetchGroupImpl... groups) {
+       org.apache.openjpa.meta.FetchGroup fg;
+       for (FetchGroupImpl group : groups) {
+           if (StringUtils.isEmpty(group.name()))
+               throw new MetaDataException(_loc.get("unnamed-fg", meta));
+
+           fg = meta.addDeclaredFetchGroup(group.name());
+           if (group.postLoad())
+               fg.setPostLoad(true);
+           for (String s : group.fetchGroups()) {
+               fg.addDeclaredInclude(s);
+           }
+       }
+       // Add the parent-child style bi-links between fetch groups in a
+       // separate pass.
+       for (FetchGroupImpl group:groups) {
+           fg = meta.getFetchGroup(group.name());
+           String[] includedFetchGropNames = fg.getDeclaredIncludes();
+           for (String includedFectchGroupName:includedFetchGropNames) {
+               org.apache.openjpa.meta.FetchGroup child =
+               meta.getFetchGroup(includedFectchGroupName);
+               if (child == null)
+                   throw new UserException(_loc.get("missing-included-fg",
+                           meta.getDescribedType().getName(), fg.getName(),
+                           includedFectchGroupName));
+               child.addContainedBy(fg);
+           }
+       }
+
+       for (FetchGroupImpl group : groups) {
+           fg = meta.getFetchGroup(group.name());
+           for (FetchAttributeImpl attr : group.attributes())
+               parseFetchAttribute(meta, fg, attr);
+       }
+   }
 
 
     /**
      * Set a field's fetch group.
      */
-    private void parseFetchAttribute(ClassMetaData meta,
-        org.apache.openjpa.meta.FetchGroup fg, FetchAttribute attr) {
+    private static void parseFetchAttribute(ClassMetaData meta,
+        org.apache.openjpa.meta.FetchGroup fg, FetchAttributeImpl attr) {
         FieldMetaData field = meta.getDeclaredField(attr.name());
         if (field == null
             || field.getManagement() != FieldMetaData.MANAGE_PERSISTENT)
@@ -1289,7 +1334,7 @@ public class AnnotationPersistenceMetaDataParser
     /**
      * Convert the given class to its OpenJPA type override equivalent.
      */
-    private static Class<?> toOverrideType(Class<?> cls) {
+    public static Class<?> toOverrideType(Class<?> cls) {
         return (cls == Entity.class)
             ? org.apache.openjpa.enhance.PersistenceCapable.class : cls;
     }
@@ -1510,12 +1555,12 @@ public class AnnotationPersistenceMetaDataParser
             case JavaTypes.ARRAY:
             case JavaTypes.COLLECTION:
             case JavaTypes.MAP:
-                if (JavaTypes.maybePC(fmd.getElement()))
+                if (fmd.getDeclaredType() == Properties.class || JavaTypes.maybePC(fmd.getElement()))
                     break;
                 // no break
             default:
                 throw new MetaDataException(_loc.get("bad-meta-anno", fmd,
-                    "OneToMany"));
+                    "ManyToMany"));
         }
 
         fmd.setInDefaultFetchGroup(anno.fetch() == FetchType.EAGER);
@@ -1631,6 +1676,8 @@ public class AnnotationPersistenceMetaDataParser
             throw new MetaDataException(_loc.get("bad-meta-anno", fmd,
                 "PersistentCollection"));
 
+        fmd.setPersistentCollection(true); 
+        
         if (!StringUtils.isEmpty(anno.mappedBy()))
             fmd.setMappedBy(anno.mappedBy());
         fmd.setInDefaultFetchGroup(anno.fetch() == FetchType.EAGER);
@@ -1819,17 +1866,17 @@ public class AnnotationPersistenceMetaDataParser
                 continue;
             }
             meta = getRepository().addQueryMetaData(_cls, query.name());
-            meta.setQueryString(query.query());
             meta.setLanguage(JPQLParser.LANG_JPQL);
+            meta.setQueryString(query.query());
             for (QueryHint hint : query.hints())
                 meta.addHint(hint.name(), hint.value());
             LockModeType lmt = processNamedQueryLockModeType(query);
-            if (lmt != null) {
+            if (lmt != null && lmt != LockModeType.NONE) {
                 meta.addHint("openjpa.FetchPlan.ReadLockMode", lmt);
             }
 
             meta.setSource(getSourceFile(), (el instanceof Class) ? el : null,
-                SourceTracker.SRC_ANNOTATIONS);
+                SourceTracker.SRC_ANNOTATIONS, getSourceFile() == null ? "" : getSourceFile().getPath());
             if (isMetaDataMode())
                 meta.setSourceMode(MODE_META);
             else if (isMappingMode())
@@ -1892,8 +1939,8 @@ public class AnnotationPersistenceMetaDataParser
             }
 
             meta = getRepository().addQueryMetaData(null, query.name());
-            meta.setQueryString(query.query());
             meta.setLanguage(QueryLanguages.LANG_SQL);
+            meta.setQueryString(query.query());
             Class<?> res = query.resultClass();
             if (ImplHelper.isManagedType(getConfiguration(), res))
                 meta.setCandidateType(res);
@@ -1906,7 +1953,7 @@ public class AnnotationPersistenceMetaDataParser
                 meta.addHint(hint.name(), hint.value());
 
             meta.setSource(getSourceFile(), (el instanceof Class) ? el : null,
-                SourceTracker.SRC_ANNOTATIONS);
+                SourceTracker.SRC_ANNOTATIONS, getSourceFile() == null ? "" : getSourceFile().getPath());
             if (isMetaDataMode())
                 meta.setSourceMode(MODE_META);
             else if (isMappingMode())
@@ -1978,6 +2025,74 @@ public class AnnotationPersistenceMetaDataParser
 			return compare;
 		}
 	}
+    
+    /**
+     * An internal class used to mimic the FetchGroup annotation.
+     * This is needed to process the fetch-group element in xml
+     * metadata with common code for the annotation.
+     */
+    static class FetchGroupImpl {
+        private String name = "";
+        private boolean postLoad = false;
+        private FetchAttributeImpl[] attributes = {};
+        private String[] fetchGroups = {};
+        
+        FetchGroupImpl(String name, boolean postLoad)
+        {
+            this.name = name;
+            this.postLoad = postLoad;
+        }
+
+        public String name() {
+            return name;
+        }
+
+        public boolean postLoad() {
+            return postLoad;
+        }
+
+        public FetchAttributeImpl[] attributes() {
+            return attributes;
+        }
+
+        public String[] fetchGroups() {
+            return fetchGroups;
+        }
+
+
+        public void setAttributes(FetchAttributeImpl[] attributes) {
+            this.attributes = attributes;
+        }
+
+        public void setFetchGroups(String[] fetchGroups) {
+            this.fetchGroups = fetchGroups;
+        }
+    }
+    
+    /**
+     * An internal class used to mimic the FetchAttribute annotation.
+     * This is needed to process the fetch-attribute element in xml
+     * metadata with common code for the annotation.
+     */
+    static class FetchAttributeImpl {
+        private String name = "";
+        private int recursionDepth = Integer.MIN_VALUE;
+        
+        public FetchAttributeImpl(String name, int recursionDepth) {
+            this.name = name;
+            this.recursionDepth = recursionDepth;
+        }
+        
+        public String name() {
+            return name;
+        }
+
+        public int recursionDepth() {
+            return recursionDepth;
+        }
+
+        
+    }
 
     protected String normalizeSequenceName(String seqName) {
         return seqName;
