@@ -20,7 +20,6 @@ package org.apache.openjpa.persistence;
 
 import java.lang.instrument.ClassFileTransformer;
 import java.lang.instrument.IllegalClassFormatException;
-import java.security.AccessController;
 import java.security.ProtectionDomain;
 import java.util.HashMap;
 import java.util.Map;
@@ -37,16 +36,14 @@ import org.apache.openjpa.conf.OpenJPAConfiguration;
 import org.apache.openjpa.conf.OpenJPAConfigurationImpl;
 import org.apache.openjpa.enhance.PCClassFileTransformer;
 import org.apache.openjpa.enhance.PCEnhancerAgent;
-import org.apache.openjpa.kernel.AbstractBrokerFactory;
 import org.apache.openjpa.kernel.Bootstrap;
 import org.apache.openjpa.kernel.BrokerFactory;
+import org.apache.openjpa.kernel.ConnectionRetainModes;
 import org.apache.openjpa.lib.conf.Configuration;
 import org.apache.openjpa.lib.conf.ConfigurationProvider;
 import org.apache.openjpa.lib.conf.Configurations;
 import org.apache.openjpa.lib.log.Log;
-import org.apache.openjpa.lib.util.J2DoPrivHelper;
 import org.apache.openjpa.lib.util.Localizer;
-import org.apache.openjpa.lib.util.Options;
 import org.apache.openjpa.meta.AbstractCFMetaDataFactory;
 import org.apache.openjpa.meta.MetaDataModes;
 import org.apache.openjpa.meta.MetaDataRepository;
@@ -91,7 +88,7 @@ public class PersistenceProviderImpl
                 return null;
             }
 
-            BrokerFactory factory = getBrokerFactory(cp, poolValue, null);
+            BrokerFactory factory = getBrokerFactory(cp, poolValue, BundleUtils.getBundleClassLoader());
             OpenJPAConfiguration conf = factory.getConfiguration();
             _log = conf.getLog(OpenJPAConfiguration.LOG_RUNTIME);            
             pd.checkPuNameCollisions(_log,name);
@@ -102,14 +99,16 @@ public class PersistenceProviderImpl
             // Create appropriate LifecycleEventManager
             loadValidator(factory);
             
-            // We need to wait to preload until after we get back a fully configured/instantiated
-            // BrokerFactory. This is because it is possible that someone has extended OpenJPA
-            // functions and they need to be allowed time to configure themselves before we go off and
-            // start instanting configurable objects (ie:openjpa.MetaDataRepository). Don't catch
-            // any exceptions here because we want to fail-fast.
-            preloadMetaDataRepository(factory);
+            if (conf.getConnectionRetainModeConstant() == ConnectionRetainModes.CONN_RETAIN_ALWAYS) {
+                // warn about EMs holding on to connections.
+                _log.warn(_loc.get("retain-always", conf.getId()));
+            }
             
-            return JPAFacadeHelper.toEntityManagerFactory(factory);
+            OpenJPAEntityManagerFactory emf = JPAFacadeHelper.toEntityManagerFactory(factory);
+            if (_log.isTraceEnabled()) {
+                _log.trace(this + " creating " + emf + " for PU " + name + ".");
+            }
+            return emf;
         } catch (Exception e) {
             if (_log != null) {
                 _log.error(_loc.get("create-emf-error", name), e);
@@ -199,18 +198,21 @@ public class PersistenceProviderImpl
                     _log.warn(_loc.get("transformer-registration-error", pui));
                 }
             }
+            
+            if (conf.getConnectionRetainModeConstant() == ConnectionRetainModes.CONN_RETAIN_ALWAYS) {
+                // warn about container managed EMs holding on to connections.
+                _log.warn(_loc.get("cm-retain-always",conf.getId()));
+            }
 
             // Create appropriate LifecycleEventManager
             loadValidator(factory);
             
-            // We need to wait to preload until after we get back a fully configured/instantiated
-            // BrokerFactory. This is because it is possible that someone has extended OpenJPA
-            // functions and they need to be allowed time to configure themselves before we go off and
-            // start instanting configurable objects (ie:openjpa.MetaDataRepository). Don't catch
-            // any exceptions here because we want to fail-fast.
-            preloadMetaDataRepository(factory);
+            OpenJPAEntityManagerFactory emf = JPAFacadeHelper.toEntityManagerFactory(factory);
+            if (_log.isTraceEnabled()) {
+                _log.trace(this + " creating container " + emf + " for PU " + pui.getPersistenceUnitName() + ".");
+            }
             
-            return JPAFacadeHelper.toEntityManagerFactory(factory);
+            return emf;
         } catch (Exception e) {
             throw PersistenceExceptions.toPersistenceException(e);
         }
@@ -256,31 +258,7 @@ public class PersistenceProviderImpl
     protected OpenJPAConfiguration newConfigurationImpl() {
         return new OpenJPAConfigurationImpl();
     }
-
-    /**
-     * Private worker method that will call to the MetaDataRepository to preload if the provided
-     * BrokerFactory is configured to do so.
-     */
-    private void preloadMetaDataRepository(BrokerFactory factory){
-        // We need to wait to preload until after we get back a fully configured/instantiated
-        // BrokerFactory. This is because it is possible that someone has extended OpenJPA
-        // functions and they need to be allowed time to configure themselves before we go off and
-        // start instantiating configurable objects (ie:openjpa.MetaDataRepository). Don't catch
-        // any exceptions here because we want to fail-fast.
-        OpenJPAConfiguration conf = factory.getConfiguration();
-        Options o = Configurations.parseProperties(Configurations.getProperties(conf.getMetaDataRepository()));
-        if (MetaDataRepository.needsPreload(o) == true) {
-            MetaDataRepository mdr = conf.getMetaDataRepositoryInstance(); 
-            mdr.setValidate(MetaDataRepository.VALIDATE_RUNTIME, true);
-            mdr.setResolve(MetaDataRepository.MODE_MAPPING_INIT, true);
-            
-            // Load persistent classes and hook in subclasser
-            ((AbstractBrokerFactory) factory).loadPersistentTypes((ClassLoader) AccessController
-                .doPrivileged(J2DoPrivHelper.getContextClassLoaderAction()));
-            mdr.preload();
-        }
-    }
-    
+   
     /**
      * Java EE 5 class transformer.
      */
