@@ -19,6 +19,7 @@
 package org.apache.openjpa.persistence;
 
 import java.io.File;
+import java.lang.reflect.Method;
 import java.net.MalformedURLException;
 import java.net.URISyntaxException;
 import java.net.URL;
@@ -41,7 +42,6 @@ import javax.persistence.spi.PersistenceUnitInfo;
 import javax.persistence.spi.PersistenceUnitTransactionType;
 import javax.sql.DataSource;
 
-import org.apache.openjpa.datacache.DataCacheMode;
 import org.apache.openjpa.lib.conf.Configuration;
 import org.apache.openjpa.lib.conf.Configurations;
 import org.apache.openjpa.lib.conf.ProductDerivations;
@@ -62,18 +62,12 @@ public class PersistenceUnitInfoImpl
 
     public static final String PERSISTENCE_VERSION = "PersistenceVersion";
     
-    /**
-     * Properties key for the SHARED_CACHE_MODE. The JPA 2.0 spec
-     * SharedCacheMode maps to OpenJPA's DataCacheMode so we're using that
-     * class' simple name as the property key.
-     */
-//    public static final String SHARED_CACHE_MODE=DataCacheMode.class.getSimpleName();  
 
     private static final Localizer s_loc = Localizer.forPackage
         (PersistenceUnitInfoImpl.class);
 
     private String _name;
-    private final Properties _props = new Properties();
+    private final HashMap<Object,Object> _props = new HashMap<Object, Object>();
     private PersistenceUnitTransactionType _transType =
         PersistenceUnitTransactionType.RESOURCE_LOCAL;
 
@@ -81,6 +75,7 @@ public class PersistenceUnitInfoImpl
     private List<String> _mappingFileNames;
     private List<String> _entityClassNames;
     private List<URL> _jarFiles;
+    private List<String> _jarFileNames;
     private String _jtaDataSourceName;
     private DataSource _jtaDataSource;
     private String _nonJtaDataSourceName;
@@ -219,8 +214,34 @@ public class PersistenceUnitInfoImpl
     }
 
     public void addJarFileName(String name) {
+        // Defer searching the classpath for jar files referenced by the jar-file element until after
+        // the XML has been parsed and it has been confirmed that OpenJPA is the desired JPA provider.
+
+        if (_jarFileNames == null) {
+            _jarFileNames = new ArrayList<String>();
+        }
+        _jarFileNames.add(name);
+    }
+
+    /**
+     * Process jar-file elements. An IllegalArgumentException may be thrown if the jar file does not exist in the
+     * classpath.
+     */
+    public void processJarFileNames() {
+        if (_jarFileNames != null) {
+            for (String name : _jarFileNames) {
+                validateJarFileName(name);
+            }
+
+            _jarFileNames.clear();
+        }
+    }
+    
+    public void validateJarFileName(String name) {
+        ClassLoader contextClassLoader = AccessController.doPrivileged(J2DoPrivHelper.getContextClassLoaderAction());
         MultiClassLoader loader = AccessController
             .doPrivileged(J2DoPrivHelper.newMultiClassLoaderAction());
+        loader.addClassLoader(contextClassLoader);
         loader.addClassLoader(getClass().getClassLoader());
         loader.addClassLoader(MultiClassLoader.THREAD_LOADER);
         URL url = AccessController.doPrivileged(
@@ -231,9 +252,28 @@ public class PersistenceUnitInfoImpl
         }
 
         // jar file is not a resource; check classpath
-        String[] cp = (AccessController.doPrivileged(
-            J2DoPrivHelper.getPropertyAction("java.class.path"))) 
-            .split(J2DoPrivHelper.getPathSeparator());
+        String classPath = null;        
+
+        //first check if the classpath is set from ant class loader
+        if (contextClassLoader instanceof MultiClassLoader) {
+            for (ClassLoader classLoader : ((MultiClassLoader) contextClassLoader).getClassLoaders()){
+                try {
+                    Method getClassPathMethod = classLoader.getClass().getMethod("getClasspath", new Class[]{});
+                    classPath = (String) getClassPathMethod.invoke(classLoader, new Object[]{});
+                    if (classPath != null) 
+                        break;
+                } catch (Exception e) {
+                    //do nothing
+                } 
+            }                
+        }                
+        
+        if (classPath == null) {
+            classPath = AccessController.doPrivileged(
+                    J2DoPrivHelper.getPropertyAction("java.class.path"));
+        }
+        String[] cp = classPath.split(J2DoPrivHelper.getPathSeparator());
+
         for (int i = 0; i < cp.length; i++) {
             if (cp[i].equals(name)
                 || cp[i].endsWith(File.separatorChar + name)) {
@@ -266,11 +306,13 @@ public class PersistenceUnitInfoImpl
     }
 
     public Properties getProperties() {
-        return _props;
+        Properties copy = new Properties();
+        copy.putAll(_props);
+        return copy;
     }
 
     public void setProperty(String key, String value) {
-        _props.setProperty(key, value);
+        _props.put(key, value);
     }
 
     public void addTransformer(ClassTransformer transformer) {
