@@ -461,7 +461,7 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
      * Gets the current context.
      */
     Context ctx() {
-        return _contexts.peek();
+        return _contexts == null || _contexts.isEmpty() ? null :  _contexts.peek();
     }
     
     //
@@ -494,7 +494,8 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
      */
     public void registerVariable(Selection<?> node, Value var, Value path) {
         if (isRegistered(node)) {
-            throw new RuntimeException(node + " is already bound");
+            return;
+            //throw new RuntimeException(node + " is already bound");
         }
         if (!var.isVariable())
             throw new RuntimeException(var.getClass() + " is not a variable");
@@ -723,4 +724,10 @@ class CriteriaQueryImpl<T> implements OpenJPACriteriaQuery<T>, AliasContext {
             return super.toString();
         }
     }
+    
+    public boolean equals(Object other) {
+        if (toString().equals(other.toString()))
+            return true;
+        return false;
+    }
 }
