@@ -570,13 +570,10 @@ class Expressions {
             Value locateSearch = path.toValue(factory, q);
             Value locateFromIndex = (from == null ? null : Expressions.toValue(from, factory, q));
             Value locatePath = Expressions.toValue(pattern, factory, q);
-            
-            return factory.add(factory.indexOf(locateSearch,
-                    locateFromIndex == null ? locatePath
-                        : factory.newArgumentList(locatePath,
-                            factory.subtract(locateFromIndex, 
-                                             factory.newLiteral(Integer.valueOf(1), Literal.TYPE_NUMBER)))),
-                                             factory.newLiteral(Integer.valueOf(1), Literal.TYPE_NUMBER));
+
+            return factory.indexOf(locateSearch,
+                locateFromIndex == null ? locatePath
+                    : factory.newArgumentList(locatePath, locateFromIndex));
         }
         
         public void acceptVisit(CriteriaExpressionVisitor visitor) {
@@ -590,7 +587,7 @@ class Expressions {
     }
     
     public static class Trim extends BinarayFunctionalExpression<String> {
-        static Expression<Character> defaultTrim = new Constant<Character>(Character.class, new Character(' '));
+        static Expression<Character> defaultTrim = new Constant<Character>(Character.class, Character.valueOf(' '));
         static Trimspec defaultSpec = Trimspec.BOTH;
         private Trimspec ts;
         
@@ -629,6 +626,8 @@ class Expressions {
                 case BOTH     : spec = null;  break;
                 }
             }
+            Character t = (Character)((Constant<Character>)e2).arg;
+            Constant<String> e2 = new Constant<String>(String.class, t.toString());
             return factory.trim(
                 Expressions.toValue(e1, factory, q), 
                 Expressions.toValue(e2, factory, q), spec);
