@@ -105,8 +105,6 @@ public class SourceCode {
      */
 	private boolean addImport(ClassName name) {
 		String pkgName = name.getPackageName();
-		if ("java.lang".equals(pkgName))
-			return false;
 		for (Import i : imports) {
 		    if (i.getClassName().hides(name)) {
 		        i.getClassName().useFullName();
@@ -127,7 +125,7 @@ public class SourceCode {
 	 * @param name fully-qualified name of a class
 	 * @return an existing class name instance or a new one. 
 	 */
-	ClassName getOrCreateImport(String name) {
+	public ClassName getOrCreateImport(String name) {
 	    for (Import i : imports) {
 	        if (i.name.getFullName().equals(name))
 	            return i.name;
@@ -141,14 +139,20 @@ public class SourceCode {
 	public SourceCode addComment(boolean inline, String... lines) {
 	    if (lines == null)
 	        return this;
-	    if (lines.length == 1 && lines[0].length() > 120)
-	        return addComment(inline, wrap(lines[0], 120-4));
 		if (comments == null) 
 		    comments = new ArrayList<Comment>();
 		Comment comment = new Comment();
 		comments.add(comment);
 		comment.makeInline(inline);
-		for (String line:lines) comment.append(line);
+		for (String line:lines) {
+		    // Handle long header comment lines...
+		    if (line.length() > 120-4) {
+		        String[] wrappedLines = wrap(line, 120-4);
+		        for (String w:wrappedLines) comment.append(w);
+		    } else {
+	            comment.append(line);
+		    }
+		}
 		return this;
 	}
 	
@@ -357,11 +361,13 @@ public class SourceCode {
 	 *
 	 */
 	public class Class extends Element<Class> {
-		private boolean isAbstract;
+        private boolean isAbstract;
+		private boolean isFinal;
 		private ClassName superCls;
 		private List<ClassName> interfaces = new ArrayList<ClassName>();
 	    private Set<Field> fields   = new TreeSet<Field>();
 	    private Set<Method> methods = new TreeSet<Method>();
+	    private Set<Constructor> constructors = new TreeSet<Constructor>();
 		
 		public Class(String name) {
 			super(name, getOrCreateImport(name));
@@ -378,10 +384,20 @@ public class SourceCode {
 			return this;
 		}
 		
-		public Class makeAbstract() {
-			isAbstract = true;
-			return this;
-		}
+        public Class makeAbstract() {
+            if (isFinal)
+                throw new IllegalArgumentException(_loc.get("src-invalid-modifier").toString());
+
+            isAbstract = true;
+            return this;
+        }
+
+        public Class makeFinal() {
+            if (isAbstract)
+                throw new IllegalArgumentException(_loc.get("src-invalid-modifier").toString());
+            isFinal = true;
+            return this;
+        }
 		
 	    /**
 	     * Adds getters and setters to every non-public field.
@@ -433,10 +449,19 @@ public class SourceCode {
 	        return method;
 	    }
 
+	    public Constructor addConstructor(){
+	        Constructor c = new Constructor(type.simpleName);
+	           if (!constructors.add(c)) 
+	                throw new IllegalArgumentException(_loc.get(
+	                    "src-duplicate-constructor", c, this).toString());
+	            return c;
+	    }
 	    public void write(PrintWriter out, int tab) {
 			super.write(out, tab);
 			if (isAbstract) 
 			    out.append("abstract ");
+			if(isFinal)
+			    out.append("final ");
 			out.print("class ");
 			out.print(type.simpleName);
 			writeList(out, BLANK, params, PARAMS_DELIMITER, false);
@@ -446,6 +471,9 @@ public class SourceCode {
 			out.println(SPACE + BLOCK_DELIMITER.start);
 	        for (Field field:fields) 
 	            field.write(out, 1);
+	        for(Constructor ctor : constructors){
+	            ctor.write(out, 1);
+	        }
 	        for (Method method:methods) 
 	            method.write(out, 1);
 	        out.println(BLOCK_DELIMITER.end);
@@ -530,11 +558,13 @@ public class SourceCode {
 	 * 
 	 *
 	 */
-	class Method  extends Element<Method> {
+	public class Method extends Element<Method> {
 		private boolean isAbstract;
 		private List<Argument<ClassName,String>> args = new ArrayList<Argument<ClassName,String>>();
 		private List<String> codeLines = new ArrayList<String>();
-		
+		int tabCount = 0;
+		String tab = "";
+
         Method(String n, String t) {
             this(n, getOrCreateImport(t));
         }
@@ -549,16 +579,46 @@ public class SourceCode {
 			return this;
 		}
 		
-		public Method addCodeLine(String line) {
-			if (isAbstract)
-                throw new IllegalStateException("abstract method " + name 
-				    + " can not have body");
-			if (!line.endsWith(SEMICOLON))
-			    line = line + SEMICOLON;
-			codeLines.add(line);
-			return this;
+		public Method addArgument(String className, String argName){
+		    ClassName cn = getOrCreateImport(className);
+		    args.add(new Argument<ClassName, String>(cn, argName," "));
+		    return this;
 		}
 		
+        public void setTab(boolean inc) {
+            if (inc)
+                tabCount++;
+            else
+                tabCount--;
+            tab = "";
+            for (int i = 0; i < tabCount * TABSIZE; i++) {
+                tab += SPACE;
+            }
+        }
+
+        public Method addCodeLine(String line) {
+            if (isAbstract)
+                throw new IllegalStateException("abstract method " + name + " can not have body");
+            // This doesn't handle try{ ... catch(){ if{
+            if (line.endsWith("{") || line.endsWith("}")) {
+
+            }
+            if (!line.endsWith(SEMICOLON)
+                && !(line.isEmpty() || line.endsWith("{") || line.endsWith("}") || line.startsWith("if")))
+                line = line + SEMICOLON;
+            codeLines.add(tab + line);
+            return this;
+        }
+
+        /**
+         * if tabInc = true, the current line, and all following lines will be tabbed.
+         *  If false, a tab will be removed.
+         */
+        public Method addCodeLine(String line, boolean tabInc) {
+            setTab(tabInc);
+            return addCodeLine(line);
+        }
+		
 		public Method makeAbstract() {
 			if (codeLines.isEmpty())
 				isAbstract = true;
@@ -601,6 +661,75 @@ public class SourceCode {
 		}
 	}
 	
+	public class Constructor extends Element<Constructor> {
+	    private List<Argument<ClassName,String>> args = new ArrayList<Argument<ClassName,String>>();
+        private List<String> codeLines = new ArrayList<String>();
+        int tabCount = 0;
+        String tab = "";
+        
+	    public Constructor(String name) {
+	        super(name, null);
+	        makePublic();
+        }
+        
+        public Constructor addArgument(Argument<ClassName,String> arg) {
+            args.add(arg);
+            return this;
+        }
+
+        public Constructor addArgument(String className, String argName) {
+            ClassName cn = getOrCreateImport(className);
+            args.add(new Argument<ClassName, String>(cn, argName, " "));
+            return this;
+        }
+        
+        public Constructor addCodeLine(String line) {
+            // This doesn't handle try{ ... catch(){ if{
+            if (line.endsWith("{") || line.endsWith("}")) {
+
+            }
+            if (!line.endsWith(SEMICOLON)
+                && !(line.isEmpty() || line.endsWith("{") || line.endsWith("}") || line.startsWith("if")))
+                line = line + SEMICOLON;
+            codeLines.add(tab + line);
+            return this;
+        }
+        /**
+         *  if tabInc = true, the current line, and all following lines will be tabbed. If false, a tab will be removed.
+         */
+        public Constructor addCodeLine(String line, boolean tabInc) {
+            setTab(tabInc);
+            return addCodeLine(line);
+        }
+        
+        public void setTab(boolean inc) {
+            if (inc)
+                tabCount++;
+            else
+                tabCount--;
+            tab = "";
+            for (int i = 0; i < tabCount * TABSIZE; i++) {
+                tab += SPACE;
+            }
+        }
+        
+        @Override
+        public void write(PrintWriter out, int tab) {
+            out.println(BLANK);
+            super.write(out, tab);
+            out.print(name);
+            writeList(out, BLANK, args, ARGS_DELIMITER, true);
+
+            out.println(SPACE + BLOCK_DELIMITER.start);
+            for (String line : codeLines) {
+                tab(out, tab+1);
+                out.println(line);
+            }
+            tab(out, tab);
+            out.println(BLOCK_DELIMITER.end);
+        }
+	    
+	}
 	/**
 	 * Represents <code>import</code> statement.
 	 *
@@ -622,7 +751,7 @@ public class SourceCode {
 		    String pkg = name.getPackageName();
 		    if (pkg.length() == 0 || pkg.equals(getPackage().name))
 		        return;
-		    out.println("import "+ name.getFullName() + SEMICOLON);
+		    out.println("import "+ name.fullName + SEMICOLON);
 		}
 		
 		public boolean equals(Object other) {
@@ -746,8 +875,8 @@ public class SourceCode {
 					if (i == 0) {
 						out.println("/** ");
 						tab(out, tab);
-					} 
-					out.println(" *  " + l);
+					}
+                    out.println(" *  " + l);
 					i++;
 				}
 				tab(out, tab);
@@ -774,6 +903,11 @@ public class SourceCode {
 	    		arrayMarker = arrayMarker + "[]"; 
 	    		name = getComponentName(name);
 	    	}
+            int start = name.indexOf("<");
+            int stop = name.lastIndexOf(">");
+            if (start != -1 && stop != -1) {
+                name = name.substring(0, start) + name.substring(stop + 1); 
+            }
 	        this.fullName = name;
 	        int dot = fullName.lastIndexOf(DOT);
 	        simpleName = (dot == -1) ? fullName : fullName.substring(dot+1);
@@ -817,7 +951,7 @@ public class SourceCode {
 	     * Compares by fully-qualified name.
 	     */
 	    public int compareTo(ClassName other) {
-	        return fullName.compareTo(other.fullName);
+	        return getFullName().compareTo(other.getFullName());
 	    }
 	    
 	    public boolean isValidTypeName(String s) {
@@ -864,7 +998,7 @@ public class SourceCode {
 	    final char end;
 	    
         public Delimiter() {
-            this((char)0, (char)0);
+            this((char)' ', (char)' ');
         }
 
 	    public Delimiter(String pair) {
