@@ -18,11 +18,9 @@
  */
 package org.apache.openjpa.slice.jdbc;
 
-import java.lang.reflect.Constructor;
 import java.sql.Connection;
 import java.sql.SQLException;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.BitSet;
 import java.util.Collection;
 import java.util.HashMap;
@@ -52,15 +50,15 @@ import org.apache.openjpa.kernel.StoreQuery;
 import org.apache.openjpa.kernel.exps.ExpressionParser;
 import org.apache.openjpa.lib.rop.MergedResultObjectProvider;
 import org.apache.openjpa.lib.rop.ResultObjectProvider;
-import org.apache.openjpa.lib.util.ConcreteClassGenerator;
 import org.apache.openjpa.lib.util.Localizer;
 import org.apache.openjpa.meta.ClassMetaData;
 import org.apache.openjpa.meta.FieldMetaData;
+import org.apache.openjpa.slice.DistributedConfiguration;
 import org.apache.openjpa.slice.DistributedStoreManager;
-import org.apache.openjpa.slice.ProductDerivation;
 import org.apache.openjpa.slice.Slice;
 import org.apache.openjpa.slice.SliceImplHelper;
 import org.apache.openjpa.slice.SliceInfo;
+import org.apache.openjpa.slice.SlicePersistence;
 import org.apache.openjpa.slice.SliceThread;
 import org.apache.openjpa.util.InternalException;
 import org.apache.openjpa.util.StoreException;
@@ -80,22 +78,8 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
     private final List<SliceStoreManager> _slices;
     private JDBCStoreManager _master;
     private final DistributedJDBCConfiguration _conf;
-    private static final Localizer _loc =
-            Localizer.forPackage(DistributedJDBCStoreManager.class);
+    private static final Localizer _loc = Localizer.forPackage(DistributedJDBCStoreManager.class);
 
-    private static final Constructor<ClientConnection> clientConnectionImpl;
-    private static final Constructor<RefCountConnection> refCountConnectionImpl;
-    static {
-        try {
-            clientConnectionImpl = ConcreteClassGenerator.
-                getConcreteConstructor(ClientConnection.class, Connection.class);
-            refCountConnectionImpl = ConcreteClassGenerator.
-                getConcreteConstructor(RefCountConnection.class, JDBCStoreManager.class, Connection.class);
-        } catch (Exception e) {
-            throw new ExceptionInInitializerError(e);
-        }
-    }
-    
     /**
      * Constructs a set of child StoreManagers each connected to a physical
      * DataSource.
@@ -108,13 +92,14 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
         super();
         _conf = conf;
         _slices = new ArrayList<SliceStoreManager>();
-        List<String> sliceNames = conf.getActiveSliceNames();
-        for (String name : sliceNames) {
-            SliceStoreManager slice =
-                    new SliceStoreManager(conf.getSlice(name));
-            _slices.add(slice);
-            if (slice.getName().equals(_conf.getMaster().getName()))
-                _master = slice;
+        List<Slice> slices = conf.getSlices(Slice.Status.ACTIVE);
+        Slice masterSlice = conf.getMasterSlice();
+        for (Slice slice : slices) {
+            SliceStoreManager store = new SliceStoreManager(slice);
+            _slices.add(store);
+            if (slice == masterSlice) {
+                _master = store;
+            }
         }
     }
 
@@ -128,8 +113,7 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
     
     public SliceStoreManager addSlice(Slice slice) {
         SliceStoreManager result = new SliceStoreManager(slice);
-        result.setContext(getContext(),
-                (JDBCConfiguration)slice.getConfiguration());
+        result.setContext(getContext(), (JDBCConfiguration)slice.getConfiguration());
         _slices.add(result);
         return result;
     }
@@ -146,7 +130,7 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
         SliceInfo result = null;
         PersistenceCapable pc = sm.getPersistenceCapable();
         Object ctx = getContext();
-        if (SliceImplHelper.isReplicated(sm)) {
+        if (_conf.isReplicated(sm.getMetaData().getDescribedType())) {
             result = SliceImplHelper.getSlicesByPolicy(pc, _conf, ctx);
         } else {
             String origin = estimateSlice(sm, edata);
@@ -160,7 +144,7 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
     }
     
     private void assignSlice(OpenJPAStateManager sm, String hint) {
-        if (SliceImplHelper.isReplicated(sm)) {
+        if (_conf.isReplicated(sm.getMetaData().getDescribedType())) {
             SliceImplHelper.getSlicesByPolicy(sm, _conf, getContext())
                 .setInto(sm);
             return;
@@ -274,7 +258,7 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
         Map<String, StateManagerSet> subsets = bin(sms, null);
         Collection<StateManagerSet> remaining = 
             new ArrayList<StateManagerSet>(subsets.values());
-        ExecutorService threadPool = SliceThread.newPool(_slices.size());
+        ExecutorService threadPool = SliceThread.getPool();
         for (int i = 0; i < _slices.size(); i++) {
             SliceStoreManager slice = _slices.get(i);
             StateManagerSet subset = subsets.get(slice.getName());
@@ -350,10 +334,10 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
      * by the associated slice identifier of each StateManager.
      */
     private Map<String, StateManagerSet> bin(Collection sms, Object edata) {
-        Map<String, StateManagerSet> subsets =  
-            new HashMap<String, StateManagerSet>();
-        for (SliceStoreManager slice : _slices)
-            subsets.put(slice.getName(), new StateManagerSet());
+        Map<String, StateManagerSet> subsets =  new HashMap<String, StateManagerSet>();
+        for (SliceStoreManager slice : _slices) {
+            subsets.put(slice.getName(), new StateManagerSet(_conf));
+        }
         for (Object x : sms) {
             OpenJPAStateManager sm = (OpenJPAStateManager) x;
             String[] targets = findSliceNames(sm, edata).getSlices();
@@ -365,7 +349,7 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
     }
 
     public Object getClientConnection() {
-        return ConcreteClassGenerator.newInstance(clientConnectionImpl, getConnection());
+        return _master.getClientConnection();
     }
 
     public Seq getDataStoreIdSequence(ClassMetaData forClass) {
@@ -430,21 +414,34 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
         return _master.newDataStoreId(oidVal, meta);
     }
 
-    public FetchConfiguration newFetchConfiguration() {
-        return _master.newFetchConfiguration();
-    }
-
     /**
      * Construct a distributed query to be executed against all the slices.
      */
     public StoreQuery newQuery(String language) {
+    	if (QueryLanguages.LANG_SQL.equals(language)) {
+    		DistributedSQLStoreQuery ret = new DistributedSQLStoreQuery(this);
+            for (SliceStoreManager slice : _slices) {
+                ret.add(slice.newQuery(language));
+            }
+            return ret;
+    	}
         ExpressionParser parser = QueryLanguages.parserForLanguage(language);
+        if (parser == null) {
+    		throw new UnsupportedOperationException("Language [" + language + "] not supported");
+        } 
+
         DistributedStoreQuery ret = new DistributedStoreQuery(this, parser);
         for (SliceStoreManager slice : _slices) {
             ret.add(slice.newQuery(language));
         }
         return ret;
     }
+    
+    @Override
+    public FetchConfiguration newFetchConfiguration() {
+        return new TargetFetchConfiguration();
+    }
+
 
     /**
      * Sets the context for this receiver and all its underlying slices.
@@ -479,13 +476,13 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
         List<Connection> list = new ArrayList<Connection>();
         for (SliceStoreManager slice : _slices)
             list.add(slice.getConnection());
-        DistributedConnection con = DistributedConnection.newInstance(list);
-        return ConcreteClassGenerator.newInstance(refCountConnectionImpl, DistributedJDBCStoreManager.this, con);
+        DistributedConnection con = new DistributedConnection(list);
+        return new RefCountConnection(con);
     }
     
     /**
      * Gets the list of slices mentioned as  
-     * {@link ProductDerivation#HINT_TARGET hint} of the given
+     * {@link SlicePersistence#HINT_TARGET hint} of the given
      * {@link FetchConfiguration#getHint(String) fetch configuration}. 
      * 
      * @return all active slices if a) the hint is not specified or b) a null 
@@ -494,16 +491,20 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
     List<SliceStoreManager> getTargets(FetchConfiguration fetch) {
         if (fetch == null)
             return _slices;
-        Object hint = fetch.getHint(ProductDerivation.HINT_TARGET);
-        if (hint == null || !(hint instanceof String)) 
+        Object hint = fetch.getHint(SlicePersistence.HINT_TARGET);
+        if (hint == null || !(hint instanceof String || hint instanceof String[])) 
             return _slices;
-        List<String> targetNames = Arrays.asList(hint.toString().split("\\,"));
+        String[] targetNames = hint instanceof String 
+                ? new String[]{hint.toString()} : (String[])hint;
         List<SliceStoreManager> targets = new ArrayList<SliceStoreManager>();
         for (SliceStoreManager slice : _slices) {
-           if (targetNames.contains(slice.getName()))
-              targets.add(slice);
-           }
-          if (targets.isEmpty())
+            for (String name : targetNames) {
+                if (slice.getName().equals(name)) {
+                    targets.add(slice);
+                }
+            }
+        }
+        if (targets.isEmpty())
             return _slices;
         return targets;
     }
@@ -528,11 +529,15 @@ class DistributedJDBCStoreManager extends JDBCStoreManager
      *  
      */
     private static class StateManagerSet extends HashSet<OpenJPAStateManager> {
+        private final DistributedConfiguration conf;
         List<OpenJPAStateManager> replicated;
         
+        StateManagerSet(DistributedConfiguration conf) {
+            this.conf = conf;
+        }
         @Override
         public boolean add(OpenJPAStateManager sm) {
-            boolean isReplicated = sm.getMetaData().isReplicated();
+            boolean isReplicated =  conf.isReplicated(sm.getMetaData().getDescribedType());
             if (isReplicated) {
                 if (replicated == null)
                     replicated = new ArrayList<OpenJPAStateManager>();
